<?php

namespace App\Http\Controllers\Bill;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Voucher;
use App\Models\Bill\Bill;
use App\Models\Bill\BillDesc;
use App\Models\Booking\BookGr;
use App\Models\Booking\GrDesc;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\FleetOwner\LorryHire;
use App\Models\GoodsMove\Challan;
use App\Models\Inventory\Purchase;
use App\Models\Master\IssueBooks;
use App\Models\Master\Party;
use App\Models\Pod\Pod;
use App\Models\Pod\PodDesc;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class BillController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Bill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $bill = Bill::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'acc_party_branch:id,branch_name',
            'income:id,account_name',

            'bill_desc',
            'bill_desc.vehicle:id,vehicle_number,vehicle_type',
            'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,source_id,destination_id,consignor_name,consignee_name,source_name,destination_name',
            'bill_desc.bookGr.consignor:id,party_name',
            'bill_desc.bookGr.consignee:id,party_name',
            'bill_desc.bookGr.source:id,station_name',
            'bill_desc.bookGr.destination:id,station_name',
            'bill_desc.pod:id,pod_number',
            
            'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',

            'bill_desc.loading_slip'
        ])->where('fin_year',$request->current_year)->get();


        $bill->bill_desc->transform(function ($desc) {
            $data = [
                'bill_desc_id' => $desc->id,
                'bill_id' => $desc->bill_id,
                'total_without_gst' => $desc->total_without_gst,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
                'rounded_off' => $desc->rounded_off,
                'grand_total' => $desc->grand_total,
            ];

            
        
            if ($desc->selection_type == 'GR') {
                $data += [
                    'gr_id' => $desc->gr_id,
                    'gr_number' => $desc->gr_number,
                    'gr_date' => $desc->bookGr->gr_date,
                    'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                    'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                    'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                    'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                    'total_pkg' => $desc->total_pkg,
                    'total_material_weight' => $desc->total_material_weight,
                    'total_chg_qty' => $desc->total_chg_qty,
                    'total_desc_value' => $desc->total_desc_value,
                    'total_invoice_value' => $desc->total_invoice_value,
                    'freight' => $desc->freight,
                    'gr_chg' => $desc->gr_chg,
                    'disc_percent' => $desc->disc_percent,
                    'disc_amount' => $desc->disc_amount,
                    'green_tax' => $desc->green_tax,
                    'detention' => $desc->detention,
                    'kanta' => $desc->kanta,
                    'unload_rate' => $desc->unload_rate,
                    'unload_amt' => $desc->unload_amt,
                    'toll_amt' => $desc->toll_amt,
                    'diversion_rate' => $desc->diversion_rate,
                    'diversion_amt' => $desc->diversion_amt,

                    'other1' => $desc->other1,
                    'other2' => $desc->other2,
                    'other3' => $desc->other3,
                    'other4' => $desc->other4,
                    'gr_desc' => json_decode($desc->gr_desc),
                    'pod_id' => $desc->pod ? $desc->pod_id : null,
                    'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                    'load_detention' => $desc->load_detention,
                    'unload_detention' => $desc->unload_detention,
                    'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                    'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                ];
            }
        
            if ($desc->selection_type == 'Loading Slip') {
                $data += [
                    'ls_id' => $desc->ls_id,
                    'ls_number' => $desc->ls_number,
                    'ls_date' => $desc->loading_slip->ls_date,
                    'branch' => $desc->loading_slip->branch,
                    'vehicle' => $desc->loading_slip->vehicle,
                    'source' => $desc->loading_slip->source,
                    'destination' => $desc->loading_slip->destination,
                    'freight' => $desc->loading_slip->freight,
                    'munshiana' => $desc->loading_slip->munshiana,
                    'dalla' => $desc->loading_slip->dalla,
                    'advance_amt' => $desc->loading_slip->advance_amt,
                    'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                    'diesel_amt' => $desc->loading_slip->diesel_amt,
                    'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                    'labour' => $desc->loading_slip->labour,
                    'detention' => $desc->loading_slip->detention,
                    'unload_detention' => $desc->loading_slip->unload_detention,
                    'balance_receivable' => $desc->loading_slip->balance_receivable,
                ];
            }

            if ($desc->selection_type == 'Supplementary') {
                $data += [
                    'supplementary_desc' => json_decode($desc->supplementary_desc),
                    'vehicle' => $desc->vehicle
                ];
            }
        
            return $data;
        });

        return response()->json(['bill'=>$bill]);
    }

    public function show(Request $request, $pk){
        //FETCH AND SEND ALL
        $bill = Bill::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'acc_party_branch:id,branch_name',
            'income:id,account_name',

            'bill_desc',
            'bill_desc.vehicle:id,vehicle_number,vehicle_type',
            'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,vehicle_number,consignee_id,source_id,destination_id,consignor_name,consignee_name,source_name,destination_name',
            'bill_desc.bookGr.consignor:id,party_name',
            'bill_desc.bookGr.consignee:id,party_name',
            'bill_desc.bookGr.source:id,station_name',
            'bill_desc.bookGr.destination:id,station_name',
            'bill_desc.pod:id,pod_number',
            
            'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',

            'bill_desc.loading_slip'
        ])->find($pk);


        $bill->bill_desc->transform(function ($desc) {
            $data = [
                'bill_desc_id' => $desc->id,
                'bill_id' => $desc->bill_id,
                'total_without_gst' => $desc->total_without_gst,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
                'rounded_off' => $desc->rounded_off,
                'grand_total' => $desc->grand_total,
            ];

            
        
            if ($desc->selection_type == 'GR') {
                $data += [
                    'gr_id' => $desc->gr_id,
                    'gr_number' => $desc->gr_number,
                    'vehicle'=>$desc->bookGr->vehicle_number,
                    'gr_date' => $desc->bookGr->gr_date,
                    'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                    'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                    'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                    'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                    'total_pkg' => $desc->total_pkg,
                    'total_material_weight' => $desc->total_material_weight,
                    'total_chg_qty' => $desc->total_chg_qty,
                    'total_desc_value' => $desc->total_desc_value,
                    'total_invoice_value' => $desc->total_invoice_value,
                    'freight' => $desc->freight,
                    'gr_chg' => $desc->gr_chg,
                    'other1' => $desc->other1,
                    'other2' => $desc->other2,
                    'other3' => $desc->other3,
                    'other4' => $desc->other4,
                    'disc_percent' => $desc->disc_percent,
                    'disc_amount' => $desc->disc_amount,
                    'green_tax' => $desc->green_tax,
                    'detention' => $desc->detention,
                    'kanta' => $desc->kanta,
                    'unload_rate' => $desc->unload_rate,
                    'unload_amt' => $desc->unload_amt,
                    'toll_amt' => $desc->toll_amt,
                    'diversion_rate' => $desc->diversion_rate,
                    'diversion_amt' => $desc->diversion_amt,

                    'gr_desc' => json_decode($desc->gr_desc),
                    'pod_id' => $desc->pod ? $desc->pod_id : null,
                    'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                    'load_detention' => $desc->load_detention,
                    'unload_detention' => $desc->unload_detention,
                    'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                    'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                ];
            }
        
            if ($desc->selection_type == 'Loading Slip') {
                $data += [
                    'ls_id' => $desc->ls_id,
                    'ls_number' => $desc->ls_number,
                    'ls_date' => $desc->loading_slip->ls_date,
                    'branch' => $desc->loading_slip->branch,
                    'vehicle' => $desc->loading_slip->vehicle,
                    'source' => $desc->loading_slip->source,
                    'destination' => $desc->loading_slip->destination,
                    'freight' => $desc->loading_slip->freight,
                    'munshiana' => $desc->loading_slip->munshiana,
                    'dalla' => $desc->loading_slip->dalla,
                    'advance_amt' => $desc->loading_slip->advance_amt,
                    'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                    'diesel_amt' => $desc->loading_slip->diesel_amt,
                    'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                    'labour' => $desc->loading_slip->labour,
                    'detention' => $desc->loading_slip->detention,
                    'unload_detention' => $desc->loading_slip->unload_detention,
                    'balance_receivable' => $desc->loading_slip->balance_receivable,
                ];
            }

            if ($desc->selection_type == 'Supplementary') {
                $data += [
                    'supplementary_desc' => json_decode($desc->supplementary_desc),
                    'vehicle' => $desc->vehicle
                ];
            }
        
            return $data;
        });

        return response()->json(['bill'=>$bill]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'Bill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'bill_number' => 'required|unique:bills,bill_number',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'bill_number.required' => 'Required',
            'bill_number.unique' => 'Already Exist !',
            'acc_party.required' => 'Required'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_bill_number($request->bill_number);
            
            // SAVING A NEW Bill
            $bill = new Bill();
            $bill->branch_id = $request->branch['id'];
            $bill->bill_type = $request->bill_type;
            $bill->fin_yr_wise = $request->fin_yr_wise;
            $bill->prefix = $request->prefix;
            $bill->max_number = $max_number;
            $bill->bill_number = $request->bill_number;
            $bill->acc_party_id = $request->acc_party['id'];
            $bill->bill_date = $request->bill_date;
            $bill->submit_date = $request->submit_date;
            $bill->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;
            $bill->billing_address = $request->billing_address;
            $bill->shipping_address = $request->shipping_address;
            $bill->gst_number = $request->gst_number;
            $bill->income = $request->income['id'];
            $bill->remarks = $request->remarks;
            $bill->bill_attachment = $request->bill_attachment;

            $bill->selection_type = $request->selection_type;
            $bill->pay_mode = $request->pay_mode;
            $bill->included_charges = $request->included_charges;
            $bill->from_date = $request->from_date;
            $bill->to_date = $request->to_date; 
            $bill->is_exempt = $request->is_exempt; 
            $bill->is_rcm = $request->is_rcm;
            
            $bill->igst_percent = $request->igst_percent;
            $bill->cgst_percent = $request->cgst_percent;
            $bill->sgst_percent = $request->sgst_percent;

            $bill->other_plus_label = $request->other_plus_label;
            $bill->other_plus = $request->other_plus;
            $bill->other_minus_label = $request->other_minus_label;
            $bill->other_minus = $request->other_minus;

            $bill->total_other1 = $request->total_other1;
            $bill->total_other2 = $request->total_other2;
            $bill->total_other3 = $request->total_other3;
            $bill->total_other4 = $request->total_other4;
            
            $bill->total_without_gst = $request->total_without_gst;
            $bill->total_cgst_amount = $request->total_cgst_amount;
            $bill->total_sgst_amount = $request->total_sgst_amount;
            $bill->total_igst_amount = $request->total_igst_amount;
            
            $bill->total_after_gst_chgs = $request->total_after_gst_chgs;
            $bill->total_without_rounding = $request->total_without_rounding;
            $bill->rounded_off = $request->rounded_off;
            $bill->grand_total_amount = $request->grand_total_amount;

            if($request->selection_type=='Supplementary'){
                $bill->supplementary_headers = json_encode($request->supplementary_headers);
            }

            $bill->fin_year = $request->current_year;

            $bill->created_by = $request->user->id;

            // Save the model to the database
            $bill->save();

             

            // SAVING BILL DESC
            $bill_desc = $request->bill_desc;

                foreach ($bill_desc as $bd) {
                    $bill_desc = new BillDesc();
                    $bill_desc->bill_id = $bill->id;
                    $bill_desc->bill_number = $bill->bill_number;
                    $bill_desc->selection_type = $request->selection_type;
                    $bill_desc->included_charges = $request->included_charges;

                    $bill_desc->total_without_gst = $bd['total_without_gst'];
                    $bill_desc->igst_amount = $bd['igst_amount'];
                    $bill_desc->cgst_amount = $bd['cgst_amount'];
                    $bill_desc->sgst_amount = $bd['sgst_amount'];
                    $bill_desc->total_without_rounding = $bd['total_without_rounding'];
                    $bill_desc->rounded_off = $bd['rounded_off'];
                    $bill_desc->grand_total = $bd['grand_total'];
                    $bill_desc->fin_year = $request->current_year;
                    
                    if($request->selection_type == 'GR'){
                        if( !$bd['gr_id'] )
                         continue;

                        $this->change_bill_status('GR', true, $bd['gr_id'], $bill_desc->included_charges);
                        
    
                        $bill_desc->gr_id = $bd['gr_id'];
                        $bill_desc->gr_number = $bd['gr_number'];
                        $bill_desc->total_pkg = $bd['total_pkg'];
                        $bill_desc->total_material_weight = $bd['total_material_weight'];
                        $bill_desc->total_chg_qty = $bd['total_chg_qty'];
                        $bill_desc->total_invc_value = $bd['total_invoice_value'];
                        $bill_desc->total_desc_value = $bd['total_desc_value'];

                        $bill_desc->freight = $bd['freight'];
                        $bill_desc->gr_chg = $bd['gr_chg'];
                        $bill_desc->disc_percent = $bd['disc_percent'];
                        $bill_desc->disc_amount = $bd['disc_amount'];
                        $bill_desc->green_tax = $bd['green_tax'];
                        $bill_desc->detention = $bd['detention'];
                        $bill_desc->kanta = $bd['kanta'];

                        $bill_desc->unload_rate = $bd['unload_rate'];
                        $bill_desc->unload_amt = $bd['unload_amt'];
                        $bill_desc->toll_amt = $bd['toll_amt'];
                        $bill_desc->diversion_rate = $bd['diversion_rate'];
                        $bill_desc->diversion_amt = $bd['diversion_amt'];


                        $bill_desc->other1 = $bd['other1'];
                        $bill_desc->other2 = $bd['other2'];
                        $bill_desc->other3 = $bd['other3'];
                        $bill_desc->other4 = $bd['other4'];

                        $bill_desc->payment_dynamic_fields = json_encode($bd['payment_dynamic_fields']);
                        $bill_desc->gr_desc = json_encode($bd['gr_desc']);
                          
                        if($bd['pod_id']){
                            $bill_desc->pod_id = $bd['pod_id'];
                            $bill_desc->pod_payment_dynamic_fields = json_encode($bd['pod_payment_dynamic_fields'])??[];
                            $bill_desc->load_detention = $bd['load_detention'];
                            $bill_desc->unload_detention = $bd['unload_detention'];
                        }
                        
                        $this->handle_update_in_gr_pod($request,$bd);
                        
                    }
                    
                    if($request->selection_type == 'Loading Slip'){
                        if( !$bd['ls_id'] )
                        continue;

                        $this->change_bill_status('Loading Slip',true,$bd['ls_id'],'NONE');

                        $bill_desc->ls_id = $bd['ls_id'];
                        $bill_desc->ls_number = $bd['ls_number'];
                    }

                    if($request->selection_type == 'Supplementary'){
                        if( !$bd['supplementary_desc']['other1'] || !$bd['supplementary_desc']['freight'] )
                        continue; 

                        $bill_desc->supplementary_desc = json_encode($bd['supplementary_desc']);
                        $bill_desc->vehicle_id = $bd['vehicle'] ? $bd['vehicle']['id'] : null;
                    }
                    
                    $bill_desc->save();
    
                }

            // BILL DESC SAVED

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $bill);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($bill->id);
            $this->createEditLog('created', 'bills', $bill->id, $bill->bill_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$bill->id, 'bill_number'=>$bill->bill_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Bill');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $bill = Bill::find($pk);

        $credit_note_check = $this->check_in_credit_note_desc('cn_number','cn_number','bill_number',$bill->bill_number);
        $cover_letter_check = $this->check_in_cover_letter_desc('cl_number','cl_number','bill_number',$bill->bill_number);
                
        if($credit_note_check['status'] || $cover_letter_check['status']){
            $status = $credit_note_check['status'] ? $credit_note_check : $cover_letter_check;
            return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($bill->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'bill_number' => 'required',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'bill_number.required' => 'Required',
            'acc_party.required' => 'Required'
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            if($bill->einvoice_status=='Success'){
                return response()->json(['error'=>'E-Invoice Generated, Cannot Edit'],500);
            }
            
            // SAVING A NEW BILL
            $bill->acc_party_id = $request->acc_party['id'];
            $bill->bill_date = $request->bill_date;
            $bill->submit_date = $request->submit_date;
            $bill->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;
            $bill->billing_address = $request->billing_address;
            $bill->shipping_address = $request->shipping_address;
            $bill->gst_number = $request->gst_number;
            $bill->income = $request->income['id'];
            $bill->remarks = $request->remarks;
            $bill->bill_attachment = $request->bill_attachment;

            $bill->selection_type = $request->selection_type;
            $bill->pay_mode = $request->pay_mode;
            $bill->included_charges = $request->included_charges;
            $bill->from_date = $request->from_date;
            $bill->to_date = $request->to_date; 
            $bill->is_exempt = $request->is_exempt;
            $bill->is_rcm = $request->is_rcm;
            
            $bill->igst_percent = $request->igst_percent;
            $bill->cgst_percent = $request->cgst_percent;
            $bill->sgst_percent = $request->sgst_percent;

            $bill->other_plus_label = $request->other_plus_label;
            $bill->other_plus = $request->other_plus;
            $bill->other_minus_label = $request->other_minus_label;
            $bill->other_minus = $request->other_minus;

            $bill->total_other1 = $request->total_other1;
            $bill->total_other2 = $request->total_other2;
            $bill->total_other3 = $request->total_other3;
            $bill->total_other4 = $request->total_other4;
            
            $bill->total_without_gst = $request->total_without_gst;
            $bill->total_cgst_amount = $request->total_cgst_amount;
            $bill->total_sgst_amount = $request->total_sgst_amount;
            $bill->total_igst_amount = $request->total_igst_amount;
            $bill->total_after_gst_chgs = $request->total_after_gst_chgs;
            $bill->total_without_rounding = $request->total_without_rounding;
            $bill->rounded_off = $request->rounded_off;
            $bill->grand_total_amount = $request->grand_total_amount;

            if($request->selection_type=='Supplementary'){
                $bill->supplementary_headers = json_encode($request->supplementary_headers);
            }

            $bill->updated_by = $request->user->id;
            $bill->save();
            // NEW BILL SAVED
            
            // SAVING Bill DESC
            $newly_added_bill_desc = $request->newly_added_bill_desc;
            foreach ($newly_added_bill_desc as $nabd) {    
                $bill_desc = new BillDesc();
                $bill_desc->bill_id = $bill->id;
                $bill_desc->bill_number = $bill->bill_number;
                $bill_desc->selection_type = $request->selection_type;
                $bill_desc->included_charges = $request->included_charges;
                $bill_desc->total_without_gst = $nabd['total_without_gst'];
                $bill_desc->igst_amount = $nabd['igst_amount'];
                $bill_desc->cgst_amount = $nabd['cgst_amount'];
                $bill_desc->sgst_amount = $nabd['sgst_amount'];
                $bill_desc->total_without_rounding = $nabd['total_without_rounding'];
                $bill_desc->rounded_off = $nabd['rounded_off'];
                $bill_desc->grand_total = $nabd['grand_total'];
                $bill_desc->fin_year = $request->current_year;
                
                if($request->selection_type == 'GR'){
                    if( !$nabd['gr_id'] )
                    continue;


                    $this->change_bill_status('GR', true, $nabd['gr_id'], $bill_desc->included_charges);
                    

                    $bill_desc->gr_id = $nabd['gr_id'];
                    $bill_desc->gr_number = $nabd['gr_number'];
                    $bill_desc->total_pkg = $nabd['total_pkg'];
                    $bill_desc->total_material_weight = $nabd['total_material_weight'];
                    $bill_desc->total_chg_qty = $nabd['total_chg_qty'];
                    $bill_desc->total_invc_value = $nabd['total_invoice_value'];
                    $bill_desc->total_desc_value = $nabd['total_desc_value'];
                    $bill_desc->freight = $nabd['freight'];
                    $bill_desc->gr_chg = $nabd['gr_chg'];
                    $bill_desc->other1 = $nabd['other1'];
                    $bill_desc->other2 = $nabd['other2'];
                    $bill_desc->other3 = $nabd['other3'];
                    $bill_desc->other4 = $nabd['other4'];
                    $bill_desc->disc_percent = $nabd['disc_percent'];
                        $bill_desc->disc_amount = $nabd['disc_amount'];
                        $bill_desc->green_tax = $nabd['green_tax'];
                        $bill_desc->detention = $nabd['detention'];
                        $bill_desc->kanta = $nabd['kanta'];

                    $bill_desc->unload_rate = $nabd['unload_rate'];
                    $bill_desc->unload_amt = $nabd['unload_amt'];
                    $bill_desc->toll_amt = $nabd['toll_amt'];
                    $bill_desc->diversion_rate = $nabd['diversion_rate'];
                    $bill_desc->diversion_amt = $nabd['diversion_amt'];
                    

                    $bill_desc->payment_dynamic_fields = json_encode($nabd['payment_dynamic_fields'])??[];
                    $bill_desc->gr_desc = json_encode($nabd['gr_desc']);
                    if($nabd['pod_id']){
                        $bill_desc->pod_id = $nabd['pod_id'];
                        $bill_desc->pod_payment_dynamic_fields = json_encode($nabd['pod_payment_dynamic_fields'])??[];
                        $bill_desc->load_detention = $nabd['load_detention'];
                        $bill_desc->unload_detention = $nabd['unload_detention'];
                    }
                   
                    $this->handle_update_in_gr_pod($request,$nabd);
                }

                if($request->selection_type == 'Loading Slip'){
                    if( !$nabd['ls_id'] )
                    continue;

                    $this->change_bill_status('Loading Slip',true,$nabd['ls_id'],'NONE');

                    $bill_desc->ls_id = $nabd['ls_id'];
                    $bill_desc->ls_number = $nabd['ls_number'];
                }

                if($request->selection_type == 'Supplementary'){
                    if( !$nabd['supplementary_desc']['other1'] || !$nabd['supplementary_desc']['freight'] )
                    continue; 

                    $bill_desc->supplementary_desc = json_encode($nabd['supplementary_desc']);
                    $bill_desc->vehicle_id = $nabd['vehicle'] ? $nabd['vehicle']['id'] : null;
                }
                
                $bill_desc->save();
                
            }
            // Bill DESC SAVED

            // Updated DESC
            $updated_bill_desc = $request->updated_bill_desc;
            foreach ($updated_bill_desc as $ubd) {
                
                $bill_desc = BillDesc::find($ubd['bill_desc_id']);
                  
                

                $bill_desc->total_without_gst = $ubd['total_without_gst'];
                $bill_desc->igst_amount = $ubd['igst_amount'];
                $bill_desc->cgst_amount = $ubd['cgst_amount'];
                $bill_desc->sgst_amount = $ubd['sgst_amount'];
                $bill_desc->total_without_rounding = $ubd['total_without_rounding'];
                $bill_desc->rounded_off = $ubd['rounded_off'];
                $bill_desc->grand_total = $ubd['grand_total'];
                
                if($bill_desc->selection_type == 'GR'){
                    if( !$ubd['gr_id'] )
                    continue;
                    $bill_desc->total_pkg = $ubd['total_pkg'];
                    $bill_desc->total_material_weight = $ubd['total_material_weight'];
                    $bill_desc->total_chg_qty = $ubd['total_chg_qty'];
                    $bill_desc->total_invc_value = $ubd['total_invoice_value'];


                    $bill_desc->total_desc_value = $ubd['total_desc_value'];
                    $bill_desc->freight = $ubd['freight'];
                    $bill_desc->gr_chg = $ubd['gr_chg'];
                    $bill_desc->other1 = $ubd['other1'];
                    $bill_desc->other2 = $ubd['other2'];
                    $bill_desc->other3 = $ubd['other3'];
                    $bill_desc->other4 = $ubd['other4'];
                    $bill_desc->disc_percent = $ubd['disc_percent'];
                        $bill_desc->disc_amount = $ubd['disc_amount'];
                        $bill_desc->green_tax = $ubd['green_tax'];
                        $bill_desc->detention = $ubd['detention'];
                        $bill_desc->kanta = $ubd['kanta'];
                    
                    $bill_desc->unload_rate = $ubd['unload_rate'];
                    $bill_desc->unload_amt = $ubd['unload_amt'];
                    $bill_desc->toll_amt = $ubd['toll_amt'];
                    $bill_desc->diversion_rate = $ubd['diversion_rate'];
                    $bill_desc->diversion_amt = $ubd['diversion_amt'];
                    

                    $bill_desc->payment_dynamic_fields = json_encode($ubd['payment_dynamic_fields'])??[];
                    $bill_desc->gr_desc = json_encode($ubd['gr_desc']);

                    if($ubd['pod_id']){
                        $bill_desc->pod_payment_dynamic_fields = json_encode($ubd['pod_payment_dynamic_fields'])??[];
                        $bill_desc->load_detention = $ubd['load_detention'];
                        $bill_desc->unload_detention = $ubd['unload_detention'];
                    }
                    
                    
                    $this->handle_update_in_gr_pod($request,$ubd);
                }

                if($request->selection_type == 'Supplementary'){
                    if( !$ubd['supplementary_desc']['other1'] || !$ubd['supplementary_desc']['freight'] )
                    continue; 

                    $bill_desc->supplementary_desc = json_encode($ubd['supplementary_desc']);
                    $bill_desc->vehicle_id = $ubd['vehicle'] ? $ubd['vehicle']['id'] : null;
                }

                $bill_desc->save();
            }
            // Bill DESC SAVED


            // DELETING EXISTING POD DESC
            $deleted_bill_desc = $request->deleted_bill_desc;
            if( !empty($deleted_bill_desc) ){
                foreach ($deleted_bill_desc as $dbd) {
                    $d_bil_d = BillDesc::find($dbd);
                    $d_bil_d->delete();

                    if($request->selection_type=='GR'){
                        $this->change_bill_status('GR', false, $d_bil_d['gr_id'], $d_bil_d->included_charges);

                        

                        if($d_bil_d->pod_id){
                            $pod_number = Pod::select('pod_number')->find($d_bil_d->pod_id);
                            if($pod_number){
                                $t_data3['t_against'] = 'POD';
                                $t_data3['t_against_no'] = $pod_number['pod_number'];
                                $t_data3['fin_year'] = $request->current_year;
                                $this->set_transaction_status($t_data3, true);
                            }
                        }
                        else{
                            $t_data2['t_against'] = 'GR';
                            $t_data2['t_against_no'] = $d_bil_d->gr_number;
                            $t_data2['fin_year'] = $request->current_year;
                            $this->set_transaction_status($t_data2, true);
                        }

                    }

                    if($request->selection_type == 'Loading Slip'){
                        $this->change_bill_status('Loading Slip',false,$d_bil_d['ls_id'],'NONE');

                        $t_data2['t_against'] = 'Loading Slip';
                        $t_data2['t_against_no'] = $d_bil_d->ls_number;
                        $t_data2['fin_year'] = $request->current_year;
                        $this->set_transaction_status($t_data2, true);

                    }
                }      
            }
            //DELETED EXISTING POD DESC

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $bill);
            // TRANSACTION RECORDED IN ACCOUNTS
            
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($bill->id);
            $this->createEditLog('updated', 'bills', $bill->id, $bill->bill_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'Bill');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
        
        $bill = Bill::find($pk);

        if($bill->einvoice_status=='Success'){
            return response()->json(['error'=>'E-Invoice Generated, Cannot Delete'],500);
        }

        $credit_note_check = $this->check_in_credit_note_desc('cn_number','cn_number','bill_number',$bill->bill_number);
        $cover_letter_check = $this->check_in_cover_letter_desc('cl_number','cl_number','bill_number',$bill->bill_number);
                
        if($credit_note_check['status'] || $cover_letter_check['status']){
            $status = $credit_note_check['status'] ? $credit_note_check : $cover_letter_check;
            return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($bill->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // LOOP THROUGH BILL DESC, CHECK selection_type, If 'GR' send gr_number  , If 'Loading Slip' send ls_number
        $bill_desc_ids = BillDesc::select('id')->where('bill_id', $bill->id)->where('fin_year',$request->current_year)->pluck('id')->toArray();
        

        // DELETING CHALLAN DESCRIPTIONS
        foreach ($bill_desc_ids as $pdi) {
            // MAKE GR TRANSACTION INACTIVE
            $bill_desc = BillDesc::find($pdi);

            if($bill_desc->selection_type == 'GR'){
                $this->change_bill_status('GR', false, $bill_desc['gr_id'], $bill_desc->included_charges);

            
                if($bill_desc->pod_id){
                    $pod_number = Pod::select('pod_number')->find($bill_desc->pod_id);
                    if($pod_number){
                        $t_data3['t_against'] = 'POD';
                        $t_data3['t_against_no'] = $pod_number['pod_number'];
                        $t_data3['fin_year'] = $request->current_year;
                        $this->set_transaction_status($t_data3, true);
                    }
                }
                else{
                    $t_data2['t_against'] = 'GR';
                    $t_data2['t_against_no'] = $bill_desc->gr_number;
                    $t_data2['fin_year'] = $request->current_year;
                    $this->set_transaction_status($t_data2, true);
                }
            }
            else if($bill_desc->selection_type == 'Loading Slip'){
                $this->change_bill_status('Loading Slip',false,$bill_desc['ls_id'],'NONE');

                $t_data2['t_against'] = 'Loading Slip';
                $t_data2['t_against_no'] = $bill_desc->ls_number;
                $t_data2['fin_year'] = $request->current_year;
                $this->set_transaction_status($t_data2, true);
            }
            
        }

 
        // DELETING BILL DESCRIPTIONS
        $bill_desc = BillDesc::where('bill_id',$pk)->where('fin_year',$request->current_year)->delete();

        //FIND AND DELETE THE BILL
        $bill->delete();


        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'BILL';
        $t_data['t_against_id'] = $bill->id;
        $t_data['t_against_no'] = $bill->bill_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $this->createEditLog('deleted', 'bills', $bill->id, $bill->bill_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = Bill::select('id', 'bill_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }
        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $query->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }


        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('bill_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $bill = $query->get();
        return response(['bills' => $bill, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Bill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Bill::select('id', 'bill_number', 'acc_party_id', 'branch_id','bill_date')
                ->with(['branch:id,branch_name,branch_code', 'acc_party:id,account_name'])
                ->where('fin_year',$request->current_year) 
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Bill::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $billQuery = Bill::select('id', 'bill_number')->where('fin_year',$request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $billQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $billQuery->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }
    
        $bills = $billQuery->orderBy('id', 'desc')->take(25)->get();
    
        return response()->json(['bills' => $bills]);
    }
    


    public function get_next_bill_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;
        $fin_yr_wise = $request->bill_number_fin_yr_wise;

        $year_stamp = $this->get_short_name2($request->current_year);



        if($type=='SYSTEM'){
            $max = Bill::where('bill_type', $type)->where('branch_id', $branch_id)->where('fin_yr_wise', $fin_yr_wise)->max('max_number');
            
            if($fin_yr_wise){
                $bill_number = $branch_code.'-'.$year_stamp.'-'.sprintf('%04d', $max+1);
            }
            else{
                $bill_number = $branch_code.'-'.sprintf('%04d', $max+1);
            }
            

            return response()->json(['bill_number' => strtoupper($bill_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = Bill::where('prefix',$prefix)->where('bill_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name,account_id','party.account:id,account_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Bill')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $bill_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);

                    $acc_party = $check->party? $check->party->account : null;
                    return response()->json(['bill_number' => strtoupper($bill_number), 'max_number' => $max+1, 'party' => $acc_party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name,account_id','party.account:id,account_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Bill')->where('issue_from', '>=', $max+1)->first();

            if( $check ){
                $bill_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;

                $acc_party = $check->party? $check->party->account : null;

                return response()->json(['bill_number' => strtoupper($bill_number), 'max_number' => $check->issue_from, 'party' => $acc_party]);
            }
            else{
                return response()->json(['errors' => ['bill_number' => ['No Valid Bill Number']]],422);
            }
        }
    }

    public function fetch_gr_and_pod_details_for_bill(Request $request){
        $type = $request->type;
        $acc_party_id = $request->acc_party_id;
        $acc_party_branch_id = $request->acc_party_branch_id;
        $pay_mode = $request->pay_mode; 
        $included_charges = $request->included_charges;
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $destination_id = $request->destination? $request->destination['id'] : null;
        $vehicle_id = $request->vehicle? $request->vehicle['id'] : null;
 
 
        $pod_required = !$request->bill_without_pod;
 
        if($type == 'GR'){
            
             $grs = BookGr::with([
                 'branch:id,branch_name,branch_code',
                 'vehicle:id,vehicle_number,vehicle_type',
                 'consignor:id,party_name',
                 'consignee:id,party_name',
                 'source:id,station_name',
                 'destination:id,station_name',
                 'acc_party:id,account_name',
                 'gr_desc:id,gr_id,material_id,pkg,material_weight,km,chg_qty,chg_unit,rate,total,invoice_number',
                 'gr_desc.material:id,material_name',
                 'pod_desc.pod:id,pod_number,freight,gr_chg,disc_percent,disc_amount,pod_payment_dynamic_fields,load_detention,unload_detention,igst_amount,cgst_amount,sgst_amount,igst_percent,cgst_percent,sgst_percent,disc_amount,disc_percent,green_tax,detention,kanta,unload_rate,unload_amt,toll_amt,diversion_rate,diversion_amt,pod_payment_dynamic_total,total_without_gst,total_without_rounding,rounded_off,grand_total',
             ])
             ->where('is_cancelled',false)
             ->where('acc_party_id', $acc_party_id)
             ->when($acc_party_branch_id, function ($query, $acc_party_branch_id) {
                 // Add the condition only if $acc_party_branch_id is not empty
                 return $query->where('acc_party_branch_id', $acc_party_branch_id);
             })
             ->where('gr_date', '>=', $from_date)
             ->where('gr_date', '<=', $to_date)
             ->where('pay_mode', $pay_mode)
             ->when($included_charges == 'Freight' || $included_charges == 'All', function ($query) {
                 $query->where('freight_bill', 0);
             })
             ->when($included_charges == 'Way Expenses' || $included_charges == 'All', function ($query) {
                 $query->where('way_exp_bill', 0);
             })
             ->when($destination_id, function ($query, $destination_id) {
                 return $query->where('destination_id', $destination_id);
             })
             ->when($vehicle_id, function ($query, $vehicle_id) {
                return $query->where('vehicle_id', $vehicle_id);
            })
             ->whereDoesntHave('voucher_desc', function ($query) {
                 $query->where('against', 'GR');
                 // Add more conditions if needed for 'against_number'
             });
 
             
             
             if ($included_charges == 'Way Expenses' || $pod_required == true) {
                 // Add condition to select only 'GRs' with 'pod_desc'
                 $grs = $grs->whereHas('pod_desc');
             }            
 
             // $grs = $grs->where(function ($query) use ($included_charges) {
 
             //     $query->whereDoesntHave('bill_desc');
 
             //     if ($included_charges == 'Freight') {
             //         $query->orWhereHas('bill_desc', function ($query) {
             //             $query->whereNotIn('included_charges', ['Freight','All']);
             //         });
                     
             //     } 
             //     elseif ($included_charges == 'Way Expenses') {
             //         $query->orWhereHas('bill_desc', function ($query) {
             //             $query->whereNotIn('included_charges', ['Way Expenses','All']);
             //         });
             //     }
             // });
 
             $grs = $grs->get();
 
 
             
             $bill_descs = $grs->map(function ($gr) use ($included_charges) {
                 $bill_desc = [
                     'bill_id' => null,
                     'bill_desc_id' => null,
                     'gr_id' => $gr->id,
                     'gr_number' => $gr->gr_number,
                     'gr_date' => $gr->gr_date,
                     'branch' => $gr->branch,
                     'vehicle' => $gr->vehicle_number,
                     'source' => $gr->source_id ? $gr->source : ['id'=> null, 'station_name'=>$gr->source_name] ,
                     'destination' => $gr->destination_id ? $gr->destination : ['id'=> null, 'station_name'=>$gr->destination_name] ,
                     'consignor' => $gr->consignor_id ? $gr->consignor : ['id'=> null, 'party_name'=>$gr->consignor_name] ,
                     'consignee' => $gr->consignee_id ? $gr->consignee : ['id'=> null, 'party_name'=>$gr->consignee_name] ,
                     'acc_party' => $gr->acc_party,
                     'vehicle_type' =>  $gr->vehicle_type,
                     'total_pkg' => $gr->total_pkg,
                     'total_material_weight' => $gr->total_material_weight,
                     'total_chg_qty' => $gr->total_chg_qty,
                     'total_desc_value' => $gr->total_desc_value,
                     'total_invoice_value' => $gr->total_invoice_value,
                     'other1' => 0,
                     'other2' => 0,
                     'other3' => 0,
                     'other4' => 0,
                     'checked' => true
                 ];
             
                 if ($included_charges == "All") {
                     $bill_desc += [
                         'freight' => $gr->freight,
                         'gr_chg' => $gr->gr_chg,
                         'disc_percent' => $gr->disc_percent,
                         
                         'green_tax' => $gr->green_tax,
                         'kanta' => $gr->kanta,
                         'detention' => $gr->detention,
                         'unload_rate' => $gr->unload_rate,
                            'unload_amt' => $gr->unload_amt,
                            'toll_amt' => $gr->toll_amt,
                            'diversion_rate' => $gr->diversion_rate,
                            'diversion_amt' => $gr->diversion_amt,

                         'gr_desc' => $gr->gr_desc,
                         'payment_dynamic_fields' => json_decode($gr->payment_dynamic_fields),
                     ];
             
                     if ($gr->pod_desc) {
 
                         $bill_desc['freight'] =$gr->pod_desc->pod->freight;
                         $bill_desc['gr_chg'] =$gr->pod_desc->pod->gr_chg;
                         $bill_desc['disc_percent'] =$gr->pod_desc->pod->disc_percent;
                         $bill_desc['disc_amount'] = $gr->pod_desc->pod->disc_amount;
 
                         $bill_desc['pod_id'] = $gr->pod_desc->pod->id;
                         $bill_desc['pod_number'] = $gr->pod_desc->pod->pod_number;
                         $bill_desc['pod_payment_dynamic_fields'] = json_decode($gr->pod_desc->pod->pod_payment_dynamic_fields)??[];
 
                         $bill_desc['load_detention'] = $gr->pod_desc->pod->load_detention;
                         $bill_desc['unload_detention'] = $gr->pod_desc->pod->unload_detention;
                         $bill_desc['igst_amount'] = $gr->pod_desc->pod->igst_amount;
                         $bill_desc['cgst_amount'] = $gr->pod_desc->pod->cgst_amount;
                         $bill_desc['sgst_amount'] = $gr->pod_desc->pod->sgst_amount;
                         $bill_desc['igst_percent'] = $gr->pod_desc->pod->igst_percent;
                         $bill_desc['cgst_percent'] = $gr->pod_desc->pod->cgst_percent;
                         $bill_desc['sgst_percent'] = $gr->pod_desc->pod->sgst_percent;
                         $bill_desc['total_without_gst'] = $gr->pod_desc->pod->total_without_gst;
                         $bill_desc['total_without_rounding'] = $gr->pod_desc->pod->total_without_rounding;
                         $bill_desc['rounded_off'] = $gr->pod_desc->pod->rounded_off;
                         $bill_desc['grand_total'] = $gr->pod_desc->pod->grand_total;
                     } else {
                         $bill_desc['disc_amount'] = $gr->disc_amount;
                         $bill_desc['pod_id'] = null;
                         $bill_desc['pod_number'] = null;
                         $bill_desc['pod_payment_dynamic_fields'] = null;
                         $bill_desc['load_detention'] = 0;
                         $bill_desc['unload_detention'] = 0;
                         $bill_desc['igst_amount'] = $gr->igst_amount;
                         $bill_desc['cgst_amount'] = $gr->cgst_amount;
                         $bill_desc['sgst_amount'] = $gr->sgst_amount;
                         $bill_desc['igst_percent'] = $gr->igst_percent;
                         $bill_desc['cgst_percent'] = $gr->cgst_percent;
                         $bill_desc['sgst_percent'] = $gr->sgst_percent;
                         $bill_desc['total_without_gst'] = $gr->total_without_gst;
                         $bill_desc['rounded_off'] = $gr->rounded_off;
                         $bill_desc['total_without_rounding'] = $gr->total_without_rounding;
                         $bill_desc['grand_total'] = $gr->grand_total;
                     }
                 } 
                 elseif ($included_charges == "Freight") {
                     $bill_desc += [
                         'freight' => $gr->freight,
                         'gr_chg' => $gr->gr_chg,
                         'disc_percent' => $gr->disc_percent,
                         'disc_amount' => $gr->disc_amount,
                         'green_tax' => $gr->green_tax,
                         'kanta' => $gr->kanta,
                         'detention' => $gr->detention,
                         'unload_rate' => $gr->unload_rate,
                            'unload_amt' => $gr->unload_amt,
                            'toll_amt' => $gr->toll_amt,
                            'diversion_rate' => $gr->diversion_rate,
                            'diversion_amt' => $gr->diversion_amt,
                         'gr_desc' => $gr->gr_desc,
                         'payment_dynamic_fields' => json_decode($gr->payment_dynamic_fields)??[],
                     ];
                     
                     if($gr->pod_desc){
                         $bill_desc['pod_id'] = $gr->pod_desc->pod->id;
                         $bill_desc['pod_number'] = $gr->pod_desc->pod->pod_number;
                     }
                     else{
                         $bill_desc['pod_id'] = null;
                         $bill_desc['pod_number'] = null;
                     }
                     
                     $bill_desc['pod_payment_dynamic_fields'] = null;
                     $bill_desc['load_detention'] = 0;
                     $bill_desc['unload_detention'] = 0;
                     $bill_desc['igst_amount'] = $gr->igst_amount;
                     $bill_desc['cgst_amount'] = $gr->cgst_amount;
                     $bill_desc['sgst_amount'] = $gr->sgst_amount;
                     $bill_desc['igst_percent'] = $gr->igst_percent;
                     $bill_desc['cgst_percent'] = $gr->cgst_percent;
                     $bill_desc['sgst_percent'] = $gr->sgst_percent;
                     $bill_desc['total_without_gst'] = $gr->total_without_gst;
                     $bill_desc['total_without_rounding'] = $gr->total_without_rounding;
                     $bill_desc['rounded_off'] = $gr->rounded_off;
                     $bill_desc['grand_total'] = $gr->grand_total;
                 } 
                 elseif ($included_charges == "Way Expenses") {
                     $bill_desc += [
                         'freight' => null,
                         'gr_chg' => null,
                         'disc_percent' => null,
                         'disc_amount' => null,
                         'green_tax' => null,
                         'kanta' => null,
                         'detention' => null,
                            'unload_rate' => null,
                                'unload_amt' => null,
                                'toll_amt' => null,
                                'diversion_rate' => null,
                                'diversion_amt' => null,
                         'gr_desc' => null,
                         'payment_dynamic_fields' => null,
                     ];
                     
                     if($gr->pod_desc){
                         $bill_desc['pod_id'] = $gr->pod_desc->pod->id;
                         $bill_desc['pod_number'] = $gr->pod_desc->pod->pod_number;
                         $bill_desc['load_detention'] = $gr->pod_desc->pod->load_detention;
                         $bill_desc['unload_detention'] = $gr->pod_desc->pod->unload_detention;
                         $bill_desc['pod_payment_dynamic_fields'] = json_decode($gr->pod_desc->pod->pod_payment_dynamic_fields)??[];
                         $bill_desc['igst_amount'] = $gr->pod_desc->pod->igst_amount;
                         $bill_desc['cgst_amount'] = $gr->pod_desc->pod->cgst_amount;
                         $bill_desc['sgst_amount'] = $gr->pod_desc->pod->sgst_amount;
                         $bill_desc['igst_percent'] = $gr->pod_desc->pod->igst_percent;
                         $bill_desc['cgst_percent'] = $gr->pod_desc->pod->cgst_percent;
                         $bill_desc['sgst_percent'] = $gr->pod_desc->pod->sgst_percent;
 
                         $pod_dynamic_tot = $gr->pod_desc->pod->pod_payment_dynamic_total;
                         
                         $bill_desc['total_without_gst'] = $pod_dynamic_tot;
                         $bill_desc['igst_amount'] = ($pod_dynamic_tot * $bill_desc['igst_percent'])/100;
                         $bill_desc['cgst_amount'] = ($pod_dynamic_tot * $bill_desc['cgst_percent'])/100;
                         $bill_desc['sgst_amount'] = ($pod_dynamic_tot * $bill_desc['sgst_percent'])/100;
                         $bill_desc['total_without_rounding'] = $pod_dynamic_tot + $bill_desc['igst_amount'] + $bill_desc['cgst_amount'] + $bill_desc['sgst_amount'];
                         $bill_desc['rounded_off'] = round($bill_desc['total_without_rounding']) - $bill_desc['total_without_rounding'];
                         $bill_desc['grand_total'] = $bill_desc['total_without_rounding'] + $bill_desc['rounded_off'];
                     }
                 }
             
                 return $bill_desc;
             });
             
        }
 
        if($type == 'Loading Slip'){
 
             $ls = LoadingSlip::with([
                     'branch:id,branch_name,branch_code',
                     'vehicle:id,vehicle_number,vehicle_type',
                     'source:id,station_name',
                     'destination:id,station_name',
                 ])
                 ->where('bill_status', 0)
                 ->where('acc_party_id', $acc_party_id)
                 ->where('ls_date', '>=', $from_date)
                 ->where('ls_date', '<=', $to_date)
                 ->whereDoesntHave('voucher_desc', function ($query) {
                     $query->where('against', 'Loading Slip');
                     // Add more conditions if needed for 'against_number'
                 })
                 ->when($vehicle_id, function ($query, $vehicle_id) {
                    return $query->where('vehicle_id', $vehicle_id);
                 })
                 ->when($destination_id, function ($query, $destination_id) {
                    return $query->where('destination_id', $destination_id);
                })
                 ->when($acc_party_branch_id, function ($query, $acc_party_branch_id) {
                    // Add the condition only if $acc_party_branch_id is not empty
                    return $query->where('acc_party_branch_id', $acc_party_branch_id);
                })
                 ->get();
 
                 $bill_descs = $ls->map(function ($ls) {
                     return [
                         'bill_id' => null,
                         'bill_desc_id' => null,
                         'ls_id' => $ls->id,
                         'ls_number' => $ls->ls_number,
                         'ls_date' => $ls->ls_date,
                         'branch' => $ls->branch,
                         'vehicle' => $ls->vehicle,
                         'source' => $ls->source,
                         'destination' => $ls->destination,
                         'freight' => $ls->freight,
                         'munshiana' => $ls->munshiana,
                         'dalla' => $ls->dalla,
                         'advance_amt' => $ls->advance_amt,
                         'balance_received_in_amt' => $ls->balance_received_in_amt,
                         'diesel_amt' => $ls->diesel_amt,
                         'payment_and_collection' => $ls->payment_and_collection,
                         'labour' => $ls->labour,
                         'detention' => $ls->detention,
                         'unload_detention' => $ls->unload_detention,
                         'balance_receivable' => $ls->balance_receivable,
                         'total_without_gst' => $ls->total_without_gst,
                         'igst_amount' => 0,
                         'cgst_amount' => 0,
                         'sgst_amount' => 0,
                         'igst_percent' =>  0,
                         'cgst_percent' =>  0,
                         'sgst_percent' =>  0,
                         'checked' => true
                     ];
                 });
 
        }
 
        return response()->json(['bill_desc'=>$bill_descs]);
 
     }

    public function get_bill_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = Bill::with([
            'branch:id,branch_name,branch_code', 
            'acc_party:id,account_name',
            'acc_party.party:id,account_id,party_name,gst_number,state,city,address1,address2,pincode', 
            'bill_desc:id,bill_id,bill_number,gr_id,vehicle_id,gr_number,ls_number,pod_id,total_desc_value,freight,supplementary_desc', 
            'bill_desc.vehicle:id,vehicle_number,vehicle_type',
            'bill_desc.pod:id,pod_number,pod_payment_dynamic_total',
            'voucher_desc'=>function($query){
                $query->where('against','Bill')->where('voucher_name','Receipt Voucher');
            }
        ])->where('fin_year', $request->current_year);
        
        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['bill_num_from']) && isset($report_data['bill_num_to'])){
            $query->where('bill_number','>=',$report_data['bill_num_from']['bill_number'])->where('bill_number','<=',$report_data['bill_num_to']['bill_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('bill_date','>=',$report_data['from_date'])->where('bill_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['selection_type']) ){
            $query->where('selection_type', $report_data['selection_type']);
        }

        if(isset($report_data['acc_party_branch']) ){
            $query->where('acc_party_branch_id', $report_data['acc_party_branch']['id']);
        }

        if(isset($report_data['paid_status']) ){
            if($report_data['paid_status'] == 'Outstanding'){
              $query->where('paid_status', 0)->where('grand_total_amount','>',0);  
            }
            else if($report_data['paid_status'] == 'Paid'){
                $query->where(function($query){
                    $query->where('paid_status', 1)->orWhere('grand_total_amount',0);
                });
            }
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }

        


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', '%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_pending_bill_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                 
        if($report_data['selection_type'] == 'GR'){
          $query = BookGr::select('id','branch_id','gr_number as against_number','gr_date as date', 'vehicle_number','vehicle_type','source_id','source_name','destination_id','destination_name','acc_party_id', 'grand_total as freight_to_show')
          ->with(['branch:id,branch_name,branch_code','source:id,station_name','destination:id,station_name','acc_party:id,account_name',
          'challan_desc:id,challan_id,challan_number,gr_id,gr_number','challan_desc.challan:id,challan_number,lorry_hire_from,lh_freight,lh_net_freight,lh_rate,lh_total_wt'
          ])
          ->where('pay_mode','TBB')->whereDoesntHave('bill_desc');
          $date = 'gr_date';
        }
        elseif($report_data['selection_type'] == 'Loading Slip'){
          $query = LoadingSlip::select('id','branch_id','ls_number as against_number','ls_date as date', 'source_id','destination_id','vehicle_number','acc_party_id', 'total_without_gst as freight_to_show')->with(['branch:id,branch_name,branch_code','source:id,station_name','destination:id,station_name','acc_party:id,account_name'])->where('bill_status',0)->whereDoesntHave('bill_desc');
          $date = 'ls_date';
        }

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

    
        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where($date,'>=',$report_data['from_date'])->where($date,'<=',$report_data['to_date']);
        }

        // if ( !empty($request->user_branch_ids) ) {
        //     $query->whereIn('branch_id', $request->user_branch_ids );
        // }

        if(isset($report_data['with_without_rate'])){
            if($report_data['with_without_rate'] == 'WithRate'){
                $query->where('grand_total','>',0);
            }
            elseif($report_data['with_without_rate'] == 'WithoutRate'){
                $query->where(function ($q) {
                    $q->whereIn('grand_total', [0, ''])
                      ->orWhereNull('grand_total');
                });
                
            }
        }

    

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('against_number', 'like', '%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_issued_bill(Request $request){
        $bill_number = $request->bill_number;
        $branch_id = $request->branch_id;
        $bill_number = $request->bill_number;
        $prefix = $request->prefix;

        // Break BILL NUMBER BASED ON ANY Separater
        $bill_number = $this->get_max_number_from_bill_number($bill_number);

        $check = IssueBooks::with(['branch' => function ($query) {
                        $query->select('id', 'branch_name','branch_code');
                    },
                    'party' => function ($query) {
                        $query->select('id', 'party_name','party_code','account_id');
                    }
                ])
                ->where('book_type', 'Bill')
                ->where('prefix', $prefix)
                ->where('branch_id', $branch_id)
                ->where('issue_from', '<=', $bill_number)
                ->where('issue_to', '>=', $bill_number)
                ->first();

        if($check){
            return response()->json(['success'=>true, 'data'=>$check],200);
        }
        else{
            return response()->json(['success'=>false, 'msg'=>'Not Issued To Correct Branch'],200);
        }

    

    }

    public function get_outstanding_bill_report(Request $request){
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = Bill::select('id','bill_number','branch_id','acc_party_id','bill_date','submit_date','grand_total_amount')->with(['branch:id,branch_name,branch_code', 'acc_party:id,account_name', 'bill_desc:id,bill_id,bill_number,gr_id,gr_number']);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('bill_date','>=',$report_data['from_date'])->where('bill_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['selection_type']) ){
            $query->where('selection_type', $report_data['selection_type']);
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }
        

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', '%'. $searchValue . '%');
            });
        }

        $data = $query->orderBy('acc_party_id')->get();

        $result_data = [];

        $currentDate = now(); // Assuming you want the current date and time

        foreach ($data as $value) {

            $amount = $value['grand_total_amount'];

            // Calculate the difference in days
            $daysDifference = $currentDate->diffInDays($value['bill_date']);

            $balance = $this->check_for_balance($value, $value->grand_total_amount, 'Bill');

 
            if($balance>1){
                $result_data[ $value->acc_party['account_name']  ] [] = [
                    'bill_id' => $value['id'],
                    'bill_number' => $value['bill_number']  ,  
                    'bill_date' => $value['bill_date']  ,
                    'submit_date' => $value['submit_date'], 
                    'bill_desc' => $value['bill_desc'],
                    'amount_0_30' => ($daysDifference>=0 && $daysDifference<=30) ? $balance : 0,
                    'amount_31_60' => ($daysDifference>=31 && $daysDifference<=60) ? $balance : 0,
                    'amount_61_90' => ($daysDifference>=61 && $daysDifference<=90) ? $balance : 0,
                    'amount_90' => ($daysDifference>=91 ) ? $balance : 0,
                ] ;
            }
            
        }


       
          
        return response()->json(['data' => $result_data, 'draw' => $request->draw]);
    }

    public function get_outstanding_creditors_bill_report(Request $request){
        $searchValue = $request->search;

        $report_data = $request->report_data;

                                    
        $result_data = [];

        if($report_data['selection_type'] == 'Lorry Hire'){

            $lorry_hire = LorryHire::select('id','lh_number','lh_date as date','lorry_hire_from','net_freight','balance as freight_to_show', \DB::raw("'Lorry Hire' as selection_type"))->with(['lorry_hire_from:id,account_name'])->where('paid_status',false);
            
            $challan = Challan::select('id','challan_number','challan_date as date','lorry_hire_from','lh_net_freight','lh_balance as freight_to_show', \DB::raw("'Challan' as selection_type"))->with(['lorry_hire_from:id,account_name'])->where('paid_status',false);
            
            if(isset($report_data['acc_party']) ){
                $lorry_hire->where('lorry_hire_from', $report_data['acc_party']['id']);
                $challan->where('lorry_hire_from', $report_data['acc_party']['id']);
            }

            if ( !empty($request->user_branch_ids) ) {
                $lorry_hire->whereIn('branch_id', $request->user_branch_ids );
                $challan->whereIn('branch_id', $request->user_branch_ids );
            }
    
    
            if(isset($report_data['from_date']) && isset($report_data['to_date'])){
                $lorry_hire->where('lh_date','>=',$report_data['from_date'])->where('lh_date','<=',$report_data['to_date']);
                $challan->where('challan_date','>=',$report_data['from_date'])->where('challan_date','<=',$report_data['to_date']);
            
                if ($searchValue) {
                    $challan->where(function ($query) use ($searchValue) {
                        $query->where('challan_number', 'like', '%'. $searchValue . '%');
                    });
                }

                if ($searchValue) {
                    $lorry_hire->where(function ($query) use ($searchValue) {
                        $query->where('lh_number', 'like', '%'. $searchValue . '%');
                    });
                }

               

            }

            $lorry_hire_data= $lorry_hire->get();
            $challan_data = $challan->get();

            $data = $lorry_hire_data->merge($challan_data);
            
        }

        if($report_data['selection_type'] == 'Purchase'){
            $purchase = Purchase::select('id','bill_number','bill_date as date','acc_party_id','total_without_rounding as freight_to_show',\DB::raw("'Purchase' as selection_type"))->where('bill_name','!=','Stock Issue')->with(['acc_party:id,account_name'])->where('paid_status',false);
            
            if(isset($report_data['acc_party']) ){
                $purchase->where('acc_party_id', $report_data['acc_party']['id']);
            }

            if(isset($report_data['from_date']) && isset($report_data['to_date'])){
                $purchase->where('bill_date','>=',$report_data['from_date'])->where('bill_date','<=',$report_data['to_date']);
            }
            
            if ( !empty($request->user_branch_ids) ) {
                $purchase->whereIn('branch_id', $request->user_branch_ids );
            }

            if ($searchValue) {
                $purchase->where(function ($query) use ($searchValue) {
                    $query->where('bill_number', 'like', '%'. $searchValue . '%');
                });
            }

            $data = $purchase->get();
        }

        $result_data = [];

        $currentDate = now(); // Assuming you want the current date and time

        foreach ($data as $value) {
            $copy = json_decode(json_encode($value), true);

            $amount = $value['freight_to_show'];

            // Calculate the difference in days
            $daysDifference = $currentDate->diffInDays($value['date']);

            if($report_data['selection_type'] == 'Lorry Hire'){
                $balance = $this->check_for_balance($value, $value->freight_to_show, ['Lorry Hire','Challan']);
              
            }
            if($report_data['selection_type'] == 'Purchase'){
                $balance = $this->check_for_balance($value, $value->freight_to_show, ['Purchase']);
            }

            if($balance>1){
                $party_name = $value->acc_party ?  $value->acc_party['account_name'] :'';

                if($report_data['selection_type'] == 'Lorry Hire'){
                    $party_name =  $copy['lorry_hire_from']['account_name'];
                }

                $result_data[ $party_name ] [] = [
                    'bill_id' => $value['id'],
                    'bill_number' => $value['challan_number'] ?? $value['lh_number'] ?? $value['bill_number'],  
                    'bill_date' => $value['date']  ,
                    'submit_date' => $value['date'], 
                    'bill_desc' => null,
                    'selection_type' => $value['selection_type'],
                    'amount_0_30' => ($daysDifference>=0 && $daysDifference<=30) ? $balance : 0,
                    'amount_31_60' => ($daysDifference>=31 && $daysDifference<=60) ? $balance : 0,
                    'amount_61_90' => ($daysDifference>=61 && $daysDifference<=90) ? $balance : 0,
                    'amount_90' => ($daysDifference>=91 ) ? $balance : 0,
                ] ;
            }
            
        }



        return response()->json(['data' => $result_data, 'draw' => $request->draw]);
    }

    protected function check_for_balance($against,$default_balance,$selection){
            $filteredVoucherDesc = $against->voucher_desc->whereIn('against', $selection);
            
            if ($filteredVoucherDesc->isEmpty()) {
                // No voucher_desc, return grand_total_amount as balance
                return $default_balance;
            }
            
            $recievedAmtSum = $filteredVoucherDesc->sum('receipt_amt');
            $deductionSum = $filteredVoucherDesc->sum('deduction');
            
            $balance = $default_balance - $recievedAmtSum - $deductionSum ;
            return $balance;
    }



    protected function get_data_for_obj($pk){
        $bill = Bill::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'income:id,account_name',
            'acc_party_branch:id,branch_name',

            'bill_desc',
            'bill_desc.vehicle:id,vehicle_number,vehicle_type',
            'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,source_id,destination_id,consignor_name,consignee_name,source_name,destination_name',
            'bill_desc.bookGr.consignor:id,party_name',
            'bill_desc.bookGr.consignee:id,party_name',
            'bill_desc.bookGr.source:id,station_name',
            'bill_desc.bookGr.destination:id,station_name',
            'bill_desc.pod:id,pod_number',
            
            'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',

            'bill_desc.loading_slip'
        ])->find($pk);


        $bill->bill_desc->transform(function ($desc) {
            $data = [
                'bill_desc_id' => $desc->id,
                'bill_id' => $desc->bill_id,
                'total_without_gst' => $desc->total_without_gst,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
                'rounded_off' => $desc->rounded_off,
                'grand_total' => $desc->grand_total,
            ];

            
        
            if ($desc->selection_type == 'GR') {
                $data += [
                    'gr_id' => $desc->gr_id,
                    'gr_number' => $desc->gr_number,
                    'gr_date' => $desc->bookGr->gr_date,
                    'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                    'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                    'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                    'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                    'total_pkg' => $desc->total_pkg,
                    'total_material_weight' => $desc->total_material_weight,
                    'total_chg_qty' => $desc->total_chg_qty,
                    'total_desc_value' => $desc->total_desc_value,
                    'total_invoice_value' => $desc->total_invoice_value,
                    'freight' => $desc->freight,
                    'gr_chg' => $desc->gr_chg,
                    'disc_percent' => $desc->disc_percent,
                    'disc_amount' => $desc->disc_amount,
                    'green_tax' => $desc->green_tax,
                    'kanta' => $desc->kanta,
                    'detention' => $desc->detention,
                    'unload_rate' => $desc->unload_rate,
                    'unload_amt' => $desc->unload_amt,
                    'toll_amt' => $desc->toll_amt,
                    'diversion_rate' => $desc->diversion_rate,
                    'diversion_amt' => $desc->diversion_amt,

                    'other1' => $desc->other1,
                    'other2' => $desc->other2,
                    'other3' => $desc->other3,
                    'other4' => $desc->other4,


                    
                    'gr_desc' => json_decode($desc->gr_desc),
                    'pod_id' => $desc->pod ? $desc->pod_id : null,
                    'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                    'load_detention' => $desc->load_detention,
                    'unload_detention' => $desc->unload_detention,
                    'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                    'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                ];
            }
        
            if ($desc->selection_type == 'Loading Slip') {
                $data += [
                    'ls_id' => $desc->ls_id,
                    'ls_number' => $desc->ls_number,
                    'ls_date' => $desc->loading_slip->ls_date,
                    'branch' => $desc->loading_slip->branch,
                    'vehicle' => $desc->loading_slip->vehicle,
                    'source' => $desc->loading_slip->source,
                    'destination' => $desc->loading_slip->destination,
                    'freight' => $desc->loading_slip->freight,
                    'munshiana' => $desc->loading_slip->munshiana,
                    'dalla' => $desc->loading_slip->dalla,
                    'advance_amt' => $desc->loading_slip->advance_amt,
                    'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                    'diesel_amt' => $desc->loading_slip->diesel_amt,
                    'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                    'labour' => $desc->loading_slip->labour,
                    'detention' => $desc->loading_slip->detention,
                    'unload_detention' => $desc->loading_slip->unload_detention,
                    'balance_receivable' => $desc->loading_slip->balance_receivable,
                ];
            }

            if ($desc->selection_type == 'Supplementary') {
                $data += [
                    'supplementary_desc' => json_decode($desc->supplementary_desc),
                    'vehicle' => $desc->vehicle
                ];
            }
        
            return $data;
        });

        return ['bill'=>$bill];
    }

    protected function get_max_number_from_bill_number($input){
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function handle_update_in_gr_pod($request,$bd){
        if(!$request->update_gr_pod_from_bill)
        return;

        $sumofgst = $request->total_cgst_amount + $request->total_sgst_amount + $request->total_igst_amount;

        // UPDATING IN GR AND POD BEGINS
        if($request->included_charges=="All"){
            if($bd['pod_id']){ // if pod is selected 
                $pod = Pod::find($bd['pod_id']);
                
                // Check if the POD HAS MORE THAN 1 DESCS, if yes then RETURN ERRROR
                if($pod->pod_desc->count() > 1){
                    return response()->json([
                        'status' => 'error',
                        'message' => 'POD has more than 1 desc'
                    ],500);
                }

                $pod->pod_payment_dynamic_fields = $bd['pod_payment_dynamic_fields'];
                $pod->load_detention = $bd['load_detention'];
                $pod->unload_detention = $bd['unload_detention'];

                $pod->pod_payment_dynamic_total = $bd['pod_payment_dynamic_total'];
                $pod->freight = $bd['freight'];
                $pod->gr_chg = $bd['gr_chg'];
                $pod->disc_percent = $bd['disc_percent'];
                $pod->disc_amount = $bd['disc_amount'];
                $pod->green_tax = $bd['green_tax'];
                $pod->detention = $bd['detention'];
                $pod->kanta = $bd['kanta'];
                $pod->unload_rate = $bd['unload_rate'];
                $pod->unload_amt = $bd['unload_amt'];
                $pod->toll_amt = $bd['toll_amt'];
                $pod->diversion_rate = $bd['diversion_rate'];
                $pod->diversion_amt = $bd['diversion_amt'];

                $pod->payment_dynamic_fields = $bd['payment_dynamic_fields'];
                $pod->total_without_gst = $bd['total_without_gst'];
                
                if($sumofgst>=1){
                    $pod->gst_paid_by = "Transporter";
                }

                $pod->igst_percent = $request->igst_percent;
                $pod->cgst_percent = $request->cgst_percent;
                $pod->sgst_percent = $request->sgst_percent;
                $pod->igst_amount = $bd['igst_amount'];
                $pod->cgst_amount = $bd['cgst_amount'];
                $pod->sgst_amount = $bd['sgst_amount'];
                $pod->total_without_rounding = $bd['total_without_rounding'];
                $pod->rounded_off = $bd['rounded_off'];
                $pod->grand_total = $bd['grand_total'];
                $pod->save();

                // Find gst values and total_without rounding and grand total for gr :-
                $bd['total_without_gst'] = $bd['total_without_gst'] - $bd['pod_payment_dynamic_total'];
                $bd['igst_amount'] = ($bd['total_without_gst'] * $request->igst_percent)/100;
                $bd['cgst_amount'] = ($bd['total_without_gst'] * $request->cgst_percent)/100;
                $bd['sgst_amount'] = ($bd['total_without_gst'] * $request->sgst_percent)/100;
                $bd['total_without_rounding'] = $bd['total_without_gst'] + $bd['igst_amount'] + $bd['cgst_amount'] + $bd['sgst_amount'];
                $bd['rounded_off'] = round($bd['total_without_rounding']) - $bd['total_without_rounding'];
                // Calculate the grand total
                $bd['grand_total']  = $bd['total_without_rounding'] + $bd['rounded_off'] ;
            }

            // Updating in gr
            $gr = BookGr::find($bd['gr_id']);
            $gr->total_desc_value = $bd['total_desc_value'];
            $gr->freight = $bd['freight'];
            $gr->gr_chg = $bd['gr_chg'];
            $gr->disc_percent = $bd['disc_percent'];
            $gr->disc_amount = $bd['disc_amount'];
            $gr->green_tax = $bd['green_tax'];
            $gr->detention = $bd['detention'];
            $gr->kanta = $bd['kanta'];
            $gr->unload_rate = $bd['unload_rate'];
            $gr->unload_amt = $bd['unload_amt'];
            $gr->toll_amt = $bd['toll_amt'];
            $gr->diversion_rate = $bd['diversion_rate'];
            $gr->diversion_amt = $bd['diversion_amt'];

            $gr->payment_dynamic_fields = $bd['payment_dynamic_fields'];
            $gr->total_without_gst = $bd['total_without_gst'];

            if($sumofgst>=1){
                $gr->gst_paid_by = "Transporter";
            }

            $gr->igst_percent = $request->igst_percent;
            $gr->cgst_percent = $request->cgst_percent;
            $gr->sgst_percent = $request->sgst_percent;
            $gr->igst_amount = $bd['igst_amount'];
            $gr->cgst_amount = $bd['cgst_amount'];
            $gr->sgst_amount = $bd['sgst_amount'];
            $gr->total_without_rounding = $bd['total_without_rounding'];
            $gr->rounded_off = $bd['rounded_off'];
            $gr->grand_total = $bd['grand_total'];
            $gr->save();

            // Updating the rates in the gr desc
            foreach ($bd['gr_desc'] as $gd) {
                $gr_desc = GrDesc::find($gd['id']);
                $gr_desc->rate = $gd['rate'];
                $gr_desc->total = $gd['total'];
                $gr_desc->save();
            }
            
        }

        if($request->included_charges=="Freight"){
            // Updating in gr
            $gr = BookGr::find($bd['gr_id']);
            $gr->total_desc_value = $bd['total_desc_value'];
            $gr->freight = $bd['freight'];
            $gr->gr_chg = $bd['gr_chg'];
            $gr->disc_percent = $bd['disc_percent'];
            $gr->disc_amount = $bd['disc_amount'];
            $gr->green_tax = $bd['green_tax'];
            $gr->detention = $bd['detention'];
            $gr->kanta = $bd['kanta'];
            $gr->unload_rate = $bd['unload_rate'];
            $gr->unload_amt = $bd['unload_amt'];
            $gr->toll_amt = $bd['toll_amt'];
            $gr->diversion_rate = $bd['diversion_rate'];
            $gr->diversion_amt = $bd['diversion_amt'];

            $gr->payment_dynamic_fields = $bd['payment_dynamic_fields'];
            $gr->total_without_gst = $bd['total_without_gst'];

            if($sumofgst>=1){
                $gr->gst_paid_by = "Transporter";
            }

            $gr->igst_percent = $request->igst_percent;
            $gr->cgst_percent = $request->cgst_percent;
            $gr->sgst_percent = $request->sgst_percent;
            $gr->igst_amount = $bd['igst_amount'];
            $gr->cgst_amount = $bd['cgst_amount'];
            $gr->sgst_amount = $bd['sgst_amount'];
            $gr->total_without_rounding = $bd['total_without_rounding'];
            $gr->rounded_off = $bd['rounded_off'];
            $gr->grand_total = $bd['grand_total'];
            $gr->save();

            // Updating the rates in the gr desc
            foreach ($bd['gr_desc'] as $gd) {
                $gr_desc = GrDesc::find($gd['id']);
                $gr_desc->rate = $gd['rate'];
                $gr_desc->total = $gd['total'];
                $gr_desc->save();
            }

            if($bd['pod_id']){ // if pod is selected 
                $pod = Pod::find($bd['pod_id']);
                
                // Check if the POD HAS MORE THAN 1 DESCS, if yes then RETURN ERRROR
                if($pod->pod_desc->count() > 1){
                    return response()->json([
                        'status' => 'error',
                        'message' => 'POD has more than 1 desc'
                    ],500);
                }

                $pod->freight = $bd['freight'];
                $pod->gr_chg = $bd['gr_chg'];
                $pod->disc_percent = $bd['disc_percent'];
                $pod->disc_amount = $bd['disc_amount'];
                $pod->green_tax = $bd['green_tax'];
                $pod->detention = $bd['detention'];
                $pod->kanta = $bd['kanta'];
                $pod->unload_rate = $bd['unload_rate'];
                $pod->unload_amt = $bd['unload_amt'];
                $pod->toll_amt = $bd['toll_amt'];
                $pod->diversion_rate = $bd['diversion_rate'];
                $pod->diversion_amt = $bd['diversion_amt'];
                $pod->payment_dynamic_fields = $bd['payment_dynamic_fields'];
                $pod->load_detention = $bd['load_detention'];
                $pod->unload_detention = $bd['unload_detention'];

                $pod->total_without_gst = $bd['total_without_gst'] + $pod->pod_payment_dynamic_total;
                
                if($sumofgst>=1){
                    $pod->gst_paid_by = "Transporter";
                }

                $pod->igst_percent = $request->igst_percent;
                $pod->cgst_percent = $request->cgst_percent;
                $pod->sgst_percent = $request->sgst_percent;
                $pod->igst_amount = ($pod->total_without_gst * $request->igst_percent)/100;
                $pod->cgst_amount = ($pod->total_without_gst * $request->cgst_percent)/100;
                $pod->sgst_amount = ($pod->total_without_gst * $request->cgst_percent)/100;
                $pod->total_without_rounding = $pod->total_without_gst + $pod->igst_amount + $pod->cgst_amount + $pod->sgst_amount;
                $pod->rounded_off = round($pod->total_without_rounding) - $pod->total_without_rounding;
                $pod->grand_total = $pod->total_without_rounding + $pod->rounded_off;
                $pod->save();
            }

            
            
        }

        if($request->included_charges=="Way Expenses"){
            $gr = BookGr::find($bd['gr_id']);

            if($bd['pod_id']){ // if pod is selected 
                $pod = Pod::find($bd['pod_id']);
                
                // Check if the POD HAS MORE THAN 1 DESCS, if yes then RETURN ERRROR
                if($pod->pod_desc->count() > 1){
                    return response()->json([
                        'status' => 'error',
                        'message' => 'POD has more than 1 desc'
                    ],500);
                }


                $pod->pod_payment_dynamic_fields = $bd['pod_payment_dynamic_fields'];
                $pod->load_detention = $bd['load_detention'];
                $pod->unload_detention = $bd['unload_detention'];
                
                
                $without_pod_total = $bd['total_without_gst'] - $pod->pod_payment_dynamic_total;
                $pod->pod_payment_dynamic_total = $bd['pod_payment_dynamic_total'];

                $pod->total_without_gst =  $without_pod_total + $pod->total_without_gst;
                $pod->igst_percent = $request->igst_percent;
                $pod->cgst_percent = $request->cgst_percent;
                $pod->sgst_percent = $request->sgst_percent;
                $pod->igst_amount = ($pod->total_without_gst * $request->igst_percent)/100;
                $pod->cgst_amount = ($pod->total_without_gst * $request->cgst_percent)/100;
                $pod->sgst_amount = ($pod->total_without_gst * $request->cgst_percent)/100;
                $pod->total_without_rounding = $pod->total_without_gst + $pod->igst_amount + $pod->cgst_amount + $pod->sgst_amount;
                $pod->rounded_off = round($pod->total_without_rounding) - $pod->total_without_rounding;
                $pod->grand_total = $pod->total_without_rounding + $pod->rounded_off;
                $pod->save();
            }
        }
        // UPDATING IN GR AND POD ENDS
    }

    public function get_gst_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        

        $query = Voucher::select('id','voucher_name','t_against','t_against_id','t_against_no')->where('voucher_name', 'TRANSACTION')
                ->whereIn('t_against', ['BILL'])
                ->where('fin_year', $request->current_year)
                ->whereHas('voucher_acc_details', function ($qr) {
                    $qr->whereIn('account_id', [12, 13, 14])->where('nature', 'cr');
                });
        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }
        
        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('voucher_date','>=',$report_data['from_date'])->where('voucher_date','<=',$report_data['to_date']);
        }

        $active_transaction_vouchers = $query->get();

        $result_data = [];
        
        
            foreach ($active_transaction_vouchers as $atv) {
                
                if($report_data['type_of_report']=='GST Report B2B'){
                    $data = [
                        'acc_party_gst' => '',
                        'receiver_name' => '',
                        'invoice_number' => '',
                        'invoice_date' => '',
                        'invoice_value' => '',
                        'place_of_supply' => '',
                        'reverse_charge' => '',
                        'tax_rate' => '',
                        'invoice_type' => '',
                        'ecommerce_gst' => '',
                        'rate' => '',
                        'taxable_value' => '',
                        'cess_amount' => '',
                    ];
                }elseif($report_data['type_of_report']=='GST Report B2C'){
                    $data = [
                        'type' => '',
                        'place_of_supply' => '',
                        'tax_rate' => '',
                        'ecommerce_gst' => '',
                        'rate' => '',
                        'taxable_value' => '',
                        'cess_amount' => '',
                    ];
                }
                

                if($atv['t_against']=='GR'){
                    $book_gr = BookGr::select('id','gr_number','gr_date','consignor_id','consignee_id','source_id','destination_id','consignor_name','consignee_name','source_id','destination_id','consignor_gst_number','consignee_gst_number','acc_party_id','pay_mode','total_without_gst','grand_total','igst_percent','cgst_percent','sgst_percent','igst_amount','cgst_amount','sgst_amount')
                            ->with(['branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'acc_party:id,account_name', 'acc_party.party:id,party_name,gst_number,account_id'])
                            ->find($atv['t_against_id']);
                    
                
                    $formattedBillDate = date('d-M-Y', strtotime($book_gr->gr_date));
                    $place_of_supply = $book_gr->destination ? $book_gr->destination->station_name : '';        
                    if($report_data['type_of_report']=='GST Report B2B'){        
                    $data = [
                        'acc_party_gst' => $book_gr->acc_party->party->gst_number,
                        'receiver_name' => $book_gr->acc_party->party->party_name,
                        'invoice_number' => $book_gr->gr_number,
                        'invoice_date' => $formattedBillDate,
                        'invoice_value' => $book_gr->grand_total,
                        'place_of_supply' => $place_of_supply,
                        'reverse_charge' => 'N',
                        'tax_rate' => 0,
                        'invoice_type' => 'Regular B2B',
                        'ecommerce_gst' => '',
                        'rate' => $book_gr->igst_percent+$book_gr->cgst_percent+$book_gr->sgst_percent,
                        'taxable_value' => $book_gr->total_without_gst,
                        'cess_amount' => 0,
                    ];
                    }
                    elseif($report_data['type_of_report']=='GST Report B2C'){
                        $data = [
                            'type' => 'OE',
                            'acc_party_gst' => $book_gr->acc_party->party->gst_number,
                            'place_of_supply' => $place_of_supply,
                            'tax_rate' => 0,
                            'ecommerce_gst' => '',
                            'rate' => $book_gr->igst_percent+$book_gr->cgst_percent+$book_gr->sgst_percent,
                            'taxable_value' => $book_gr->total_without_gst,
                            'cess_amount' => 0,
                        ];
                    }

                    
                    $result_data[] = $data;
                }
                elseif($atv['t_against']=='POD'){
                $pod = Pod::with([
                            'branch:id,branch_name,branch_code',
                            'acc_party:id,account_name',
                            'pod_desc:id,gr_id,pod_id,gr_number',
                            'pod_desc.bookGr:id,gr_number,gr_date,pay_mode,acc_party_id,vehicle_id,delivery_pt,source_id,destination_id,consignor_id,consignee_id,consignor_name,consignee_name,destination_name,total_without_gst',
                            'pod_desc.bookGr.acc_party:id,account_name',
                            'pod_desc.bookGr.consignor:id,party_name',
                            'pod_desc.bookGr.consignee:id,party_name',
                            'pod_desc.bookGr.source:id,station_name',
                            'pod_desc.bookGr.destination:id,station_name'       
                        ])
                        ->find($atv['t_against_id']);

                    $place_of_supply = $pod->pod_desc ? ($pod->pod_desc[0]->bookGr->destination ? $pod->pod_desc[0]->bookGr->destination->station_name : '') : '';
                    
                    $formattedBillDate = date('d-M-Y', strtotime($pod->delivery_date));

                    if($report_data['type_of_report']=='GST Report B2B'){
                        $data = [
                            'acc_party_gst' => $pod->acc_party->party->gst_number,
                            'receiver_name' => $pod->acc_party->party->party_name,
                            'invoice_number' => $pod->pod_number,
                            'invoice_date' => $formattedBillDate,
                            'invoice_value' => $pod->grand_total,
                            'place_of_supply' => $place_of_supply,
                            'reverse_charge' => 'N',
                            'tax_rate' => 0,
                            'invoice_type' => 'Regular B2B',
                            'ecommerce_gst' => '',
                            'rate' => $pod->igst_percent+$pod->cgst_percent+$pod->sgst_percent,
                            'taxable_value' => $pod->total_without_gst,
                            'cess_amount' => 0,
                        ];
                    }
                    elseif($report_data['type_of_report']=='GST Report B2C'){
                        $data = [
                            'type' => 'OE',
                            'acc_party_gst' => $pod->acc_party->party->gst_number,
                            'place_of_supply' => $place_of_supply,
                            'tax_rate' => 0,
                            'ecommerce_gst' => '',
                            'rate' =>  $pod->igst_percent+$pod->cgst_percent+$pod->sgst_percent,
                            'taxable_value' => $pod->total_without_gst,
                            'cess_amount' => 0,
                        ];
                    }

                    $result_data[] = $data;
                }
                elseif($atv['t_against']=='BILL'){
                    $bill = Bill::with([
                                'branch:id,branch_name,branch_code',
                                'acc_party:id,account_name',
                                'bill_desc',
                                'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,source_id,destination_id,consignor_name,consignee_name,source_name,destination_name,total_without_gst',
                                'bill_desc.bookGr.consignor:id,party_name',
                                'bill_desc.bookGr.consignee:id,party_name',
                                'bill_desc.bookGr.source:id,station_name',
                                'bill_desc.bookGr.destination:id,station_name',
                                'bill_desc.pod:id,pod_number,pod_payment_dynamic_total',
                            ])
                        ->find($atv['t_against_id']);

                    $place_of_supply = substr($bill->acc_party->party->gst_number,0,2).'-'.$bill->acc_party->party->state;
                
                    $formattedBillDate = date('d-M-Y', strtotime($bill->bill_date));

                    if($report_data['type_of_report']=='GST Report B2B'){
                        $data = [
                            'acc_party_gst' => $bill->acc_party->party->gst_number,
                            'receiver_name' => $bill->acc_party->party->party_name,
                            'invoice_number' => $bill->bill_number,
                            'invoice_date' => $formattedBillDate,
                            'invoice_value' => $bill->grand_total_amount,
                            'place_of_supply' => $place_of_supply,
                            'reverse_charge' => 'N',
                            'tax_rate' => 0,
                            'invoice_type' => 'Regular B2B',
                            'ecommerce_gst' => '',
                            'rate' => $bill->igst_percent+$bill->cgst_percent+$bill->sgst_percent,
                            'taxable_value' => $bill->total_without_gst+$bill->other_plus-$bill->other_minus,
                            'cess_amount' => 0,
                        ];
                    }
                    elseif($report_data['type_of_report']=='GST Report B2C'){
                        $data = [
                            'type' => 'OE',
                            'acc_party_gst' => $bill->acc_party->party->gst_number,
                            'place_of_supply' => $place_of_supply,
                            'tax_rate' => 0,
                            'ecommerce_gst' => '',
                            'rate' => $bill->igst_percent+$bill->cgst_percent+$bill->sgst_percent,
                            'taxable_value' => $bill->total_without_gst+$bill->other_plus-$bill->other_minus,
                            'cess_amount' => 0,
                        ];
                    }
    
                    $result_data[] = $data;
                }
            
            
        }
    

        return response()->json(['data' => $result_data, 'draw' => $request->draw]);
    }


    protected function record_transaction_in_accounts($request, $bill){
        $t_data = [];
        $t_data['t_against'] = 'BILL';
        $t_data['t_against_id'] = $bill->id;
        $t_data['t_against_no'] = $bill->bill_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->bill_date;
        $t_data['voucher_time'] = $request->bill_time;
        $t_data['t_against_narration'] = $bill->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($bill->income && $bill->acc_party_id && $request->total_without_rounding>0){

            $AFTER_TAX = $request->total_green_tax +  $request->total_detention + $request->total_kanta;
           
            $t_data['voucher_accounting_details']=[];

            $twg = $request->total_without_gst + $request->other_plus - $request->other_minus;

            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$bill->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$request->total_without_rounding, 'cr_amount'=>0 ];
            

            if(!$request->is_rcm){
                 // CREDIT SIDE
                if($request->total_cgst_amount > 0){ 
                    // CREDIT CGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->total_cgst_amount ];
                }
                if($request->total_sgst_amount > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->total_sgst_amount ];
                }
                if($request->total_igst_amount > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->total_igst_amount ];
                }

            }
           

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$bill->income, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$twg+$AFTER_TAX ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }



            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 

            

            $this->create_transaction_voucher( $t_data );
        }

        if($bill->selection_type=='GR' || $bill->selection_type=='Loading Slip'){
            // LOOP THROUGH BILL DESC, CHECK selection_type, If 'GR' send gr_number  , If 'Loading Slip' send ls_number
            $bill_desc_ids = BillDesc::select('id')->where('bill_id', $bill->id)->where('fin_year',$request->current_year)->pluck('id')->toArray();
            
            // DELETING CHALLAN DESCRIPTIONS
            foreach ($bill_desc_ids as $pdi) {
                // MAKE GR TRANSACTION INACTIVE
                $bill_desc = BillDesc::find($pdi);

                if($bill_desc->selection_type == 'GR'){
                    

                    if($bill_desc->pod_id){
                        $pod_number = Pod::select('pod_number')->find($bill_desc->pod_id);
                        if($pod_number){
                            $t_data3['t_against'] = 'POD';
                            $t_data3['t_against_no'] = $pod_number['pod_number'];
                            $t_data3['fin_year'] = $request->current_year;
                            $this->set_transaction_status($t_data3, false);
                        }
                    }
                    else{
                        $t_data2['t_against'] = 'GR';
                        $t_data2['t_against_no'] = $bill_desc->gr_number;
                        $t_data2['fin_year'] = $request->current_year;
                        $this->set_transaction_status($t_data2, false);
                    }
                }
                else if($bill_desc->selection_type == 'Loading Slip'){
                    $t_data2['t_against'] = 'Loading Slip';
                    $t_data2['t_against_no'] = $bill_desc->ls_number;
                    $t_data2['fin_year'] = $request->current_year;
                    $this->set_transaction_status($t_data2, false);
                }

                
            }
        }

    }

    


    public function change_bill_status($selection_type, $status , $pk, $included_charges){
        if($selection_type == 'GR'){
            $GR_UPDATE = BookGr::find($pk);
            if($included_charges=='Freight' || $included_charges=='All'){
            $GR_UPDATE->freight_bill = $status;
            }
            if($included_charges=='Way Expenses' || $included_charges=='All'){
            $GR_UPDATE->way_exp_bill = $status;
            }
            $GR_UPDATE->save();
        }
        else if($selection_type == 'Loading Slip'){
            $LS_UPDATE = LoadingSlip::find($pk);
            $LS_UPDATE->bill_status = $status;
            $LS_UPDATE->save();
        }
    }

    
}

