<?php

namespace App\Http\Controllers\Booking;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherDesc;
use App\Models\Booking\BookGr;
use App\Models\Booking\DeliveryOrderDesc;
use App\Models\Booking\EwayGr;
use App\Models\GoodsMove\ChallanDesc;
use App\Models\Booking\GrDesc;
use App\Models\Booking\PackagePosition;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\Master\IssueBooks;
use App\Models\Master\Material;
use App\Models\Master\Party;
use App\Models\Master\Station;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class GrController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'GR');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $grs = BookGr::with([
                    'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name','destination2:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number',
                    'gr_desc',
                    'gr_desc.material' => function ($query) {
                        $query->select('id', 'material_name');
                    },
                    'eway_details',
                    'acc_party' => function ($query) {
                        $query->select('id', 'account_name','account_type');
                    },
                    'acc_party.party:id,account_id',
                    'acc_party_branch:id,branch_name',
                    'consignor_branch:id,branch_name',
                    'consignee_branch:id,branch_name',
                    'driver_payment_from:id,account_name',
                    'created_by:id,username',
                    'delivery_order:id,do_number',
                    'lorry_hire_from:id,account_name',
                    'dr_on_account:id,account_name',
                    'lh_payment_acc1:id,account_name',
                    'lh_payment_acc2:id,account_name',
                    'lh_payment_acc3:id,account_name',
                    'diesel_acc1:id,account_name',
                    'diesel_acc2:id,account_name',

                    ])->where('fin_year',$request->current_year)->get();
        return response()->json($grs);

        // 'lorry_hire_from:id,account_name',
        //             'dr_on_account:id,account_name',
        //             'lh_payment_acc1:id,account_name',
        //             'lh_payment_acc2:id,account_name',
        //             'lh_payment_acc3:id,account_name',
        //             'diesel_acc1:id,account_name',
        //             'diesel_acc2:id,account_name',
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $gr = BookGr::with([
                    'branch:id,branch_name,branch_code,gst_number','source:id,station_name', 'destination:id,station_name','destination2:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number','driver:id,emp_name',
                    'gr_desc',
                    'gr_desc.material' => function ($query) {
                        $query->select('id', 'material_name');
                    },
                    'eway_details',
                    'acc_party' => function ($query) {
                        $query->select('id', 'account_name','account_type');
                    },
                    'acc_party.party:id,account_id',
                    'acc_party_branch:id,branch_name',
                    
                    'consignor_branch:id,branch_name',
                    'consignee_branch:id,branch_name',
                    'created_by:id,username',
                    'driver_payment_from:id,account_name',
                    'pod_desc:id,pod_number,gr_id,pod_id',
                    'delivery_order:id,do_number',
                    'bill_desc:id,bill_id,bill_number,gr_id',
                    'lorry_hire_from:id,account_name',
                    'dr_on_account:id,account_name',
                    'lh_payment_acc1:id,account_name',
                    'lh_payment_acc2:id,account_name',
                    'lh_payment_acc3:id,account_name',
                    'diesel_acc1:id,account_name',
                    'diesel_acc2:id,account_name',
                    ])->find($pk);

        return response()->json(['gr'=>$gr]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'GR');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'gr_type' => 'required',
            'gr_number' => 'required|unique:book_grs,gr_number',
            'source' => 'required',
            'destination' => 'required',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'gr_type.required' => 'Required',
            'gr_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'gr_number.unique' => 'Already Exist !',
            'acc_party.required' => 'required'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE GR TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_gr_number($request->gr_number);
            if($request->gr_type == 'PRINTED'){
                $check = IssueBooks::select('prefix','party_id')->where('prefix',$request->prefix)->where('branch_id', $request->branch['id'])->where('book_type','GR')->where('issue_from', '<=', $max_number)->where('issue_to', '>=', $max_number)->first();
                if(!$check){
                    return response()->json(['message'=>'Book Not Issued To The Branch'], 422);
                }
                if($check->party_id){
                    if($check->party_id != $request->consignor['id']){
                        return response()->json(['message'=>'Book Not Issued To The Consignor'], 422);
                    }
                }
                
            }
            
            
            // SAVING A NEW GR
            $gr = new BookGr();
            $gr->branch_id = $request->branch['id'];
            $gr->gr_type = $request->gr_type;
            $gr->prefix = $request->prefix;
            $gr->max_number = $max_number;
            $gr->gr_number = $request->gr_number;
            $gr->shipment_number = $request->shipment_number;
            $gr->po_no = $request->po_no;
            $gr->do_id = $request->do_id;
            $gr->do_number = $request->do_number;

            $gr->gr_date = $request->gr_date;
            $gr->gr_time = $request->gr_time;
            $gr->extra_chg_label = $request->extra_chg_label;



            $source_id = $this->add_if_not_in_master('Station', $request->source, $request->current_year);
            $gr->source_id = $source_id;
            $gr->source_name = $request->source['station_name'];

            $destination_id = $this->add_if_not_in_master('Station', $request->destination, $request->current_year);
            $gr->destination_id = $destination_id;
            $gr->destination_name = $request->destination['station_name'];

            $destination2_id = $this->add_if_not_in_master('Station', $request->destination2, $request->current_year);
            $gr->destination2_id = $destination2_id;
            $gr->destination2_name = $request->destination2 ? $request->destination2['station_name'] : null;
            
            $gr->tpt_mode = $request->tpt_mode;

            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $gr->vehicle_id = $vehicle_id;
            $gr->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            
            $gr->vehicle_type = $request->vehicle_type;
            $gr->driver_id = $request->driver ? $request->driver['id'] : null;
            $gr->driver_name = $request->driver ? $request->driver['emp_name'] : null;
            $gr->driver_mob_number = $request->driver_mob_number;
            $gr->fssain = $request->fssain;
            $gr->driver_payment_from = $request->driver_payment_from ? $request->driver_payment_from['id'] :null;
            $gr->driver_payment_amt = $request->driver_payment_amt;
            $gr->edd = $request->edd;
            $gr->pay_mode = $request->pay_mode;
               
            if($request->pay_mode == 'TO PAY'){
                $gr->income_account_id = 6;  // TO PAY
            }elseif($request->pay_mode == 'PAID'){
                $gr->income_account_id = 7;  // PAID
            }


            $gr->delivery_pt = $request->delivery_pt;
            $gr->truck_status = $request->truck_status;
            $gr->truck_status_value = $request->truck_status_value;
            $gr->consignor_id = $request->consignor ? $request->consignor['id'] : null;
            $gr->consignor_name = $request->consignor ? $request->consignor['party_name'] : null;
            $gr->consignor_branch_id = $request->consignor_branch ? $request->consignor_branch['id'] : null;
            $gr->consignor_branch_name = $request->consignor_branch ? $request->consignor_branch['branch_name'] : null;
            $gr->consignor_gst_number = $request->consignor_gst_number;
            $gr->consignor_phone_number = $request->consignor_phone_number;
            $gr->consignor_address = $request->consignor_address;

            $gr->consignee_id = $request->consignee ? $request->consignee['id'] : null;
            $gr->consignee_name = $request->consignee ? $request->consignee['party_name'] : null;
            $gr->consignee_branch_id = $request->consignee_branch ? $request->consignee_branch['id'] : null;
            $gr->consignee_branch_name = $request->consignee_branch ? $request->consignee_branch['branch_name'] : null;
            $gr->consignee_gst_number = $request->consignee_gst_number;
            $gr->consignee_phone_number = $request->consignee_phone_number;
            $gr->consignee_address = $request->consignee_address;

            $gr->acc_party_id = $request->acc_party ? $request->acc_party['id'] : null; 
            $gr->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;

            $gr->total_pkg = $request->total_pkg;
            $gr->total_material_weight = $request->total_material_weight;
            $gr->total_km = $request->total_km;
            $gr->total_chg_qty = $request->total_chg_qty;
            $gr->total_desc_value = $request->total_desc_value;
            $gr->total_invoice_value = $request->total_invoice_value;

            $gr->p_marka = $request->p_marka;
            $gr->remarks = $request->remarks;


            $gr->booking_comm_percent = $request->booking_comm_percent;
            $gr->booking_comm_rate = $request->booking_comm_rate;
            $gr->booking_comm_charged_on = $request->booking_comm_charged_on;
            $gr->booking_comm_amount = $request->booking_comm_amount;

            $gr->eway_pic = $request->eway_pic;
            $gr->other_pic = $request->other_pic;

            $gr->print_at_save = $request->print_at_save;

            $gr->lorry_hire_from = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;
            $gr->hire_remarks = $request->hire_remarks;
            $gr->dr_on_account = $request->dr_on_account ? $request->dr_on_account['id'] : null;

            $gr->lh_total_pkg = $request->lh_total_pkg;
            $gr->lh_total_wt = $request->lh_total_wt;
            $gr->lh_unit = $request->lh_unit;
            $gr->lh_rate = $request->lh_rate;
            $gr->lh_freight = $request->lh_freight;
            $gr->lh_extra_point = $request->lh_extra_point;
            $gr->material_cost = $request->material_cost;
            $gr->loading_labour = $request->loading_labour;
            $gr->loading_detention = $request->loading_detention;
            $gr->unloading_labour = $request->unloading_labour;
            $gr->unloading_detention = $request->unloading_detention;
            $gr->dala = $request->dala;
            $gr->bilty_chg = $request->bilty_chg;
            $gr->other_add_label = $request->other_add_label;
            $gr->other_add = $request->other_add;
            $gr->other_minus_label = $request->other_minus_label;
            $gr->other_minus = $request->other_minus;


            $gr->lh_net_freight = $request->lh_net_freight;
            $gr->lh_tds_percent = $request->lh_tds_percent;
            $gr->lh_tds = $request->lh_tds;
            $gr->lh_payment_acc1 = $request->lh_payment_acc1 ? $request->lh_payment_acc1['id'] : null;
            $gr->lh_payment_acc2 = $request->lh_payment_acc2 ? $request->lh_payment_acc2['id'] : null;
            $gr->lh_payment_acc3 = $request->lh_payment_acc3 ? $request->lh_payment_acc3['id'] : null;
            $gr->diesel_acc1 = $request->diesel_acc1 ? $request->diesel_acc1['id'] : null;
            $gr->diesel_acc2 = $request->diesel_acc2 ? $request->diesel_acc2['id'] : null;
            $gr->lh_advance = $request->lh_advance;
            $gr->lh_advance_date = $request->lh_advance_date;
            $gr->lh_payment = $request->lh_payment;
            $gr->lh_payment2 = $request->lh_payment2;
            $gr->diesel_rate1 = $request->diesel_rate1;
            $gr->diesel_rate2 = $request->diesel_rate2;
            $gr->diesel_qty1 = $request->diesel_qty1;
            $gr->diesel_qty2 = $request->diesel_qty2;
            $gr->diesel_amt1 = $request->diesel_amt1;
            $gr->diesel_amt2 = $request->diesel_amt2;
            $gr->lh_balance = $request->lh_balance;


            $gr->freight = $request->freight;
            $gr->gr_chg = $request->gr_chg;
            $gr->disc_percent = $request->disc_percent;
            $gr->disc_amount = $request->disc_amount;
            $gr->payment_dynamic_fields = $request->payment_dynamic_fields;
            $gr->gst_paid_by = $request->gst_paid_by;
            $gr->total_without_gst = $request->total_without_gst;
            $gr->igst_percent = $request->igst_percent;
            $gr->cgst_percent = $request->cgst_percent;
            $gr->sgst_percent = $request->sgst_percent;
            $gr->igst_amount = $request->igst_amount;
            $gr->cgst_amount = $request->cgst_amount;
            $gr->sgst_amount = $request->sgst_amount;
            $gr->green_tax = $request->green_tax;
            $gr->kanta = $request->kanta;
            $gr->detention = $request->detention;

            $gr->unload_rate = $request->unload_rate;
            $gr->unload_amt = $request->unload_amt;
            $gr->toll_amt = $request->toll_amt;
            $gr->diversion_rate = $request->diversion_rate;
            $gr->diversion_amt = $request->diversion_amt;

            $gr->total_without_rounding = $request->total_without_rounding;
            $gr->rounded_off = $request->rounded_off;
            $gr->grand_total = $request->grand_total;

            $gr->freight_bill = false;
            $gr->way_exp_bill = false;

            $gr->fin_year = $request->current_year; 

            $gr->created_by = $request->user->id;

            // Save the model to the database
            $gr->save();


            // SAVING PACKAGE DETAILS
            $pkg_details = $request->pkg_details;
            foreach ($pkg_details as $pd) {
                if(!$pd['chg_unit'] || !$pd['chg_qty'])
                continue;
                
                $pkg_det = new GrDesc();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;

                $material_id = $this->add_if_not_in_master('Material', $pd['material'], $request->current_year);

                $pkg_det->material_id = $material_id;
                $pkg_det->destination_id = $request->destination['id'];
                $pkg_det->packing_method = $pd['packing_method'];
                $pkg_det->pkg = $pd['pkg'];
                $pkg_det->nos = $pd['nos'];
                $pkg_det->material_weight = $pd['material_weight'];
                $pkg_det->km = $pd['km'];
                $pkg_det->chg_unit = $pd['chg_unit'];
                $pkg_det->chg_qty = $pd['chg_qty'];
                $pkg_det->rate = $pd['rate'];
                $pkg_det->total = $pd['total'];
                $pkg_det->invoice_number = $pd['invoice_number'];
                $pkg_det->invoice_date = $pd['invoice_date'];
                $pkg_det->invoice_value = $pd['invoice_value'];
                $pkg_det->payment_dynamic_fields = $pd['payment_dynamic_fields'];
                $pkg_det->fin_year = $request->current_year;

                $pkg_det->free_pkgs = $pd['pkg'];
                $pkg_det->save();


                // SAVING PACKAGE POSITIONS
                $pkg_position = new PackagePosition();
                $pkg_position->gr_id = $gr->id;
                $pkg_position->gr_number = $gr->gr_number;
                $pkg_position->gr_desc_id = $pkg_det->id;
                $pkg_position->position = $request->branch['id'];
                $pkg_position->pkg = $pd['pkg'];
                $pkg_position->origin = $request->branch['id'];
                $pkg_position->save();
                // PACKAGE POSITIONS SAVED
            }
            // PACKAGE DETAILS SAVED

            // SAVING EWAY DETAILS
            $eway_details = $request->eway_details;
            foreach ($eway_details as $ed) {
                if(!$ed['eway_number'])
                continue;

                // $check_unique = EwayGr::where('eway_number',$ed['eway_number'])->exists();
                // if($check_unique){
                //     return response()->json(['error'=>'E-Way in use - '.$ed['eway_number'], 'data'=>$ed['eway_number']],500);
                // }
                
                $pkg_det = new EwayGr();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;
                $pkg_det->eway_number = $ed['eway_number'];
                $pkg_det->eway_exp_date = $ed['eway_exp_date'];
                $pkg_det->fin_year = $request->current_year;
                $pkg_det->save();
            }
            // EWAY DETAILS SAVED

            if($gr->do_id){
                $do_desc_id = DeliveryOrderDesc::select('id','do_id','do_number')->where('do_id',$gr->do_id)->value('id');
                $balance = $this->find_remaining_balance_for_do_desc($do_desc_id);

                if($balance['remaining_qty'] < 0){
                    dd('RR QTY EXCEEDED');
                }
            }

            // RECORD THE TRANSACTION IN ACCOUNTS
            
            $this->record_transaction_in_accounts($request, $gr);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($gr->id);
            $this->createEditLog('created', 'book_grs', $gr->id, $gr->gr_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$gr->id, 'gr_number'=>$gr->gr_number, 'branch'=>json_encode($request->branch)], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'GR');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $gr = BookGr::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($gr->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'gr_type' => 'required',
            'gr_number' => 'required',
            'source' => 'required',
            'destination' => 'required',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'gr_type.required' => 'Required',
            'gr_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'acc_party.required' => 'required'
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            $pod_check = $this->check_in_pod_desc('pod_id','pod_number', 'gr_id', $pk);
            //$pod_check['status'] = false;
            $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'gr_id', $pk);
            $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);
            $voucher_check  = $this->check_in_voucher_desc('voucher_number','voucher_number','GR',$gr->gr_number);
            $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','GR',$gr->gr_number);
            //$diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','GR',$gr->gr_number);
            $diesel_receipt_check['status'] = false;
            $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','GR',$gr->gr_number);

                
            if($pod_check['status'] || $bill_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status'] ){

                $gr->shipment_number = $request->shipment_number;
                $gr->po_no = $request->po_no;
                $gr->save();
                
               $status = $bill_check['status'] ? $bill_check : ($pod_check['status'] ? $pod_check :  ($voucher_check['status'] ? $voucher_check : ($lorry_hire_check['status'] ? $lorry_hire_check : ($diesel_receipt_check['status'] ? $diesel_receipt_check : $driver_settlement_desc_check) )));
               return response()->json(['error'=>'Document in use', 'data'=>$status],500);
            };


            // SAVING A NEW GR

            $gr_number = $request->gr_number;
        
               if($gr->gr_number != $gr_number && stripos($gr->gr_number, 'dummy') !== false){
                $check = BookGr::where('gr_number',$gr_number)->exists();
                if($check){
                    return response()->json(['error'=>'GR Number Already Exist'],500);
                }

                // DELETE THE TRANSACTION VOUCHER
                $t_data = [];
                $t_data['t_against'] = 'GR';
                $t_data['t_against_no'] = $gr->gr_number;
                $t_data['t_against_id'] = $gr->id;
                $t_data['fin_year'] = $request->current_year;

                $this->delete_transaction_voucher($t_data);


                $diesel_recipt = DieselReceipt::where('against','GR')->where('booking_number',$gr->gr_number)->get();
                foreach ($diesel_recipt as $dr) {
                    $dr->booking_number = $gr_number;
                    $dr->save();
                }


                $gr_desc = GrDesc::where('gr_number',$gr->gr_number)->get();
                foreach ($gr_desc as $gd) {
                    $gd->gr_number = $gr_number;
                    $gd->save();
                }

                $eway_desc = EwayGr::where('gr_number',$gr->gr_number)->get();
                foreach ($eway_desc as $ed) {
                    $ed->gr_number = $gr_number;
                    $ed->save();
                }

                $package_position = PackagePosition::where('gr_number',$gr->gr_number)->get();
                foreach ($package_position as $pp) {
                    $pp->gr_number = $gr_number;
                    $pp->save();
                }
              
                $challan_desc = ChallanDesc::where('gr_number',$gr->gr_number)->get();
                foreach ($challan_desc as $gd) {
                    $gd->gr_number = $gr_number;
                    $gd->save();
                }

                $gr->gr_number = $gr_number;

                $gr->max_number = $this->get_max_number_from_gr_number($gr_number);
            }


            $gr->gr_date = $request->gr_date;
            $gr->gr_time = $request->gr_time;
            $gr->extra_chg_label = $request->extra_chg_label;
            $gr->shipment_number = $request->shipment_number;
            $gr->po_no = $request->po_no;
            $gr->do_id = $request->do_id;
            $gr->do_number = $request->do_number;

            $source_id = $this->add_if_not_in_master('Station', $request->source, $request->current_year);
            $gr->source_id = $source_id;
            $gr->source_name = $request->source['station_name'];

            $destination_id = $this->add_if_not_in_master('Station', $request->destination, $request->current_year);
            $gr->destination_id = $destination_id;
            $gr->destination_name = $request->destination['station_name'];

            $destination2_id = $this->add_if_not_in_master('Station', $request->destination2, $request->current_year);
            $gr->destination2_id = $destination2_id;
            $gr->destination2_name = $request->destination2 ? $request->destination2['station_name'] : null;
            $gr->truck_status = $request->truck_status;
            $gr->truck_status_value = $request->truck_status_value;
            $gr->tpt_mode = $request->tpt_mode;

            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $gr->vehicle_id = $vehicle_id;
            $gr->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $gr->vehicle_type = $request->vehicle_type;
            $gr->driver_id = $request->driver ? $request->driver['id'] : null;
            $gr->driver_name = $request->driver ? $request->driver['emp_name'] : null;
            $gr->driver_mob_number = $request->driver_mob_number;
            $gr->fssain = $request->fssain;
            $gr->driver_payment_from = $request->driver_payment_from ? $request->driver_payment_from['id'] :null;
            $gr->driver_payment_amt = $request->driver_payment_amt;
            $gr->edd = $request->edd;
            $gr->pay_mode = $request->pay_mode;

            if($request->pay_mode == 'TO PAY'){
                $gr->income_account_id = 6;  // TO PAY
            }elseif($request->pay_mode == 'PAID'){
                $gr->income_account_id = 7;  // PAID
            }

            $gr->delivery_pt = $request->delivery_pt;

            $gr->consignor_id = $request->consignor ? $request->consignor['id'] : null;
            $gr->consignor_name = $request->consignor ? $request->consignor['party_name'] : null;
            $gr->consignor_branch_id = $request->consignor_branch ? $request->consignor_branch['id'] : null;
            $gr->consignor_branch_name = $request->consignor_branch ? $request->consignor_branch['branch_name'] : null;
            $gr->consignor_gst_number = $request->consignor_gst_number;
            $gr->consignor_phone_number = $request->consignor_phone_number;
            $gr->consignor_address = $request->consignor_address;

            $gr->consignee_id = $request->consignee ? $request->consignee['id'] : null;
            $gr->consignee_name = $request->consignee ? $request->consignee['party_name'] : null;
            $gr->consignee_branch_id = $request->consignee_branch ? $request->consignee_branch['id'] : null;
            $gr->consignee_branch_name = $request->consignee_branch ? $request->consignee_branch['branch_name'] : null;
            $gr->consignee_gst_number = $request->consignee_gst_number;
            $gr->consignee_phone_number = $request->consignee_phone_number;
            $gr->consignee_address = $request->consignee_address;

            $gr->acc_party_id = $request->acc_party ? $request->acc_party['id'] : null; 
            $gr->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;

            if(!$challan_check['status']){
            $gr->total_pkg = $request->total_pkg;
            }
            $gr->total_material_weight = $request->total_material_weight;

            $gr->total_km = $request->total_km;
            $gr->total_chg_qty = $request->total_chg_qty;
            $gr->total_desc_value = $request->total_desc_value;
            $gr->total_invoice_value = $request->total_invoice_value;

            $gr->p_marka = $request->p_marka;
            $gr->remarks = $request->remarks;

            $gr->booking_comm_percent = $request->booking_comm_percent;
            $gr->booking_comm_rate = $request->booking_comm_rate;
            $gr->booking_comm_charged_on = $request->booking_comm_charged_on;
            $gr->booking_comm_amount = $request->booking_comm_amount;

            $gr->eway_pic = $request->eway_pic;
            $gr->other_pic = $request->other_pic;


            $gr->print_at_save = $request->print_at_save;

            $gr->lorry_hire_from = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;
            $gr->hire_remarks = $request->hire_remarks;

            $gr->dr_on_account = $request->dr_on_account ? $request->dr_on_account['id'] : null;


            $gr->lh_total_pkg = $request->lh_total_pkg;
            $gr->lh_total_wt = $request->lh_total_wt;
            $gr->lh_unit = $request->lh_unit;
            $gr->lh_rate = $request->lh_rate;
            $gr->lh_freight = $request->lh_freight;
            $gr->lh_extra_point = $request->lh_extra_point;
            $gr->material_cost = $request->material_cost;
            $gr->loading_labour = $request->loading_labour;
            $gr->loading_detention = $request->loading_detention;
            $gr->unloading_labour = $request->unloading_labour;
            $gr->unloading_detention = $request->unloading_detention;
            $gr->dala = $request->dala;
            $gr->bilty_chg = $request->bilty_chg;
            $gr->other_add_label = $request->other_add_label;
            $gr->other_add = $request->other_add;
            $gr->other_minus_label = $request->other_minus_label;
            $gr->other_minus = $request->other_minus;
            $gr->lh_net_freight = $request->lh_net_freight;
            $gr->lh_tds_percent = $request->lh_tds_percent;
            $gr->lh_tds = $request->lh_tds;
            $gr->lh_payment_acc1 = $request->lh_payment_acc1 ? $request->lh_payment_acc1['id'] : null;
            $gr->lh_payment_acc2 = $request->lh_payment_acc2 ? $request->lh_payment_acc2['id'] : null;
            $gr->lh_payment_acc3 = $request->lh_payment_acc3 ? $request->lh_payment_acc3['id'] : null;
            $gr->diesel_acc1 = $request->diesel_acc1 ? $request->diesel_acc1['id'] : null;
            $gr->diesel_acc2 = $request->diesel_acc2 ? $request->diesel_acc2['id'] : null;
            $gr->lh_advance = $request->lh_advance;
            $gr->lh_advance_date = $request->lh_advance_date;
            $gr->lh_payment = $request->lh_payment;
            $gr->lh_payment2 = $request->lh_payment2;
            $gr->diesel_rate1 = $request->diesel_rate1;
            $gr->diesel_rate2 = $request->diesel_rate2;
            $gr->diesel_qty1 = $request->diesel_qty1;
            $gr->diesel_qty2 = $request->diesel_qty2;
            $gr->diesel_amt1 = $request->diesel_amt1;
            $gr->diesel_amt2 = $request->diesel_amt2;
            $gr->lh_balance = $request->lh_balance;


            $gr->freight = $request->freight;
            $gr->gr_chg = $request->gr_chg;
            $gr->disc_percent = $request->disc_percent;
            $gr->disc_amount = $request->disc_amount;
            $gr->payment_dynamic_fields = $request->payment_dynamic_fields;
            $gr->gst_paid_by = $request->gst_paid_by;
            $gr->total_without_gst = $request->total_without_gst;
            $gr->igst_percent = $request->igst_percent;
            $gr->cgst_percent = $request->cgst_percent;
            $gr->sgst_percent = $request->sgst_percent;
            $gr->igst_amount = $request->igst_amount;
            $gr->cgst_amount = $request->cgst_amount;
            $gr->sgst_amount = $request->sgst_amount;
            $gr->green_tax = $request->green_tax;
            $gr->kanta = $request->kanta;
            $gr->detention = $request->detention;

            $gr->unload_rate = $request->unload_rate;
            $gr->unload_amt = $request->unload_amt;
            $gr->toll_amt = $request->toll_amt;
            $gr->diversion_rate = $request->diversion_rate;
            $gr->diversion_amt = $request->diversion_amt;

            $gr->total_without_rounding = $request->total_without_rounding;
            $gr->rounded_off = $request->rounded_off;
            $gr->grand_total = $request->grand_total;

            $gr->updated_by = $request->user->id;

            $gr->save();
            // NEW PARTY SAVED

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $gr);
            // TRANSACTION RECORDED IN ACCOUNTS

            
            // SAVING NEWLY ADDED  EWAY DETAILS
            $newly_added_eway_details = $request->newly_added_eway_details;
            foreach ($newly_added_eway_details as $naed) {
                if(!$naed['eway_number'])
                continue;

                // $check_unique = EwayGr::where('eway_number',$naed['eway_number'])->where('fin_year',$request->current_year)->exists();
                // if($check_unique){
                //     return response()->json(['error'=>'E-Way in use - '.$naed['eway_number'], 'data'=>$naed['eway_number']],500);
                // }
                
                $pkg_det = new EwayGr();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;
                $pkg_det->eway_number = $naed['eway_number'];
                $pkg_det->eway_exp_date = $naed['eway_exp_date'];
                $pkg_det->fin_year = $request->current_year;
                $pkg_det->save();
            }
            // NEWLY ADDED  EWAY DETAILS SAVED

            // SAVING UPDATES ON EXISTING   EWAY DETAILS
            $updated_eway_details = $request->updated_eway_details;
            if( !empty($updated_eway_details) ){
                foreach ($updated_eway_details as $ued) {
                    if(!$ued['eway_number'])
                    continue;

                    // $check_unique = EwayGr::where('eway_number',$ued['eway_number'])->where('gr_id','!=',$gr->id)->where('fin_year',$request->current_year)->exists();
                    // if($check_unique){
                    //     return response()->json(['error'=>'E-Way in use - '.$ued['eway_number'], 'data'=>$ued['eway_number']],500);
                    // }
                    
                    $pkg_det = EwayGr::find( $ued['id'] );
                    $pkg_det->eway_number = $ued['eway_number'];
                    $pkg_det->eway_exp_date = $ued['eway_exp_date'];
                    $pkg_det->save();
                }
            }
            // UPDATES SAVED ON EXISTING  EWAY DETAILS

            // DELETING EXISTING  EWAY DETAILS
            $deleted_eway_details = $request->deleted_eway_details;
            if( !empty($deleted_eway_details) ){
                    $pkg_det = EwayGr::whereIn('id', $deleted_eway_details);
                    $pkg_det->delete();
            }
            //DELETED EXISTING  EWAY DETAILS
            

           
            
            // SAVING NEWLY ADDED PACKAGE DETAILS
            if(!$challan_check['status']){

                $newly_added_pkg_details = $request->newly_added_pkg_details;
                foreach ($newly_added_pkg_details as $napd) {
                    if( !$napd['chg_unit'] || !$napd['chg_qty'])
                    continue;
                    
                    $pkg_det = new GrDesc();
                    $pkg_det->gr_id = $gr->id;
                    $pkg_det->gr_number = $gr->gr_number;

                    $material_id = $this->add_if_not_in_master('Material', $napd['material'], $request->current_year);

                    $pkg_det->material_id = $material_id;

                    $pkg_det->destination_id = $request->destination['id'];
                    $pkg_det->packing_method = $napd['packing_method'];
                    $pkg_det->pkg = $napd['pkg'];
                    $pkg_det->nos = $napd['nos'];
                    $pkg_det->material_weight = $napd['material_weight'];
                    $pkg_det->km = $napd['km'];
                    $pkg_det->chg_unit = $napd['chg_unit'];
                    $pkg_det->chg_qty = $napd['chg_qty'];
                    $pkg_det->rate = $napd['rate'];
                    $pkg_det->total = $napd['total'];
                    $pkg_det->invoice_number = $napd['invoice_number'];
                    $pkg_det->invoice_date = $napd['invoice_date'];
                    $pkg_det->invoice_value = $napd['invoice_value'];
                    $pkg_det->payment_dynamic_fields = $napd['payment_dynamic_fields'];
                    
                    $pkg_det->fin_year = $request->current_year;

                    $pkg_det->free_pkgs = $napd['pkg'];
                    $pkg_det->save();

                    // SAVING PACKAGE POSITIONS
                    $pkg_position = new PackagePosition();
                    $pkg_position->gr_id = $gr->id;
                    $pkg_position->gr_number = $gr->gr_number;
                    $pkg_position->gr_desc_id = $pkg_det->id;
                    $pkg_position->position = $request->branch['id'];
                    $pkg_position->pkg = $napd['pkg'];
                    $pkg_position->origin = $request->branch['id'];
                    $pkg_position->save();
                    // PACKAGE POSITIONS SAVED
                }

            }
            // NEWLY ADDED PACKAGE DETAILS SAVED

            // SAVING UPDATES ON EXISTING  PACKAGE DETAILS
            $updated_pkg_details = $request->updated_pkg_details;
            if( !empty($updated_pkg_details) ){
                foreach ($updated_pkg_details as $upd) {
                    if(!$upd['chg_unit'] || !$upd['chg_qty'])
                    continue;
                       
                        $pkg_det = GrDesc::find( $upd['id'] );

                        if(!$challan_check['status']){
    
                            $pkg_det->pkg = $upd['pkg'];
                        }


                        $material_id = $this->add_if_not_in_master('Material', $upd['material'], $request->current_year);

                        $pkg_det->material_id = $material_id;
                        $pkg_det->destination_id = $request->destination['id'];
                        $pkg_det->material_weight = $upd['material_weight'];


                        $pkg_det->packing_method = $upd['packing_method'];
                        $pkg_det->nos = $upd['nos'];
                        $pkg_det->km = $upd['km'];

                        $pkg_det->chg_unit = $upd['chg_unit'];
                        $pkg_det->chg_qty = $upd['chg_qty'];
                        $pkg_det->rate = $upd['rate'];
                        $pkg_det->total = $upd['total'];
                        $pkg_det->invoice_number = $upd['invoice_number'];
                        $pkg_det->invoice_date = $upd['invoice_date'];
                        $pkg_det->invoice_value = $upd['invoice_value'];
                        $pkg_det->free_pkgs = $upd['pkg'];
                        $pkg_det->payment_dynamic_fields = $upd['payment_dynamic_fields'];
                        $pkg_det->save();

                        if(!$challan_check['status']){
                        // UPDATING PACKAGE POSITIONS
                        $pkg_position = PackagePosition::where('gr_desc_id', $pkg_det->id)->first();
                        $pkg_position->pkg = $upd['pkg'];
                        $pkg_position->save();
                        }
                        // UPDATING POSITIONS SAVED
                }
            }
            // UPDATES SAVED ON EXISTING PACKAGE DETAILS

            // DELETING EXISTING PACKAGE DETAILS
            if(!$challan_check['status']){
                $deleted_pkg_details = $request->deleted_pkg_details;
                if( !empty($deleted_pkg_details) ){
                        $pkg_det = GrDesc::whereIn('id', $deleted_pkg_details);

                        // DELETING PACKAGE POSITIONS
                        $pkg_position = PackagePosition::whereIn('gr_desc_id', $deleted_pkg_details);
                        $pkg_position->delete();

                        $pkg_det->delete();
                        
                }
            }
            //DELETED EXISTING PACKAGE DETAILS


            // IF EXISTS IN CHALLAN, SIMPLY 
            $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);

            if($challan_check['status']){
                DB::commit();

                $status= $challan_check; 
                return response()->json(['error'=>'Document in use', 'data'=>['message'=>'Challan '.$status['index'].' Restricted PKG & Wt Update.']],200);
            };
    

            if($gr->do_id){
                $do_desc_id = DeliveryOrderDesc::select('id','do_id','do_number')->where('do_id',$gr->do_id)->value('id');
                $balance = $this->find_remaining_balance_for_do_desc($do_desc_id);

                if($balance['remaining_qty'] < 0){
                    dd('RR QTY EXCEEDED');
                }
            }

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($gr->id);
            $this->createEditLog('updated', 'book_grs', $gr->id, $gr->gr_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'GR');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        
    try {
        // TRANSACTION BEGINS
        DB::beginTransaction();
    

        $gr = BookGr::find($pk);

        $pod_check = $this->check_in_pod_desc('pod_id','pod_number', 'gr_id', $pk);
        $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'gr_id', $pk);
        $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);
        $voucher_check  = $this->check_in_voucher_desc('voucher_number','voucher_number','GR',$gr->gr_number);
        $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','GR',$gr->gr_number);
        $diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','GR',$gr->gr_number);
        $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','GR',$gr->gr_number);
            
        if($pod_check['status'] || $bill_check['status'] || $challan_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status'] ){
           $status = $bill_check['status'] ? $bill_check : ($pod_check['status'] ? $pod_check : ($challan_check['status'] ? $challan_check : ($voucher_check['status'] ? $voucher_check : ($lorry_hire_check['status'] ? $lorry_hire_check : ($diesel_receipt_check['status'] ? $diesel_receipt_check : $driver_settlement_desc_check) ))));
           return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };
        
        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($gr->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        //FIND AND DELETE THE PACKAGE POSITIONS
        $pkg_positions = PackagePosition::where('gr_id', $pk)->delete();
        $gr_desc_ids = GrDesc::where('gr_id', $pk)->delete();

        //FIND AND DELETE THE BRANCH
        $gr->delete();
        // COMMISSIONS WILL BE CASCADED AUTOMATICALLY
        
        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'GR';
        $t_data['t_against_no'] = $gr->gr_number;
        $t_data['t_against_id'] = $gr->id;
        $t_data['fin_year'] = $request->current_year;

        $this->delete_transaction_voucher($t_data);

        $this->createEditLog('deleted', 'book_grs', $gr->id, $gr->gr_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

        // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function cancel_gr(Request $request, $pk){
        $test = $this->is_edit_permitted($request->user->id, 'GR');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
        

            $gr = BookGr::find($pk);

            if( $edit_valid_till_hours!=null ){
                $validity = Carbon::parse($gr->created_at)->addHours($edit_valid_till_hours);
                if( !$validity->isFuture() ){
                    return response()->json(['Permission Timed Out'],500);
                }
            }

            $challan_check  = $this->check_in_challan_desc('challan_number','challan_number','gr_id',$pk);
                
            if($challan_check['status']){
                DB::commit();

                $status= $challan_check; 
                return response()->json(['error'=>'Document in use', 'data'=>['message'=>'Challan '.$status['index'].' Restricted Desc.']],500);
            };
            

            $gr->is_cancelled = !$gr->is_cancelled;

            $gr->save();
            
            // DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'GR';
            $t_data['t_against_no'] = $gr->gr_number;
            $t_data['fin_year'] = $request->current_year;

            if($gr->is_cancelled){
                $this->set_transaction_status($t_data, false);;
            }else{
                // RECORD THE TRANSACTION IN ACCOUNTS
                $this->set_transaction_status($t_data, true);;
                // TRANSACTION RECORDED IN ACCOUNTS
            }

            $newVal = $this->get_data_for_obj($gr->id);
            $this->createEditLog('updated', 'book_grs', $gr->id, $gr->gr_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
                
                // TRANSACTIONS COMMITTED
                DB::commit();
                return response()->json(['status'=>$gr->is_cancelled], 200); // RETURNED RESPONSE

            } catch (\Exception $e) {
                DB::rollback();
                return response()->json(['error' => $e->getMessage()], 500);
            }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = BookGr::select('id', 'gr_number','vehicle_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }
        elseif( $request->branch_id ){
            $query->where('branch_id', $request->branch_id );
        }
        
        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $query->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }

       // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];
            $whereInFilters = isset($request->filterby['whereIn']) ? $request->filterby['whereIn'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }

        foreach ($whereInFilters as $whereInFilter) {
            if(isset($whereInFilter['key']) && isset($whereInFilter['values'])){
                $query->whereIn($whereInFilter['key'], $whereInFilter['values']);
            }
        }  
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%')
            ->orWhere('vehicle_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $limit = 25;

        $query->take($limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $grs = $query->get();
        return response(['grs' => $grs, 'draw' => $request->draw]);
    }

    public function autocomplete_gr_for_pod(Request $request) {
        // Initialize the query builder
        $query = BookGr::select('id', 'gr_number')->where('is_cancelled',false)->whereHas('package_position', function($query) use ($request){
           // $query->where('pkg','>',0)->where('is_delivered',0);
           $query->where('is_delivered',0);
        });
    
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%');
        }
    
        // Apply a custom where clause to filter records where 'pod_desc' does not exist
        $query->whereDoesntHave('pod_desc');
    
        // Limit the number of results
        $query->take(40)->orderBy('max_number', 'asc');
    
        // Retrieve the filtered records
        $grs = $query->get();
    
        return response(['grs' => $grs]);
    }
    
    public function autocomplete_gr_for_challan(Request $request){
        // Initialize the query builder
        $currentYear = $request->current_year;

        $query = PackagePosition::select('gr_id as id', 'gr_number')->whereHas('bookGr', function($query) use ($currentYear){
            $query->where('is_cancelled',false);
        });

        $selected_branch_ids = $request->selected_branch_ids;
        
        $query->whereIn('position', $selected_branch_ids )->where('pkg','>',0)->where('is_delivered',0)->distinct();
        
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $grs = $query->get();
        return response(['grs' => $grs, 'draw' => $request->draw]);
    }

    public function autocomplete_gr_for_eway(Request $request){
        // Initialize the query builder
        $query = EwayGr::select('gr_id as id','gr_number')->where('fin_year',$request->current_year)->where('is_delivered',0)->distinct();

        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('gr_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $limit = $request->limit ? $request->limit : 10;

        $query->take($limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $grs = $query->get();
        return response(['grs' => $grs, 'draw' => $request->draw]);
    }
    

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'GR');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = BookGr::select('id', 'gr_number', 'grand_total', 'gr_date','gr_time' , 'branch_id','source_id', 'destination_id', 'consignor_id', 'consignee_id')
                ->with(['branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'destination2:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name']) 
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_issued_gr(Request $request){
        $gr_number = $request->gr_number;
        $branch_id = $request->branch_id;
        $gr_number = $request->gr_number;
        $prefix = $request->prefix;

        if(!$prefix){
            $prefix = null;
        }

        // Break GR NUMBER BASED ON ANY Separater
        $gr_number = $this->get_max_number_from_gr_number($gr_number);

        $check = IssueBooks::with(['branch' => function ($query) {
                        $query->select('id', 'branch_name','branch_code','gst_number');
                    },
                    'party' => function ($query) {
                        $query->select('id', 'party_name','party_code','account_id');
                    }
                ])
                ->where('book_type', 'GR')
                ->where('prefix', $prefix)
                ->where('branch_id', $branch_id)
                ->where('issue_from', '<=', $gr_number)
                ->where('issue_to', '>=', $gr_number)
                ->first();

        if($check){
            return response()->json(['success'=>true, 'msg'=>'Issued Correctly', 'data'=>$check],200);
        }
        else{

            $check = IssueBooks::with(['branch' => function ($query) {
                            $query->select('id', 'branch_name','branch_code','gst_number');
                        },
                        'party' => function ($query) {
                            $query->select('id', 'party_name','party_code','account_id');
                        }
                    ])
                    ->where('book_type', 'GR')
                    ->where('prefix', $prefix)
                    ->where('issue_from', '<=', $gr_number)
                    ->where('issue_to', '>=', $gr_number)
                    ->first();
            if($check){
                return response()->json(['success'=>true, 'msg'=>'Not Issued To Correct Branch', 'data'=>$check],200);
            }
            else{
                return response()->json(['success'=>false, 'msg'=>'Not Issued'],200);
            }

        }

    

    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = BookGr::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {

        $grsQuery = BookGr::select('id', 'gr_number')->where('fin_year',$request->current_year);
         
        if ( !empty($request->user_branch_ids) ) {
            $grsQuery->whereIn('branch_id', $request->user_branch_ids );
        }
        elseif( $request->branch_id ){
            $grsQuery->where('branch_id', $request->branch_id );
        }



        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $grsQuery->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }
        
        $grs = $grsQuery->orderBy('id', 'desc')->take(13)->get();
        
        return response()->json(['grs' => $grs]);
    }

    public function track_gr(Request $request , $pk){

        $data = BookGr::select('id','gr_number','gr_date', 'consignor_id','consignee_id','source_id','vehicle_id','vehicle_number','vehicle_type','destination_id','consignor_address','consignee_address','remarks','delivery_pt','pay_mode','other_pic')
        ->with([
             'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number,owner_id,self_market','driver:id,emp_name',
             'vehicle.party:id,party_name,mob_number,pan_number',
             'gr_desc',
             'gr_desc.material' => function ($query) {
                 $query->select('id', 'material_name');
             },
             'eway_details',
             'acc_party' => function ($query) {
                 $query->select('id', 'account_name','account_type');
             },
             'challan_desc:id,challan_number,challan_id,gr_id,loaded_pkgs',
             'challan_desc.challan:id,challan_number,challan_date,challan_time,source_id,destination_id,vehicle_id,vehicle_number,remarks',
             'challan_desc.challan.source:id,station_name',
             'challan_desc.challan.destination:id,station_name',
             'challan_desc.challan.vehicle:id,vehicle_number',
             'challan_desc.challan_rec_desc:id,challan_desc_id,gr_id,challan_receipt_id,challan_rec_number',
             'challan_desc.challan_rec_desc.challan_receipt:id,challan_rec_number,arrival_date,arrival_time,remarks',
             'pod_desc:id,gr_id,pod_id,pod_number',
             'pod_desc.pod:id,pod_number,delivery_date,delivery_time,remarks,pod_pic',
             'bill_desc:id,gr_id,bill_number,bill_id',
             'bill_desc.bill:id,bill_number,bill_date',
             'bill_desc.bill.cn_desc',
             'bill_desc.bill.cn_desc.credit_note:id,cn_number,cn_date',
             'package_position:id,gr_id,gr_number,position,pkg,is_delivered',
             'package_position.branch:id,branch_name,branch_code',
             'diesel_receipt:id,dr_number,booking_date,booking_id,against,qty,rate,amount,cash_amount',
             'driver_settlement_desc',
             'driver_settlement_desc.driver_settlement',
             'driver_settlement_desc.driver_settlement.vehicle:id,vehicle_number',
             'driver_settlement_desc.driver_settlement.driver:id,emp_name',
        ])
        ->find($pk);


        $v_query = VoucherDesc::select('id', 'voucher_id', 'voucher_number', 'against_number', 'against_id', 'against', 'receipt_amt', 'voucher_name')
                    ->where('fin_year', $request->current_year);

                // Add 'GR' condition if $data['id'] is set
                if (isset($data['id'])) {
                    $v_query->where(function ($qr) use ($data) {
                        $qr->where('against', 'GR')->where('against_id', $data['id']);
                    });
                }

                // Add 'POD' condition if $data['pod_desc']['pod_id'] is set
                if (isset($data['pod_desc']['pod_id'])) {
                    $v_query->orWhere(function ($qr) use ($data) {
                        $qr->where('against', 'POD')->where('against_id', $data['pod_desc']['pod_id']);
                    });
                }

                // Add 'Bill' condition if $data['bill_desc']['bill_id'] is set
                if (isset($data['bill_desc']) && count($data['bill_desc'])) {
                    $v_query->orWhere(function ($qr) use ($data) {
                        // Extract 'bill_id' from each object in $data['bill_desc']
                        $bill_ids = collect($data['bill_desc'])->pluck('bill_id')->toArray();
                        $bill_nos = collect($data['bill_desc'])->pluck('bill_number')->toArray();

                        // Add condition to the query
                        $qr->where('against', 'Bill')->whereIn('against_id', $bill_ids)->whereIn('against_number', $bill_nos);
                        
                    });
                }

                $voucher_desc = $v_query->get();



        $data['voucher_desc'] = $voucher_desc;


        return response($data);

    }


    public function get_next_gr_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        $user_power = $request->user->power;
        $user_id = $request->user->id;

        if($type=='SYSTEM'){
            $max = BookGr::where('branch_id',$branch_id)->where('gr_type', $type)->max('max_number');
            $gr_number = $branch_code.'-'.sprintf('%06d', $max+1);
            
            return response()->json(['gr_number' => strtoupper($gr_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = BookGr::where('prefix', $prefix)
                    ->where('gr_type', $type)
                    ->where('branch_id', $branch_id)
                    ->max('max_number');

            if(!$user_power){
                $last_by_user = BookGr::where('prefix', $prefix)
                            ->where('gr_type', $type)
                            ->where('branch_id', $branch_id)
                            ->where('created_by', $user_id)
                            ->orderBy('id','desc')
                            ->first();
                if($last_by_user){
                    $max = $last_by_user->max_number;
                }
            }

            

            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id','issue_to','separator_key')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','GR')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                
                $separator = $check->separator_key;
                $issue_to_length = strlen((string)$check->issue_to);
                $padded_max = str_pad($max + 1, $issue_to_length, '0', STR_PAD_LEFT);

                if($max && $check){
                    $gr_number = $prefix ? $prefix.$separator.$padded_max : ($max+1);
                    return response()->json(['gr_number' => strtoupper($gr_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from','issue_to','separator_key')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','GR')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $separator = $check->separator_key;
                $issue_to_length = strlen((string)$check->issue_to);
                $padded_max = str_pad($check->issue_from, $issue_to_length, '0', STR_PAD_LEFT);

                $gr_number = $prefix ? $prefix.$separator.$padded_max: $check->issue_from;
                return response()->json(['gr_number' => strtoupper($gr_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['gr_number' => ['No Valid GR Number']]],422);
            }
        }
    }

    public function fetch_gr_desc_for_challan(Request $request){
        $gr_selection = $request->gr_selection;
        $branch = $request->branch;

        $selected_branch_ids = $request->selected_branch_ids;
        
        $type = $gr_selection['GRSelectionType'];

        if($type == 'GR WISE'){
            // GET DESCS ALONG WITH THEIR GRS
            $gr_descs = GrDesc::with('material:id,material_name','bookGr:id,branch_id,do_id,gr_number,gr_date,consignor_id,consignee_id,vehicle_id,driver_id,vehicle_type,source_id,destination_id,destination_name,consignor_name,consignee_name,grand_total,pay_mode','bookGr.delivery_order','bookGr.delivery_order.do_desc','bookGr.consignor:id,party_name','bookGr.consignee:id,party_name','bookGr.destination:id,station_name', 'bookGr.source:id,station_name', 'bookGr.vehicle:id,vehicle_number','bookGr.driver:id,emp_name')
                       ->where('gr_id', $gr_selection['gr_number']['id'])
                       ->where('free_pkgs','>',0)
                       ->whereHas('bookGr', function ($query) {
                        $query->where('is_cancelled', false);
                       })
                       ->get();
        }
        else if( $type == 'STATION WISE' ){
            $gr_descs = GrDesc::with('material:id,material_name','bookGr:id,branch_id,gr_number,gr_date,consignor_id,consignee_id,vehicle_id,driver_id,vehicle_type,source_id,destination_id,destination_name,consignor_name,consignee_name,grand_total,pay_mode','bookGr.consignor:id,party_name','bookGr.delivery_order','bookGr.delivery_order.do_desc','bookGr.consignee:id,party_name','bookGr.destination:id,station_name', 'bookGr.source:id,station_name', 'bookGr.vehicle:id,vehicle_number','bookGr.driver:id,emp_name')
                       ->where('destination_id', $gr_selection['station']['id'])
                       ->where('free_pkgs','>',0)
                       ->whereHas('bookGr', function ($query) {
                        $query->where('is_cancelled', false);
                       })
                       ->get();
        }
        else if( $type =='ALL' ){
            // GET ALL GR DESCS WHERE free_pks > 0 
            $gr_descs = GrDesc::with('material:id,material_name','bookGr:id,branch_id,gr_number,gr_date,consignor_id,consignee_id,vehicle_id,driver_id,vehicle_type,source_id,destination_id,destination_name,consignor_name,consignee_name,grand_total,pay_mode','bookGr.consignor:id,party_name','bookGr.delivery_order','bookGr.delivery_order.do_desc','bookGr.consignee:id,party_name','bookGr.destination:id,station_name', 'bookGr.source:id,station_name', 'bookGr.vehicle:id,vehicle_number','bookGr.driver:id,emp_name')
                       ->where('free_pkgs','>',0)
                       ->whereHas('bookGr', function ($query) {
                        $query->where('is_cancelled', false);
                       })
                       ->take($gr_selection['to'])
                       ->get();
        }

        $gr_descs->transform(function ($gr_desc) use ($selected_branch_ids) {
            $free = PackagePosition::where('gr_desc_id', $gr_desc->id)->whereIn('position', $selected_branch_ids)->sum('pkg');
            $diesel_amount = DieselReceipt::where('booking_id', $gr_desc->gr_id)->where('against','GR')->sum('amount');
            return [
                'branch_id' => $gr_desc->bookGr->branch_id,
                'challan_desc_id' => null,
                'gr_id' => $gr_desc->gr_id,
                'gr_number' => $gr_desc->gr_number,
                'gr_date' => $gr_desc->bookGr->gr_date,
                'challan_id' => null,
                'challan_number' => null,
                'gr_desc_id' => $gr_desc->id,
                'pkg' => $gr_desc->pkg,
                'wt' => $gr_desc->material_weight,
                'km'=> $gr_desc->km,
                'diesel_amount' => $diesel_amount,
                'free_pkgs' => $free,
                'loaded_pkgs' => $free,
                'material' => $gr_desc->material['material_name'],
                'consignor' => $gr_desc->bookGr->consignor_id ? $gr_desc->bookGr->consignor['party_name'] : $gr_desc->bookGr->consignor_name,
                'consignee' => $gr_desc->bookGr->consignee_id ? $gr_desc->bookGr->consignee['party_name'] : $gr_desc->bookGr->consignee_name,
                'destination' => $gr_desc->bookGr->destination_id ?  $gr_desc->bookGr->destination['station_name'] : $gr_desc->bookGr->destination_name,
                'grand_total' => $gr_desc->bookGr->grand_total,
                'pay_mode' => $gr_desc->bookGr->pay_mode, 
                'source_obj' => $gr_desc->bookGr->source,
                'destination_obj' => $gr_desc->bookGr->destination,
                'vehicle_obj' => $gr_desc->bookGr->vehicle,
                'vehicle_type' => $gr_desc->bookGr->vehicle_type,
                'driver_obj' => $gr_desc->bookGr->driver,
                'do_desc' =>  $gr_desc->bookGr->delivery_order ? $gr_desc->bookGr->delivery_order->do_desc ? $gr_desc->bookGr->delivery_order->do_desc[0] : null : null,
                'checked' => true,
            ];
        });
        return response()->json(['gr_descs' => $gr_descs]);
        
    }

    public function fetch_gr_details_for_pod(Request $request, $pk){
        $gr = BookGr::with([
                    'branch:id,branch_name,branch_code',
                    'acc_party' => function ($query) {
                        $query->select('id', 'account_name','account_type');
                    },
                    'acc_party.party:id,account_id',
                    'acc_party_branch:id,branch_name',
                    'vehicle:id,vehicle_number,vehicle_type',
                    'consignor:id,party_name',
                    'consignee:id,party_name',
                    'source:id,station_name',
                    'destination:id,station_name',
                    'destination2:id,station_name',
                    'gr_desc',   
                    'challan_desc',
                    'challan_desc.challan:id,challan_number,lorry_hire_from',
                    'challan_desc.challan.lorry_hire_from:id,account_name,account_type'

              ])
              ->find($pk);
        
        $data = [
                'pod_desc_id' => null,
                'pod_id' => null,
                'branch' => $gr->branch,
                'gr_id' => $gr->id,
                'gr_number' => ['id'=> $gr->id, 'gr_number'=>$gr->gr_number],
                'gr_date' => $gr->gr_date,
                'source' => $gr->source_id ? $gr->source : ['id'=> null, 'station_name'=>$gr->source_name] ,
                'destination' => $gr->destination_id ? $gr->destination : ['id'=> null, 'station_name'=>$gr->destination_name] ,
                'consignor' => $gr->consignor_id ? $gr->consignor : ['id'=> null, 'party_name'=>$gr->consignor_name] ,
                'consignee' => $gr->consignee_id ? $gr->consignee : ['id'=> null, 'party_name'=>$gr->consignee_name] ,
                'total_pkg' => $gr->total_pkg,
                'total_km' => $gr->total_km, 
                'po_no' => $gr->po_no,
                'shipment_number' => $gr->shipment_number, 
                'total_material_weight' => $gr->total_material_weight,
                'chg_unit' => '', // Add your logic for getting the chg_unit here
                'total_chg_qty' => $gr->total_chg_qty,
                'rate' => $gr->gr_desc ? $gr->gr_desc[0]->rate : 0,
                'total_desc_value' => $gr->total_desc_value,
                'total_invc_value' => $gr->total_invoice_value,
                'payment_dynamic_fields' => $gr->payment_dynamic_fields,
                'pay_mode' => $gr->pay_mode,
                'vehicle'=> $gr->vehicle ? $gr->vehicle : ($gr->vehicle_number ? ['id'=> null, 'vehicle_number'=>$gr->vehicle_number, 'vehicle_type'=>$gr->vehicle_type] : null),
                'acc_party'=> $gr->acc_party, 
                'acc_party_branch'=> $gr->acc_party_branch,
                'freight' => $gr->freight,
                'gr_chg' => $gr->gr_chg,
                'gst_paid_by' => $gr->gst_paid_by,
                'disc_percent' => $gr->disc_percent,
                'disc_amount' => $gr->disc_amount,
                'total_without_gst' => $gr->total_without_gst,
                'green_tax' => $gr->green_tax,
                'kanta' => $gr->kanta,
                'detention' => $gr->detention,
                'unload_rate' => $gr->unload_rate,
                'unload_amt' => $gr->unload_amt,
                'toll_amt' => $gr->toll_amt,
                'diversion_rate' => $gr->diversion_rate,
                'diversion_amt' => $gr->diversion_amt,

                'igst_amount' => $gr->igst_amount,
                'cgst_amount' => $gr->cgst_amount,
                'sgst_amount' => $gr->sgst_amount,
                'igst_percent' => $gr->igst_percent,
                'cgst_percent' => $gr->cgst_percent,
                'sgst_percent' => $gr->sgst_percent,
                'lorry_hire_from'=>null,
                'challan'=>null
        ];

        if(count($gr->challan_desc)){
            $copy = json_decode(json_encode($gr->challan_desc[0]));
            $data['lorry_hire_from'] = $copy->challan->lorry_hire_from;
            $data['challan'] = $copy->challan;

        }

        return response()->json(['gr' => $data]);
        
    }

    public function get_gr_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = BookGr::with(
                            ['consignor:id,party_name','consignee:id,party_name,state','acc_party:id,account_name','acc_party.party:id,account_id,gst_number,state,party_code','source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number,vehicle_type','gr_desc','gr_desc.material:id,material_name',
                                'challan_desc' => function ($query) {
                                    $query->select('gr_id','challan_id', 'challan_number')->groupBy('gr_id','challan_id', 'challan_number');
                                },
                                'pod_desc' => function ($query) {
                                    $query->select('gr_id','pod_id', 'pod_number')->groupBy('gr_id', 'pod_id','pod_number');
                                },
                                'bill_desc' => function ($query) {
                                    $query->select('gr_id', 'bill_id','bill_number')->groupBy('gr_id', 'bill_id','bill_number');
                                },
                                'voucher_desc'=>function($query){
                                    $query->select('against_number','voucher_id', 'voucher_number','receipt_amt')->where('against','GR')->groupBy('against_number','voucher_id', 'voucher_number','receipt_amt');
                                },
                                'delivery_order:id,do_number',
                                'voucher_desc.voucher:id,voucher_number,narration,voucher_date',
                                'challan_desc.challan:id,challan_number,lh_net_freight',
                                'lorry_hire_from:id,account_name,account_type',
                                'lh_voucher_desc',
                                'lh_voucher_desc.voucher:id,voucher_number,narration,voucher_date',

                        
                            ]
                        )->where('fin_year',$request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['gr_num_from']) && isset($report_data['gr_num_to'])){
            $query->where('gr_number','>=',$report_data['gr_num_from']['gr_number'])->where('gr_number','<=',$report_data['gr_num_to']['gr_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('gr_date','>=',$report_data['from_date'])->where('gr_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['consignor']) ){
            $query->where('consignor_id', $report_data['consignor']['id']);
        }

        if(isset($report_data['delivery_order']) ){
            $query->where('do_id', $report_data['delivery_order']['id']);
        }

        if(isset($report_data['consignee']) ){
            $query->where('consignee_id', $report_data['consignee']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['pay_mode']) ){
            $query->where('pay_mode', $report_data['pay_mode']);
        }

        if(isset($report_data['source']) ){
            $query->where('source_id', $report_data['source']['id']);
        }

        if(isset($report_data['destination']) ){
            $query->where('destination_id', $report_data['destination']['id']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['lorry_hire_from']) ){
            $query->where('lorry_hire_from', $report_data['lorry_hire_from']['id']);
        }
        

        if(isset($report_data['paid_status']) ){
            if($report_data['paid_status']=='Paid')
            $query->where('paid_status', 1);
            else if($report_data['paid_status']=='Outstanding')
            $query->where('paid_status', 0);
        }

        if(isset($report_data['lh_paid_status']) ){
            if($report_data['lh_paid_status'] == 'Outstanding'){
              $query->where('lh_paid_status', 0)->where('lh_net_freight','>',0);  
            }
            else if($report_data['lh_paid_status'] == 'Paid'){
                $query->where(function($query){
                    $query->where('lh_paid_status', 1)->orWhere('lh_net_freight',0);
                });
            }            
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like','%'. $searchValue . '%')
                ->orWhereHas('acc_party', function ($query) use ($searchValue) {
                    $query->where('account_name', 'like', '%' . $searchValue . '%');
                })
                ->orWhere('source_name', 'like','%'. $searchValue . '%')
                ->orWhere('destination_name', 'like','%'. $searchValue . '%')
                ->orWhere('vehicle_number', 'like','%'. $searchValue . '%');
            });
        }
         
        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
        
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    public function get_gr_toll_wise_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = BookGr::select('id','gr_number','gr_date','gr_time','shipment_number','consignor_id','consignor_name','consignee_name','consignee_id','acc_party_id','edd','source_id','destination_id','vehicle_id','vehicle_number','pay_mode','total_pkg','total_material_weight','total_chg_qty','total_invoice_value','igst_amount','cgst_amount','sgst_amount','total_without_rounding','is_cancelled')
                        ->with(
                            ['consignor:id,party_name','consignee:id,party_name','acc_party:id,account_name','acc_party.party:id,account_id,gst_number','source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number',
                                
                                'pod_desc' => function ($query) {
                                    $query->select('gr_id','pod_id', 'pod_number')->groupBy('gr_id', 'pod_id','pod_number');
                                },
                                'pod_desc.pod:id,pod_number,delivery_date,delivery_time',
                            ]
                        )
                        ->whereHas('pod_desc')
                        ->where('fin_year',$request->current_year);

        
        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['gr_num_from']) && isset($report_data['gr_num_to'])){
            $query->where('gr_number','>=',$report_data['gr_num_from']['gr_number'])->where('gr_number','<=',$report_data['gr_num_to']['gr_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('gr_date','>=',$report_data['from_date'])->where('gr_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['consignor']) ){
            $query->where('consignor_id', $report_data['consignor']['id']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['consignee']) ){
            $query->where('consignee_id', $report_data['consignee']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['pay_mode']) ){
            $query->where('pay_mode', $report_data['pay_mode']);
        }

        if(isset($report_data['paid_status']) ){
            if($report_data['paid_status']=='Paid')
            $query->where('paid_status', 1);
            else if($report_data['paid_status']=='Outstanding')
            $query->where('paid_status', 0);
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like','%'. $searchValue . '%')
                ->orWhereHas('acc_party', function ($query) use ($searchValue) {
                    $query->where('account_name', 'like', '%' . $searchValue . '%');
                })
                ->orWhere('source_name', 'like','%'. $searchValue . '%')
                ->orWhere('destination_name', 'like','%'. $searchValue . '%')
                ->orWhere('vehicle_number', 'like','%'. $searchValue . '%');
            });
        }
         
            $data = $query->orderBy('gr_date','asc')->get();

            $result_data = [];

            foreach ($data as $value) {
                $gr_date = $value['gr_date'];
                $gr_time = $value['gr_time'];
                $pod_date = $value['pod_desc']['pod']['delivery_date'];
                $pod_time = $value['pod_desc']['pod']['delivery_time'];

                $gr_datetime = $gr_date . ' ' . $gr_time;
                $pod_datetime = $pod_date . ' ' . $pod_time;

                $voucher_sum = Voucher::select('id', 'branch_id', 'total_dr_amount', 'total_cr_amount', 'narration', 'voucher_number', 'voucher_date', 'bank_date', 'voucher_name', 'created_by')
                    ->whereRaw("CONCAT(voucher_date, ' ', voucher_time) BETWEEN ? AND ?", [$gr_datetime, $pod_datetime]);




                if (isset($value['vehicle_id'])) {
                    $voucher_sum->whereHas('voucher_acc_details', function ($qr) use ($report_data) {
                                // Adds a condition to filter voucher_acc_details where related account's account_name matches party_name
                                $qr->whereHas('account', function ($sqr) use ($report_data) {
                                    $sqr->where('account_name', 'FASTAG EXPENSE');
                                });
                            });

                    $voucher_sum->whereHas('voucher_descs', function ($qr) use ($value) {
                                        $qr->where('against_id', $value['vehicle_id'])
                                        ->where('against', 'VEHICLE_COST_CENTER');
                                    });

                    // Sum the receipt_amt for the filtered voucher_descs


                    $totalAmount = $voucher_sum->sum('total_dr_amount');

                    $value['amount'] = $totalAmount ;

                    $result_data[] = $value;
                }

            
            }


          
        return response()->json(['data' => $result_data, 'draw' => $request->draw]);
    }


    public function get_pending_challan_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = BookGr::select('id','gr_number','gr_date','consignor_id','consignee_id','acc_party_id','edd','source_id','destination_id','vehicle_id','vehicle_number','pay_mode','total_pkg','total_material_weight','total_invoice_value','igst_amount','cgst_amount','sgst_amount','total_without_rounding','is_cancelled')
        ->with(
            ['consignor:id,party_name','consignee:id,party_name','acc_party:id,account_name','acc_party.party:id,account_id,gst_number','source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number']
        )
        ->where('fin_year',$request->current_year)
        ->whereDoesntHave('challan_desc');

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('gr_date','>=',$report_data['from_date'])->where('gr_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['consignor']) ){
            $query->where('consignor_id', $report_data['consignor']['id']);
        }

        if(isset($report_data['consignee']) ){
            $query->where('consignee_id', $report_data['consignee']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }


        if(isset($report_data['source']) ){
            $query->where('source_id', $report_data['source']['id']);
        }

        if(isset($report_data['destination']) ){
            $query->where('destination_id', $report_data['destination']['id']);
        }

        if(isset($report_data['self_market']) ){
            // WHERE VEHICLE relationship is self_or_market

            $query->whereHas('vehicle', function ($query) use ($report_data) {
                $query->where('self_market', $report_data['self_market']);
            });
        }

        if(isset($report_data['owner']) ){
            $query->whereHas('vehicle', function ($query) use ($report_data) {
                $query->where('owner_id', $report_data['owner']['id']);
            });
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_gr_stock_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
        
        $branch = $report_data['branch'];
        $from_date = $report_data['from_date'];
        $to_date = $report_data['to_date'];
        $status = $report_data['status'];
                
                // Define the package_position relationship separately
                $packagePositionRelationship = function ($query) use ($status, $branch) {
                    if ($status == 'Pending') {
                        $query->where('position', $branch['id'])->where('pkg', '>', 0)->where('is_delivered', 0);
                    }

                    if ($status == 'Delivered') {
                        $query->where('position', '!=', 'IN TRANSIT')->where('is_delivered', 1);
                    }

                    if ($status == 'In Transit') {
                        $query->where('position', 'IN TRANSIT')->where('pkg', '>', 0)->where('is_delivered', 0);
                    }
                };

                // Use the defined relationship in the with method
                $query = BookGr::with([
                    'branch:id,branch_name',
                    'consignor:id,party_name',
                    'consignee:id,party_name',
                    'source:id,station_name',
                    'destination:id,station_name',
                    'pod_desc' => function ($query) {
                        $query->select('gr_id', 'pod_id', 'pod_number')->groupBy('gr_id', 'pod_id', 'pod_number');
                    },
                    'package_position' => $packagePositionRelationship,
                    'package_position.challan_desc:id,gr_id,challan_id,challan_number,loaded_pkgs',
                    'package_position.gr_desc:id,material_id,pkg',
                    'package_position.gr_desc.material:id,material_name',
                    'package_position.branch:id,branch_code'
                ])
                    ->where('gr_date', '>=', $from_date)
                    ->where('gr_date', '<=', $to_date)
                    ->when($status == 'Delivered' || $status == 'In Transit', function ($query) use ($branch) {
                        $query->where('branch_id', $branch['id']);
                    })
                    ->whereHas('package_position', $packagePositionRelationship);

                
                    if ( !empty($request->user_branch_ids) ) {
                        $query->whereIn('branch_id', $request->user_branch_ids );
                    }       
    

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like', $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }

        return response()->json(['data' => $data, 'draw' => $request->draw]);

    }


    public function get_gr_data_for_eway_partb(Request $request,$pk){
        $eway_numbers = EwayGr::where('gr_id',$pk)->pluck('eway_number')->toArray();

        $gr = BookGr::select('source_id','destination_id')->with(['source:id,station_name', 'destination:id,station_name'])->find($pk);

        $data = [
            'source' => $gr->source_id ? $gr->source : null,
            'destination' => $gr->destination_id ? $gr->destination : null,
            'eway_numbers' => $eway_numbers
        ];

        return response()->json(['data' => $data]);

    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $gr = BookGr::with([
            'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number', 'driver:id,emp_name',
            'gr_desc',
            'gr_desc.material' => function ($query) {
                $query->select('id', 'material_name');
            },
            'eway_details',
            'acc_party' => function ($query) {
                $query->select('id', 'account_name');
            },
            'acc_party_branch:id,branch_name',
            'consignor_branch:id,branch_name',
            'consignee_branch:id,branch_name',
            'driver_payment_from:id,account_name',
                    'created_by:id,username',
                    'delivery_order:id,do_number'
            ])->find($pk);

        return ['gr'=>$gr];
    }

    protected function find_remaining_balance_for_do_desc($do_desc_id){
        // Find the delivery order description
        $do_desc = DeliveryOrderDesc::find($do_desc_id);
    
        // Check if the delivery order description exists
        if ($do_desc) {
            // Find the related delivery order
            $book_gr_ids = BookGr::where('do_id', $do_desc['do_id'])->pluck('id');
    
            $sum_field = 'material_weight';
            if($do_desc->unit=='PKG'){
                $sum_field = 'pkg';
            }
            elseif($do_desc->unit=='KG' || $do_desc->unit=='MT' || $do_desc->unit=='Quintal'){
                $sum_field = 'material_weight';
            }
    
            // Find the sum of pkg belonging to book_gr relationships of do_desc
            $sum_pkg = GrDesc::whereIn('gr_id', $book_gr_ids)
                            ->where('material_id', $do_desc->material_id)
                            ->sum($sum_field);
    
            return ['used_qty'=>$sum_pkg, 'remaining_qty'=>$do_desc->qty - $sum_pkg, 'total_qty'=>$do_desc->qty]  ;
        }
    
    }

    protected function get_max_number_from_gr_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function record_transaction_in_accounts($request, $gr){
            $t_data = [];
            $t_data['t_against'] = 'GR';
            $t_data['t_against_id'] = $gr->id;
            $t_data['fin_year'] = $request->current_year;
            $t_data['t_against_no'] = $gr->gr_number;
            
            $this->delete_transaction_voucher($t_data);
            
            $t_data['branch'] = $request->branch;
            $t_data['voucher_date'] = $request->gr_date;
            $t_data['voucher_time'] = $request->gr_time;
            $t_data['t_against_narration'] = $gr->remarks;
            $t_data['fin_year'] = $request->current_year;

            $t_data['voucher_accounting_details'] = [];
            
            
                $t_data['cancellable'] = 1;
                if($request->pay_mode == 'TO PAY' || $request->pay_mode == 'PAID'){
                    if($gr->income_account_id && $request->total_without_rounding>0){

                        $AFTER_GST = $gr->green_tax + $gr->kanta + $gr->detention; 
                        $t_data['voucher_accounting_details']=[];
                        // DEBIT PARTY ACCOUNT
                        $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$request->total_without_rounding, 'cr_amount'=>0 ];
                        

                        // CREDIT SIDE
                        if($request->cgst_amount > 0){
                            // CREDIT CGST ACCOUNT
                            $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->cgst_amount ];
                        }
                        if($request->sgst_amount > 0){
                            // CREDIT SGST ACCOUNT
                            $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->sgst_amount ];
                        }
                        if($request->igst_amount > 0){
                            // CREDIT IGST ACCOUNT
                            $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->igst_amount ];
                        }

                        // CREDIT INCOME ACCOUNT
                        $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->income_account_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->total_without_gst+$AFTER_GST ];

                        
                        //CHECK TOTAL DR AND CR
                        $total_dr = 0;
                        $total_cr = 0;

                        foreach ($t_data['voucher_accounting_details'] as $vad) {
                            $total_dr += $vad['dr_amount'];
                            $total_cr += $vad['cr_amount'];
                        }

                                    // Round both $total_dr and $total_cr to two decimal places
                        $total_dr_rounded = round(floatval($total_dr), 2);
                        $total_cr_rounded = round(floatval($total_cr), 2);

                        // Check if they are not equal
                        if ($total_dr_rounded != $total_cr_rounded) {
                            // Output the values for debugging
                            dd($total_dr_rounded, $total_cr_rounded);
                            
                            // Return response with an error message
                            return response()->json(['message' => 'Error In Accounting Entries'], 422);
                        }


                                    $t_data['total_dr_amount'] = $total_dr;
                                    $t_data['total_cr_amount'] = $total_cr; 


                                    $this->create_transaction_voucher( $t_data );
                                }
                }

                $t_data['cancellable'] = 0;
                if($gr->driver_payment_amt>0 && $gr->driver && $gr->driver_payment_from){
                    $driver_account_id = $this->get_acc_id($request->driver['emp_name']);
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$driver_account_id, 'nature'=>'dr' , 'dr_amount'=>$request->driver_payment_amt, 'cr_amount'=>0 ];
                    

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->driver_payment_from, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->driver_payment_amt ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    $total_dr_rounded = round(floatval($total_dr), 2);
                    $total_cr_rounded = round(floatval($total_cr), 2);

                    // Check if they are not equal
                    if ($total_dr_rounded != $total_cr_rounded) {
                                // Output the values for debugging
                                dd($total_dr_rounded, $total_cr_rounded);
                        
                        // Return response with an error message
                        return response()->json(['message' => 'Error In Accounting Entries'], 422);
                    }


                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr; 


                    $this->create_transaction_voucher( $t_data );
                }

                $t_data['cancellable'] = 0;
                if($gr->lh_net_freight>0 && $gr->lorry_hire_from){
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->dr_on_account, 'nature'=>'dr' , 'dr_amount'=>$request->lh_net_freight, 'cr_amount'=>0 ];
                    

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_net_freight ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    if($total_dr != $total_cr){
                        return response()->json(['message'=>'Error In Accounting Entries'], 422);
                    }


                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr; 


                    $this->create_transaction_voucher( $t_data );
                }
                
                $t_data['cancellable'] = 0;
                if($gr->lorry_hire_from && $gr->lh_tds>0){
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$gr->lh_tds, 'cr_amount'=>0 ];
                    
                    // TDS PAYABLE
                    $t_data['voucher_accounting_details'][] = ['account_id'=>4, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$gr->lh_tds ];
                    
                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    // Round both $total_dr and $total_cr to two decimal places
                    $total_dr_rounded = round(floatval($total_dr), 2);
                    $total_cr_rounded = round(floatval($total_cr), 2);

                    // Check if they are not equal
                    if ($total_dr_rounded != $total_cr_rounded) {
                        // Output the values for debugging
                        dd($total_dr_rounded, $total_cr_rounded);
                        
                        // Return response with an error message
                        return response()->json(['message' => 'Error In Accounting Entries'], 422);
                    }

                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr;


                    $this->create_transaction_voucher( $t_data );

                }

                $t_data['cancellable'] = 0;
                if($gr->lh_advance>0 && $gr->lh_payment_acc1 && $gr->lorry_hire_from){
                     $t_data['voucher_date'] = $gr->lh_advance_date ?? $request->gr_date;
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->lh_advance, 'cr_amount'=>0 ];
                    

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lh_payment_acc1, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_advance ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    if($total_dr != $total_cr){
                        return response()->json(['message'=>'Error In Accounting Entries'], 422);
                    }


                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr; 


                    $this->create_transaction_voucher( $t_data );
                }
                
                 $t_data['voucher_date'] = $request->gr_date;

                $t_data['cancellable'] = 0;
                if($gr->lh_payment>0 && $gr->lh_payment_acc2 && $gr->lorry_hire_from){
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->lh_payment, 'cr_amount'=>0 ];
                    

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lh_payment_acc2, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_payment ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    if($total_dr != $total_cr){
                        return response()->json(['message'=>'Error In Accounting Entries'], 422);
                    }


                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr; 


                    $this->create_transaction_voucher( $t_data );
                }

                $t_data['cancellable'] = 0;
                if($gr->lh_payment2>0 && $gr->lh_payment_acc3 && $gr->lorry_hire_from){
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->lh_payment2, 'cr_amount'=>0 ];
                    

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lh_payment_acc3, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_payment2 ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    if($total_dr != $total_cr){
                        return response()->json(['message'=>'Error In Accounting Entries'], 422);
                    }


                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr; 


                    $this->create_transaction_voucher( $t_data );
                }

                $t_data['cancellable'] = 0;
                if($gr->diesel_amt1>0 && $gr->diesel_acc1 && $gr->lorry_hire_from){
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->diesel_amt1, 'cr_amount'=>0 ];
                    

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->diesel_acc1, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->diesel_amt1 ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    if($total_dr != $total_cr){
                        return response()->json(['message'=>'Error In Accounting Entries'], 422);
                    }


                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr; 


                    $this->create_transaction_voucher( $t_data );
                }

                $t_data['cancellable'] = 0;
                if($gr->diesel_amt2>0 && $gr->diesel_acc1 && $gr->lorry_hire_from){
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->diesel_amt2, 'cr_amount'=>0 ];
                    

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->diesel_acc2, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->diesel_amt2 ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                    if($total_dr != $total_cr){
                        return response()->json(['message'=>'Error In Accounting Entries'], 422);
                    }


                    $t_data['total_dr_amount'] = $total_dr;
                    $t_data['total_cr_amount'] = $total_cr; 


                    $this->create_transaction_voucher( $t_data );
                }
          

                

    }

    


}
