<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherAccDetails;
use App\Models\Account\VoucherDesc;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\FleetOwner\DriverSettlement;
use App\Models\FleetOwner\DriverSettlementDesc;
use App\Models\GoodsMove\Challan;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DriverSettlementController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'DriverSettlement');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $d_settlements = DriverSettlement::with([
            'branch:id,branch_name,branch_code',
            'vehicle:id,vehicle_number,account_id',
            'driver:id,emp_name,account_id',
            'd_settlement_desc'
        ])->where('fin_year',$request->current_year)->get();

        $d_settlements->d_settlement_desc->transform(function ($desc) {
            if($desc->against == 'GR'){
                $book_gr = BookGr::select('gr_date','acc_party_id','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                $date = $book_gr->gr_date;
                $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : null;
                $source = $book_gr->source ? $book_gr->source->station_name : null;
                $destination = $book_gr->destination ? $book_gr->destination->station_name : null;
            }
            elseif($desc->against == 'Loading Slip'){
                $l_slip = LoadingSlip::select('ls_date','acc_party_id','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                $date = $l_slip->ls_date;
                $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : null;
                $source = $l_slip->source ? $l_slip->source->station_name : null;
                $destination = $l_slip->destination ? $l_slip->destination->station_name : null;
            }
            elseif($desc->against == 'Challan'){
                $challan = Challan::select('challan_date','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                $date = $challan->challan_date;
                $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : null;
                $source = $challan->source ? $challan->source->station_name : null;
                $destination = $challan->destination ? $challan->destination->station_name : null;
            }
                
                return [
                    'd_settlement_desc_id' => $desc->id,
                    'against' => $desc->against,
                    'against_number' => $desc->against_number,
                    'against_id' => $desc->against_id,
                    'vehicle_number' => $vehicle,
                    'source' => $source,
                    'destination' => $destination,
                    'date' => $date,
                    'diesel_receipt_amt' => $desc->diesel_receipt_amt,
                    'diesel_receipt_qty' => $desc->diesel_receipt_qty,
                        'alloted_dsl' => $desc->alloted_dsl,
                        'dsl_diff' => $desc->dsl_diff,
                        'dsl_rate' => $desc->dsl_rate,
                        'dsl_diff_amount' => $desc->dsl_diff_amount,
                    'freight' => $desc->freight,
                    'advance' => $desc->advance,
                    'reading' => $desc->reading,
                    'return_freight' => $desc->return_freight,
                    'green_tax' => $desc->green_tax,
                    'diesel_amt' => $desc->diesel_amt,
                    'toll' => $desc->toll,
                    'state_entry' => $desc->state_entry,
                    'legal_exp' => $desc->legal_exp,
                    'food' => $desc->food,
                    'maintainance' => $desc->maintainance,
                    'parking' => $desc->parking,
                    'dalla' => $desc->dalla,
                    'other1' => $desc->other1,
                    'other2' => $desc->other2,
                    'other3' => $desc->other3,
                    'total' => $desc->total
                ];
        });

        return response()->json($d_settlements);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $d_settlement = DriverSettlement::with([
            'branch:id,branch_name,branch_code',
            'vehicle:id,vehicle_number,account_id',
            'driver:id,emp_name,account_id',
            'd_settlement_desc'
        ])->find($pk);

        $d_settlement->d_settlement_desc->transform(function ($desc) {
            if($desc->against == 'GR'){
                $book_gr = BookGr::select('gr_date','acc_party_id','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                $date = $book_gr->gr_date;
                $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : null;
                $source = $book_gr->source ? $book_gr->source->station_name : null;
                $destination = $book_gr->destination ? $book_gr->destination->station_name : null;
            }
            elseif($desc->against == 'Loading Slip'){
                $l_slip = LoadingSlip::select('ls_date','acc_party_id','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                $date = $l_slip->ls_date;
                $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : null;
                $source = $l_slip->source ? $l_slip->source->station_name : null;
                $destination = $l_slip->destination ? $l_slip->destination->station_name : null;
            }
            elseif($desc->against == 'Challan'){
                $challan = Challan::select('challan_date','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                $date = $challan->challan_date;
                $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : null;
                $source = $challan->source ? $challan->source->station_name : null;
                $destination = $challan->destination ? $challan->destination->station_name : null;
            }
                
                return [
                    'd_settlement_desc_id' => $desc->id,
                    'against' => $desc->against,
                    'against_number' => $desc->against_number,
                    'against_id' => $desc->against_id,
                    'vehicle_number' => $vehicle,
                    'source' => $source,
                    'destination' => $destination,
                    'date' => $date,
                    'diesel_receipt_amt' => $desc->diesel_receipt_amt,
                    'diesel_receipt_qty' => $desc->diesel_receipt_qty,
                        'alloted_dsl' => $desc->alloted_dsl,
                        'dsl_diff' => $desc->dsl_diff,
                        'dsl_rate' => $desc->dsl_rate,
                        'dsl_diff_amount' => $desc->dsl_diff_amount,

                    'freight' => $desc->freight,
                    'advance' => $desc->advance,
                    'reading' => $desc->reading,
                    'return_freight' => $desc->return_freight,
                    'green_tax' => $desc->green_tax,
                    'diesel_amt' => $desc->diesel_amt,
                    'toll' => $desc->toll,
                    'state_entry' => $desc->state_entry,
                    'legal_exp' => $desc->legal_exp,
                    'food' => $desc->food,
                    'maintainance' => $desc->maintainance,
                    'parking' => $desc->parking,
                    'dalla' => $desc->dalla,
                    'other1' => $desc->other1,
                    'other2' => $desc->other2,
                    'other3' => $desc->other3,
                    'total' => $desc->total
                ];
        });

        return response()->json(['driver_settlement' =>$d_settlement]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'DriverSettlement');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'ds_number' => 'required|unique:driver_settlements,ds_number',
        ], [
            'branch.required' => 'Required',
            'ds_number.required' => 'Required',
            'ds_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE CHALLAN TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_ds_number($request->ds_number);
            
            // SAVING A NEW DS
            $d_settlement = new DriverSettlement();
            $d_settlement->branch_id = $request->branch['id'];
            $d_settlement->max_number = $max_number;
            $d_settlement->ds_number = $request->ds_number;
            $d_settlement->settle_date = $request->settle_date;

            $d_settlement->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $d_settlement->driver_id = $request->driver ? $request->driver['id'] : null;
            $d_settlement->driver_account_id = $request->driver ? $request->driver['account_id'] : null;

            $d_settlement->against = $request->against;
            $d_settlement->from_date = $request->from_date;
            $d_settlement->till_date = $request->till_date;
            $d_settlement->last_balance = $request->last_balance;

            $d_settlement->total_diesel_receipt_amt = $request->total_diesel_receipt_amt;
            $d_settlement->total_diesel_receipt_qty = $request->total_diesel_receipt_qty;
            $d_settlement->total_alloted_dsl = $request->total_alloted_dsl;
            $d_settlement->total_dsl_diff = $request->total_dsl_diff;
            $d_settlement->total_dsl_diff_amount = $request->total_dsl_diff_amount;
            
            $d_settlement->total_freight = $request->total_freight;
            $d_settlement->total_advance = $request->total_advance;
            $d_settlement->total_return_freight = $request->total_return_freight;
            $d_settlement->total_green_tax = $request->total_green_tax;
            $d_settlement->total_diesel_amt = $request->total_diesel_amt;
            $d_settlement->total_toll = $request->total_toll;
            $d_settlement->total_state_entry = $request->total_state_entry;
            $d_settlement->total_legal_exp = $request->total_legal_exp;
            $d_settlement->total_food = $request->total_food;
            $d_settlement->total_maintainance = $request->total_maintainance;
            $d_settlement->total_parking = $request->total_parking;
            $d_settlement->total_dalla = $request->total_dalla;
            $d_settlement->total_other1 = $request->total_other1;
            $d_settlement->total_other2 = $request->total_other2;
            $d_settlement->total_other3 = $request->total_other3;
            $d_settlement->total_desc_total = $request->total_desc_total;

            $d_settlement->start_km = $request->start_km;
            $d_settlement->end_km = $request->end_km;
            $d_settlement->total_km = $request->total_km;
            $d_settlement->mileage = $request->mileage;

            $d_settlement->empty_voucher_amt = $request->empty_voucher_amt;
            $d_settlement->unmarked_diesel_amt = $request->unmarked_diesel_amt;
            $d_settlement->unmarked_diesel_qty = $request->unmarked_diesel_qty;
            $d_settlement->unmarked_pump_advance_amt = $request->unmarked_pump_advance_amt;


            $d_settlement->fastag_amt = $request->fastag_amt;
            $d_settlement->mcd_toll = $request->mcd_toll;
            $d_settlement->driver_expense = $request->driver_expense;
            $d_settlement->total_fuel_expense = $request->total_fuel_expense;
            $d_settlement->total_trip_expense = $request->total_trip_expense;
            $d_settlement->driver_balance = $request->driver_balance;

            $d_settlement->other1_label = $request->other1_label;
            $d_settlement->other2_label = $request->other2_label;
            $d_settlement->other3_label = $request->other3_label;

            $d_settlement->remarks = $request->remarks;


            $d_settlement->total_hire_freight = $request->total_hire_freight;
            $d_settlement->total_profit_loss = $request->total_profit_loss;
            $d_settlement->salary_amount = $request->salary_amount;

            $d_settlement->fin_year = $request->current_year;

            $d_settlement->created_by = $request->user->id;

            // Save the model to the database
            $d_settlement->save();

            

            // SAVING SETTLEMENT DETAILS
            $d_settlement_desc = $request->d_settlement_desc;
            foreach ($d_settlement_desc as $dsd) {
                $ds_dc = new DriverSettlementDesc();
                $ds_dc->driver_settlement_id = $d_settlement->id;
                $ds_dc->ds_number = $d_settlement->ds_number;
                $ds_dc->against = $dsd['against'];
                $ds_dc->against_id = $dsd['against_id'];
                $ds_dc->against_number = $dsd['against_number'];
                $ds_dc->diesel_receipt_amt = $dsd['diesel_receipt_amt'];

                $ds_dc->diesel_receipt_qty = $dsd['diesel_receipt_qty'];
                $ds_dc->alloted_dsl = $dsd['alloted_dsl'];
                $ds_dc->dsl_diff = $dsd['dsl_diff'];
                $ds_dc->dsl_rate = $dsd['dsl_rate'];
                $ds_dc->dsl_diff_amount = $dsd['dsl_diff_amount'];

                $ds_dc->freight = $dsd['freight'];
                $ds_dc->advance = $dsd['advance'];
                $ds_dc->reading = $dsd['reading'];
                $ds_dc->return_freight = $dsd['return_freight'];
                $ds_dc->green_tax = $dsd['green_tax'];
                $ds_dc->diesel_amt = $dsd['diesel_amt'];
                $ds_dc->toll = $dsd['toll'];
                $ds_dc->state_entry = $dsd['state_entry'];
                $ds_dc->legal_exp = $dsd['legal_exp'];
                $ds_dc->food = $dsd['food'];
                $ds_dc->maintainance = $dsd['maintainance'];
                $ds_dc->parking = $dsd['parking'];
                $ds_dc->dalla = $dsd['dalla'];
                $ds_dc->other1 = $dsd['other1'];
                $ds_dc->other2 = $dsd['other2'];
                $ds_dc->other3 = $dsd['other3'];
                $ds_dc->total = $dsd['total'];
                $ds_dc->fin_year = $request->current_year;
                $ds_dc->save();
            }
            // SETTLEMENT DETAILS SAVED


            $advance_without_against = VoucherAccDetails::whereHas('voucher', function ($query) use ($d_settlement,$request) {
                $query->whereBetween('voucher_date',[$request->from_date, $request->till_date])
                ->whereIn('voucher_name',['Payment Voucher','Journal'])
                ->whereDoesntHave('voucher_descs');
            })
            ->where('account_id', $d_settlement->driver_account_id)
            ->update(['settel_status' => 1, 'settel_number' => $d_settlement->ds_number]);


            // $advance_without_against = Voucher::whereBetween('voucher_date',[$request->from_date, $request->till_date])
            //                             ->whereHas('voucher_acc_details', function ($query) use ($d_settlement) {
            //                                 $query->where('account_id', $d_settlement->driver_account_id) ;
            //                             })
            //                             ->whereIn('voucher_name',['Payment Voucher','Journal'])
            //                             ->whereDoesntHave('voucher_descs')
            //                             ->update(['settel_status' => 1, 'settel_number' => $d_settlement->ds_number]);

            $unmarked_diesel_amt = DieselReceipt::whereBetween('booking_date',[$request->from_date, $request->till_date])
                    ->where('driver_on_account',$d_settlement->driver_account_id)
                    ->whereNull('booking_id')
                    ->update(['settel_status' => 1, 'settel_number' => $d_settlement->ds_number]);


            // RECORD THE TRANSACTION IN ACCOUNTS
           $this->record_transaction_in_accounts($request,$d_settlement);
            // TRANSACTION RECORDED IN ACCOUNTS

             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($d_settlement->id);
            $this->createEditLog('created', 'driver_settlements', $d_settlement->id, $d_settlement->ds_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$d_settlement->id, 'ds_number'=>$d_settlement->ds_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'DriverSettlement');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $d_settlement = DriverSettlement::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($d_settlement->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'ds_number' => 'required'
        ], [
            'branch.required' => 'Required',
            'ds_number.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW DS
            $d_settlement->settle_date = $request->settle_date;
            
            $d_settlement->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $d_settlement->driver_id = $request->driver ? $request->driver['id'] : null;
            $d_settlement->driver_account_id = $request->driver ? $request->driver['account_id'] : null;

            $d_settlement->against = $request->against;
            $d_settlement->from_date = $request->from_date;
            $d_settlement->till_date = $request->till_date;
            $d_settlement->last_balance = $request->last_balance;

            $d_settlement->total_diesel_receipt_amt = $request->total_diesel_receipt_amt;
            $d_settlement->total_diesel_receipt_qty = $request->total_diesel_receipt_qty;
            $d_settlement->total_alloted_dsl = $request->total_alloted_dsl;
            $d_settlement->total_dsl_diff = $request->total_dsl_diff;
            $d_settlement->total_dsl_diff_amount = $request->total_dsl_diff_amount;

            $d_settlement->total_freight = $request->total_freight;
            $d_settlement->total_advance = $request->total_advance;
            $d_settlement->total_return_freight = $request->total_return_freight;
            $d_settlement->total_green_tax = $request->total_green_tax;
            $d_settlement->total_diesel_amt = $request->total_diesel_amt;
            $d_settlement->total_toll = $request->total_toll;
            $d_settlement->total_state_entry = $request->total_state_entry;
            $d_settlement->total_legal_exp = $request->total_legal_exp;
            $d_settlement->total_food = $request->total_food;
            $d_settlement->total_maintainance = $request->total_maintainance;
            $d_settlement->total_parking = $request->total_parking;
            $d_settlement->total_dalla = $request->total_dalla;
            $d_settlement->total_other1 = $request->total_other1;
            $d_settlement->total_other2 = $request->total_other2;
            $d_settlement->total_other3 = $request->total_other3;
            $d_settlement->total_desc_total = $request->total_desc_total;

            $d_settlement->start_km = $request->start_km;
            $d_settlement->end_km = $request->end_km;
            $d_settlement->total_km = $request->total_km;
            $d_settlement->mileage = $request->mileage;
            
            $d_settlement->empty_voucher_amt = $request->empty_voucher_amt;
            $d_settlement->unmarked_diesel_amt = $request->unmarked_diesel_amt;
            $d_settlement->unmarked_diesel_qty = $request->unmarked_diesel_qty;
            $d_settlement->unmarked_pump_advance_amt = $request->unmarked_pump_advance_amt;

            $d_settlement->fastag_amt = $request->fastag_amt;
            $d_settlement->mcd_toll = $request->mcd_toll;
            $d_settlement->driver_expense = $request->driver_expense;
            $d_settlement->total_fuel_expense = $request->total_fuel_expense;
            $d_settlement->total_trip_expense = $request->total_trip_expense;
            $d_settlement->driver_balance = $request->driver_balance;
            
            $d_settlement->other1_label = $request->other1_label;
            $d_settlement->other2_label = $request->other2_label;
            $d_settlement->other3_label = $request->other3_label;

            $d_settlement->remarks = $request->remarks;

            $d_settlement->total_hire_freight = $request->total_hire_freight;
            $d_settlement->total_profit_loss = $request->total_profit_loss;
            $d_settlement->salary_amount = $request->salary_amount;

            $d_settlement->updated_by = $request->user->id;

            $d_settlement->save();
            // NEW CHALLAN SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_d_settlement_desc = $request->newly_added_d_settlement_desc;
            foreach ($newly_added_d_settlement_desc as $nald) {
                
                $ds_dc = new DriverSettlementDesc();
                $ds_dc->driver_settlement_id = $d_settlement->id;
                $ds_dc->ds_number = $d_settlement->ds_number;
                $ds_dc->against = $nald['against'];
                $ds_dc->against_id = $nald['against_id'];
                $ds_dc->against_number = $nald['against_number'];
                
                $ds_dc->diesel_receipt_amt = $nald['diesel_receipt_amt'];
                $ds_dc->diesel_receipt_qty = $nald['diesel_receipt_qty'];
                $ds_dc->alloted_dsl = $nald['alloted_dsl'];
                $ds_dc->dsl_diff = $nald['dsl_diff'];
                $ds_dc->dsl_rate = $nald['dsl_rate'];
                $ds_dc->dsl_diff_amount = $nald['dsl_diff_amount'];

                $ds_dc->freight = $nald['freight'];
                $ds_dc->advance = $nald['advance'];

                $ds_dc->reading = $nald['reading'];
                $ds_dc->return_freight = $nald['return_freight'];
                $ds_dc->green_tax = $nald['green_tax'];
                $ds_dc->diesel_amt = $nald['diesel_amt'];
                $ds_dc->toll = $nald['toll'];
                $ds_dc->state_entry = $nald['state_entry'];
                $ds_dc->legal_exp = $nald['legal_exp'];
                $ds_dc->food = $nald['food'];
                $ds_dc->maintainance = $nald['maintainance'];
                $ds_dc->parking = $nald['parking'];
                $ds_dc->dalla = $nald['dalla'];
                $ds_dc->other1 = $nald['other1'];
                $ds_dc->other2 = $nald['other2'];
                $ds_dc->other3 = $nald['other3'];
                $ds_dc->total = $nald['total'];
                $ds_dc->fin_year = $request->current_year;
                $ds_dc->save();
            }
            // PACKAGE DETAILS SAVED

            // SAVING UPDATES ON EXISTING  PACKAGE DETAILS
            $updated_d_settlement_desc = $request->updated_d_settlement_desc;
            if( !empty($updated_d_settlement_desc) ){
                foreach ($updated_d_settlement_desc as $ucd) {
                       
                    $ds_dc = DriverSettlementDesc::find($ucd['d_settlement_desc_id']);

                    $ds_dc->alloted_dsl = $ucd['alloted_dsl'];
                    $ds_dc->dsl_diff = $ucd['dsl_diff'];
                    $ds_dc->dsl_rate = $ucd['dsl_rate'];
                    $ds_dc->dsl_diff_amount = $ucd['dsl_diff_amount'];

                    $ds_dc->reading = $ucd['reading'];
                    $ds_dc->return_freight = $ucd['return_freight'];
                    $ds_dc->green_tax = $ucd['green_tax'];
                    $ds_dc->diesel_amt = $ucd['diesel_amt'];
                    $ds_dc->toll = $ucd['toll'];
                    $ds_dc->state_entry = $ucd['state_entry'];
                    $ds_dc->legal_exp = $ucd['legal_exp'];
                    $ds_dc->food = $ucd['food'];
                    $ds_dc->maintainance = $ucd['maintainance'];
                    $ds_dc->parking = $ucd['parking'];
                    $ds_dc->dalla = $ucd['dalla'];
                    $ds_dc->other1 = $ucd['other1'];
                    $ds_dc->other2 = $ucd['other2'];
                    $ds_dc->other3 = $ucd['other3'];
                    $ds_dc->total = $ucd['total'];
                    $ds_dc->save();
                }
            }
            // UPDATES SAVED ON EXISTING PACKAGE DETAILS

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_d_settlement_desc = $request->deleted_d_settlement_desc;
            if( !empty($deleted_d_settlement_desc) ){
                foreach ($deleted_d_settlement_desc as $dcd) {
                    $lhd = DriverSettlementDesc::find($dcd);
                    $lhd->delete();
                }
                    
            }
            //DELETED EXISTING PACKAGE DETAILS

             // $advance_without_against = VoucherAccDetails::whereHas('voucher', function ($query) use ($d_settlement,$request) {
            //                                 $query->whereBetween('voucher_date',[$request->from_date, $request->till_date])
            //                                 ->whereIn('voucher_name',['Payment Voucher','Journal'])
            //                                 ->whereDoesntHave('voucher_descs');
            //                             })
            //                             ->where('account_id', $d_settlement->driver_account_id)
            //                             ->update(['settel_status' => 1, 'settel_number' => $d_settlement->ds_number]);

            // $advance_without_against = Voucher::whereBetween('voucher_date',[$request->from_date, $request->till_date])
            //                             ->whereHas('voucher_acc_details', function ($query) use ($d_settlement) {
            //                                 $query->where('account_id', $d_settlement->driver_account_id) ;
            //                             })
            //                             ->whereIn('voucher_name',['Payment Voucher','Journal'])
            //                             ->whereDoesntHave('voucher_descs')
            //                             ->update(['settel_status' => 1, 'settel_number' => $d_settlement->ds_number]);
            
            // $unmarked_diesel_amt = DieselReceipt::whereBetween('booking_date',[$request->from_date, $request->till_date])
            //                         ->where('driver_on_account',$d_settlement->driver_account_id)
            //                         ->whereNull('booking_id')
            //                         ->update(['settel_status' => 1, 'settel_number' => $d_settlement->ds_number]);


            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request,$d_settlement);
            // TRANSACTION RECORDED IN ACCOUNTS
            

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($d_settlement->id);
            $this->createEditLog('updated', 'driver_settlements', $d_settlement->id, $d_settlement->ds_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'DriverSettlement');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $d_settlement = DriverSettlement::find($pk);


        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($d_settlement->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


            $d_settlement_desc_ids = DriverSettlementDesc::select('id')->where('driver_settlement_id', $pk)->pluck('id')->toArray();

            $d_settlement_desc = DriverSettlementDesc::whereIn('id', $d_settlement_desc_ids)->delete();


            $advance_without_against = VoucherAccDetails::whereHas('voucher', function ($query) use ($d_settlement) {
                $query->whereBetween('voucher_date',[$d_settlement->from_date, $d_settlement->till_date])
                ->whereIn('voucher_name',['Payment Voucher','Journal'])
                ->whereDoesntHave('voucher_descs');
            })
            ->where('account_id', $d_settlement->driver_account_id)
            ->update(['settel_status' => 0, 'settel_number' => null]);

            // $advance_without_against = Voucher::whereBetween('voucher_date',[$d_settlement->from_date, $d_settlement->till_date])
            //                             ->whereHas('voucher_acc_details', function ($query) use ($d_settlement) {
            //                                 $query->where('account_id', $d_settlement->driver_account_id) ;
            //                             })
            //                             ->whereIn('voucher_name',['Payment Voucher','Journal'])
            //                             ->whereDoesntHave('voucher_descs')
            //                             ->update(['settel_status' => 0, 'settel_number' => null]);

            $unmarked_diesel_amt = DieselReceipt::whereBetween('booking_date',[$d_settlement->from_date, $d_settlement->till_date])
                    ->where('driver_on_account',$d_settlement->driver_account_id)
                    ->whereNull('booking_id')
                    ->update(['settel_status' => 0, 'settel_number' => null]);


            //FIND AND DELETE THE BRANCH
            $d_settlement->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

            //DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'Driver Settlement';
            $t_data['t_against_no'] = $d_settlement->ds_number;
            $t_data['t_against_id'] = $d_settlement->id;
            $t_data['fin_year'] = $request->current_year;

            $this->delete_transaction_voucher($t_data);


            $this->createEditLog('deleted', 'driver_settlements', $d_settlement->id, $d_settlement->ds_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = DriverSettlement::select('id', 'ds_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('ds_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $d_settlements = $query->get();
        return response(['driver_settlements' => $d_settlements, 'draw' => $request->draw]);
    }


    public function fetch_against_details(Request $request){
        $against = $request->against;
        $from_date = $request->from_date;
        $till_date = $request->till_date;
        $vehicle_account_id = $request->vehicle ? $request->vehicle['account_id'] : null;
        $vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
        $driver_account_id = $request->driver ? $request->driver['account_id'] : null;
        $driver_id = $request->driver ? $request->driver['id'] : null;
        
        $data = [];

        $advance_without_against = VoucherAccDetails::where('account_id', $driver_account_id)
                                    ->whereHas('voucher', function ($query) use ($from_date, $till_date) {
                                        $query->whereBetween('voucher_date',[$from_date, $till_date])
                                        ->whereIn('voucher_name',['Payment Voucher','Journal'])
                                        ->whereDoesntHave('voucher_descs');
                                    })
                                    ->where('settel_status',0)
                                    ->where('nature','dr')
                                    ->sum('dr_amount'); 
            

        // $advance_without_against = Voucher::whereBetween('voucher_date',[$from_date, $till_date])
        //                 ->whereHas('voucher_acc_details', function ($query) use ($driver_account_id, $vehicle_account_id) {
        //                     $query->where('account_id', $driver_account_id)->where('nature','dr');  
        //                 })
        //                 ->whereIn('voucher_name',['Payment Voucher','Journal'])
        //                 ->where('settel_status',0)
        //                 ->whereDoesntHave('voucher_descs')
        //                 ->sum('total_dr_amount');


        if($against == 'GR' || $against == 'All'){
            $against_details = BookGr::select('id', 'gr_number', 'gr_date', 'vehicle_id', 'driver_id', 'source_id', 'destination_id','driver_payment_amt', 'grand_total')
                            ->with([
                                'vehicle:id,vehicle_number',
                                'driver:id,emp_name',
                                'source:id,station_name',
                                'destination:id,station_name'
                            ])
                            ->where('is_cancelled', false)
                            ->whereDoesntHave('d_settlement_desc', function ($query) {
                                $query->where('against', 'GR');
                            })
                            ->where(function ($query) use ($vehicle_id, $driver_id) {
                                if ($vehicle_id) {
                                    $query->where('vehicle_id', $vehicle_id);
                                }
                                if ($driver_id) {
                                    $query->where('driver_id', $driver_id);
                                }
                            })
                            ->whereBetween('gr_date', [$from_date, $till_date])
                            ->get();

                  
            
            foreach($against_details as $ad){
                $diesel_receipt_amt = DieselReceipt::where('against','GR')->where('booking_id', $ad->id)->sum('amount');
                $diesel_receipt_qty = DieselReceipt::where('against','GR')->where('booking_id', $ad->id)->sum('qty');

                // I want to get sum of received_amt in voucher_desc where record matches, and the voucher_acc_details has account_id matching to driver_id or vehicle_id
                $advance = VoucherDesc::where('against', 'GR_COST_CENTER') 
                ->where('against_id', $ad->id) 
                ->whereHas('voucher_acc_details', function ($query) use ($driver_account_id, $vehicle_account_id) {
                    $query->where('account_id', $driver_account_id) 
                        ->orWhere('account_id', $vehicle_account_id); 
                })
                ->sum('received_amt'); 


                $cash_receipt_amt = DieselReceipt::where('against','GR')->where('booking_id', $ad->id)->where('driver_on_account',$driver_account_id)->sum('cash_amount');


                $data[] = [
                    'against_id' => $ad->id,
                    'against'=>'GR',
                    'against_number'=>$ad->gr_number,
                    'date' => $ad->gr_date, 
                    'vehicle_number' => $ad->vehicle ? $ad->vehicle->vehicle_number : null,
                    'driver' => $ad->driver ? $ad->driver->emp_name : null,
                    'source'=> $ad->source ? $ad->source->station_name : null,
                    'destination'=> $ad->destination ? $ad->destination->station_name : null,
                    'diesel_receipt_amt' => $diesel_receipt_amt,
                    'diesel_receipt_qty' => $diesel_receipt_qty,
                    'alloted_dsl' => $diesel_receipt_qty,
                    'dsl_diff' => null,
                    'dsl_rate' => 90,
                    'dsl_diff_amount' => null,
                    'freight' => $ad->grand_total,
                    'advance' => $advance + $cash_receipt_amt + $ad->driver_payment_amt,
                    'reading' => null,
                    'return_freight' => null,
                    'green_tax' => null,
                    'diesel_amt' => null,
                    'toll' => null,
                    'state_entry' => null,
                    'legal_exp' => null,
                    'food' => null,
                    'maintainance' => null,
                    'parking' => null,
                    'dalla' => null,
                    'other1' => null,
                    'other2' => null,
                    'other3' => null,
                    'total' => null,
                    'checked' => true
                ];
            }
        }
        
        if($against == 'Loading Slip' || $against == 'All'){
            $against_details = LoadingSlip::select('id','ls_number','ls_date','vehicle_id','driver_id','source_id','destination_id','total_without_gst','driver_payment_amt','advance_received_in','advance_amt')->with([
                                    'vehicle:id,vehicle_number',
                                    'driver:id,account_name',
                                    'source:id,station_name',
                                    'destination:id,station_name',
                                    'd_settlement_desc'
            ])
            ->whereDoesntHave('d_settlement_desc', function ($query) {
                $query->where('against', 'Loading Slip');
            })
            ->where(function ($query) use ($vehicle_id, $driver_account_id) {
                if ($vehicle_id) {
                    $query->where('vehicle_id', $vehicle_id);
                }
                if ($driver_account_id) {
                    $query->where('driver_id', $driver_account_id);
                }
            })
            ->whereBetween('ls_date', [$from_date, $till_date]);

            // Get the results
            $against_details = $against_details->get();
            


            foreach($against_details as $ad){
                $diesel_receipt_amt = DieselReceipt::where('against','Loading Slip')->where('booking_id', $ad->id)->sum('amount');
                $diesel_receipt_qty = DieselReceipt::where('against','Loading Slip')->where('booking_id', $ad->id)->sum('qty');
                // I want to get sum of received_amt in voucher_desc where record matches, and the voucher_acc_details has account_id matching to driver_id or vehicle_id
                $advance = VoucherDesc::where('against', 'LS_COST_CENTER') 
                ->where('against_id', $ad->id) 
                ->whereHas('voucher_acc_details', function ($query) use ($driver_account_id, $vehicle_account_id) {
                    $query->where('account_id', $driver_account_id) 
                        ->orWhere('account_id', $vehicle_account_id); 
                })
                ->sum('received_amt');
                

                $advance += $ad->driver_payment_amt;

                $cash_receipt_amt = DieselReceipt::where('against','Loading Slip')->where('booking_id', $ad->id)->where('driver_on_account',$driver_account_id)->sum('cash_amount');

                if($ad->advance_received_in == $driver_account_id){
                    $advance += $ad->advance_amt;
                }

                $data[] = [
                    'against_id' => $ad->id,
                    'd_settlement_desc' => $ad->d_settlement_desc,
                    'against'=>'Loading Slip',
                    'against_number'=>$ad->ls_number,
                    'date' => $ad->ls_date, 
                    'vehicle_number' => $ad->vehicle ? $ad->vehicle->vehicle_number : null,
                    'driver' => $ad->driver ? $ad->driver->account_name : null,
                    'source'=> $ad->source ? $ad->source->station_name : null,
                    'destination'=> $ad->destination ? $ad->destination->station_name : null,
                    'diesel_receipt_amt' => $diesel_receipt_amt,
                    'diesel_receipt_qty' => $diesel_receipt_qty,
                    'alloted_dsl' => $diesel_receipt_qty,
                    'dsl_diff' => null,
                    'dsl_rate' => 90,
                    'dsl_diff_amount' => null,
                    'freight' => $ad->total_without_gst,
                    'advance' => $advance + $cash_receipt_amt,
                    'reading' => null,
                    'return_freight' => null,
                    'green_tax' => null,
                    'diesel_amt' => null,
                    'toll' => null,
                    'state_entry' => null,
                    'legal_exp' => null,
                    'food' => null,
                    'maintainance' => null,
                    'parking' => null,
                    'dalla' => null,
                    'other1' => null,
                    'other2' => null,
                    'other3' => null,
                    'total' => null,
                    'checked' => true
                ];
            }

        }
        
        if($against == 'Challan' || $against == 'All'){
            $against_details = Challan::select('id', 'challan_number', 'challan_date', 'vehicle_id', 'driver_id', 'source_id', 'destination_id', 'total_freight')
                                ->with([
                                    'vehicle:id,vehicle_number',
                                    'driver:id,emp_name',
                                    'source:id,station_name',
                                    'destination:id,station_name'
                                ])
                                ->whereDoesntHave('d_settlement_desc', function ($query) {
                                    $query->where('against', 'Challan');
                                })
                                ->whereBetween('challan_date', [$from_date, $till_date])
                                ->where(function ($query) use ($vehicle_id, $driver_id) {
                                    if ($vehicle_id) {
                                        $query->where('vehicle_id', $vehicle_id);
                                    }
                                    if ($driver_id) {
                                        $query->where('driver_id', $driver_id);
                                    }
                                })
                                ->get();

             
            
            foreach($against_details as $ad){
                $diesel_receipt_amt = DieselReceipt::where('against','Challan')->where('booking_id', $ad->id)->sum('amount');
                $diesel_receipt_qty = DieselReceipt::where('against','Challan')->where('booking_id', $ad->id)->sum('qty');

                // I want to get sum of received_amt in voucher_desc where record matches, and the voucher_acc_details has account_id matching to driver_id or vehicle_id
                $advance = VoucherDesc::where('against', 'CHALLAN_COST_CENTER') 
                ->where('against_id', $ad->id) 
                ->whereHas('voucher_acc_details', function ($query) use ($driver_account_id, $vehicle_account_id) {
                    $query->where('account_id', $driver_account_id) 
                        ->orWhere('account_id', $vehicle_account_id); 
                })
                ->sum('received_amt');

                $cash_receipt_amt = DieselReceipt::where('against','Challan')->where('booking_id', $ad->id)->where('driver_on_account',$driver_account_id)->sum('cash_amount');

                $data[] = [
                    'against_id' => $ad->id,
                    'against'=>'Challan',
                    'against_number'=>$ad->challan_number,
                    'date' => $ad->challan_date, 
                    'vehicle_number' => $ad->vehicle ? $ad->vehicle->vehicle_number : null,
                    'driver' => $ad->driver ? $ad->driver->emp_name : null,
                    'source'=> $ad->source ? $ad->source->station_name : null,
                    'destination'=> $ad->destination ? $ad->destination->station_name : null,
                    'diesel_receipt_amt' => $diesel_receipt_amt,
                    'diesel_receipt_qty' => $diesel_receipt_qty,
                    'alloted_dsl' => $diesel_receipt_qty,
                    'dsl_diff' => null,
                    'dsl_rate' => 90,
                    'dsl_diff_amount' => null,
                    'freight' => $ad->total_freight,
                    'advance' => $advance + $cash_receipt_amt,
                    'reading' => null,
                    'return_freight' => null,
                    'green_tax' => null,
                    'diesel_amt' => null,
                    'toll' => null,
                    'state_entry' => null,
                    'legal_exp' => null,
                    'food' => null,
                    'maintainance' => null,
                    'parking' => null,
                    'dalla' => null,
                    'other1' => null,
                    'other2' => null,
                    'other3' => null,
                    'total' => null,
                    'checked' => true
                ];
            }
        }

        if($against=='All'){
            $data = collect($data)->sortBy('date')->values()->all();


        }

        $unmarked_diesel_amt = DieselReceipt::whereBetween('booking_date',[$from_date, $till_date])
        ->where('driver_on_account',$driver_account_id)
        ->whereNull('booking_id')
        ->where('settel_status',0)
        ->sum('amount');
    
        $unmarked_diesel_qty = DieselReceipt::whereBetween('booking_date',[$from_date, $till_date])
            ->where('driver_on_account',$driver_account_id)
            ->whereNull('booking_id')
            ->where('settel_status',0)
            ->sum('qty');

        $unmarked_pump_advance_amt = DieselReceipt::whereBetween('booking_date',[$from_date, $till_date])
            ->where('driver_on_account',$driver_account_id)
            ->whereNull('booking_id')
            ->where('settel_status',0)
            ->sum('cash_amount'); // Changed to correct field for pump advance amount


            $FASTAG_EXPENSE = $this->get_acc_id('FASTAG EXPENSE');
            $fastag_amt = 0;
            if (isset($vehicle_id) && $FASTAG_EXPENSE) {
                $voucher_sum = Voucher::select('id', 'branch_id', 'total_dr_amount', 'total_cr_amount', 'narration', 'voucher_number', 'voucher_date', 'bank_date', 'voucher_name', 'created_by')
                ->whereBetween('voucher_date', [$from_date, $till_date]);

                

                $voucher_sum->whereHas('voucher_acc_details', function ($qr) use ($FASTAG_EXPENSE) {
                            // Adds a condition to filter voucher_acc_details where related account's account_name matches party_name
                            $qr->whereHas('account', function ($sqr) use ($FASTAG_EXPENSE) {
                                $sqr->where('account_name', 'FASTAG EXPENSE');
                            });
                        });

                $voucher_sum->whereHas('voucher_descs', function ($qr) use ($vehicle_id) {
                                    $qr->where('against_id', $vehicle_id)
                                    ->where('against', 'VEHICLE_COST_CENTER');
                                });

                // Sum the receipt_amt for the filtered voucher_descs


                $fastag_amt = $voucher_sum->sum('total_dr_amount');
            }

        

            return response()->json([
                'against_details' => $data,
                'advance_without_against' => $advance_without_against,
                'unmarked_diesel_amt' => $unmarked_diesel_amt,
                'fastag_amt' => $fastag_amt,
                'unmarked_diesel_qty' => $unmarked_diesel_qty,
                'unmarked_pump_advance_amt' => $unmarked_pump_advance_amt
            ]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'DriverSettlement');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = DriverSettlement::with([
                'branch:id,branch_name,branch_code',
                'vehicle:id,vehicle_number',
                'driver:id,emp_name'
                ])
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('ds_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = DriverSettlement::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $d_settlementsQuery = DriverSettlement::select('id', 'ds_number')->where('fin_year',$request->current_year);
        
        if (!empty($request->user_branch_ids)) {
            $d_settlementsQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        
        $d_settlements = $d_settlementsQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['driver_settlements' => $d_settlements]);
    }
    


    public function get_next_ds_number(Request $request){
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        $max = DriverSettlement::where('branch_id',$branch_id)->max('max_number');
        $d_settlement_number = $branch_code .'-'.'DS'.'-'.sprintf('%06d', $max+1);
        return response()->json(['ds_number' => strtoupper($d_settlement_number), 'max_number' => $max+1]);
    }

    public function get_driver_settlement_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = DriverSettlement::with(['branch:id,branch_name,branch_code','vehicle:id,vehicle_number', 'driver:id,emp_name','d_settlement_desc'])->where('fin_year',$request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['ds_num_from']) && isset($report_data['ds_num_to'])){
            $query->where('ds_number','>=',$report_data['ds_num_from']['ds_number'])->where('ds_number','<=',$report_data['ds_num_to']['ds_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('settle_date','>=',$report_data['from_date'])->where('settle_date','<=',$report_data['to_date']);
        }
        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('ds_number', 'like','%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_pending_settlement_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;

        $vehicle_id = $report_data['vehicle'] ? $report_data['vehicle']['id'] : null;
        $driver_id = $report_data['driver'] ? $report_data['driver']['id'] : null;
        $from_date = $report_data['from_date'];
        $till_date = $report_data['to_date'];
        $vehicle_account_id = $report_data['vehicle'] ? $report_data['vehicle']['account_id'] : null;
        $driver_account_id = $report_data['driver'] ? $report_data['driver']['account_id'] : null;

        $against = $report_data['against'];

        $data = [];

        $company_name = $request->company['company_name'];

        if($against == 'GR' || $against == 'All'){
            $against_details = BookGr::select('id', 'gr_number', 'gr_date', 'vehicle_id', 'driver_id', 'source_id', 'destination_id', 'grand_total')
                    ->with([
                        'vehicle:id,vehicle_number',
                        'driver:id,emp_name',
                        'source:id,station_name',
                        'destination:id,station_name'
                    ])
                    ->where('is_cancelled', false)
                    ->whereDoesntHave('d_settlement_desc', function ($query) {
                        $query->where('against', 'GR');
                    })
                    ->where(function ($query) use ($vehicle_id, $driver_id) {
                        if ($vehicle_id) {
                            $query->where('vehicle_id', $vehicle_id);
                        }
                        if ($driver_id) {
                            $query->where('driver_id', $driver_id);
                        }
                    })
                    ->whereBetween('gr_date', [$from_date, $till_date])
                    ->whereHas('vehicle', function($query) use ($company_name) {
                        $query->where('self_market','SELF');
                    })
                    ->get();

                  
            
            foreach($against_details as $ad){
                $data[] = [
                    'against_id' => $ad->id,
                    'against'=>'GR',
                    'against_number'=>$ad->gr_number,
                    'date' => $ad->gr_date, 
                    'vehicle_number' => $ad->vehicle ? $ad->vehicle->vehicle_number : null,
                    'driver' => $ad->driver ? $ad->driver->emp_name : null,
                    'source'=> $ad->source ? $ad->source->station_name : null,
                    'destination'=> $ad->destination ? $ad->destination->station_name : null,
                ];
            }
        }
        
        if($against == 'Loading Slip' || $against == 'All'){
            $against_details = LoadingSlip::select('id','ls_number','ls_date','vehicle_id','driver_id','source_id','destination_id','total_without_gst','driver_payment_amt')->with([
                                    'vehicle:id,vehicle_number',
                                    'driver:id,account_name',
                                    'source:id,station_name',
                                    'destination:id,station_name',
                                    'd_settlement_desc'
            ])
            ->whereDoesntHave('d_settlement_desc', function ($query) {
                $query->where('against', 'Loading Slip');
            })
            ->where(function ($query) use ($vehicle_id, $driver_account_id) {
                if ($vehicle_id) {
                    $query->where('vehicle_id', $vehicle_id);
                }
                if ($driver_account_id) {
                    $query->where('driver_id', $driver_account_id);
                }
            })
            ->whereHas('vehicle', function($query) use ($company_name) {
                $query->where('self_market','SELF');
            })
            ->whereBetween('ls_date', [$from_date, $till_date]);

            // Get the results
            $against_details = $against_details->get();
            


            foreach($against_details as $ad){
                $data[] = [
                    'against_id' => $ad->id,
                    'd_settlement_desc' => $ad->d_settlement_desc,
                    'against'=>'Loading Slip',
                    'against_number'=>$ad->ls_number,
                    'date' => $ad->ls_date, 
                    'vehicle_number' => $ad->vehicle ? $ad->vehicle->vehicle_number : null,
                    'driver' => $ad->driver ? $ad->driver->account_name : null,
                    'source'=> $ad->source ? $ad->source->station_name : null,
                    'destination'=> $ad->destination ? $ad->destination->station_name : null,
                ];
            }

        }
        
        if($against == 'Challan' || $against == 'All'){
            $against_details = Challan::select('id', 'challan_number', 'challan_date', 'vehicle_id', 'driver_id', 'source_id', 'destination_id', 'total_freight')
                                ->with([
                                    'vehicle:id,vehicle_number',
                                    'driver:id,emp_name',
                                    'source:id,station_name',
                                    'destination:id,station_name'
                                ])
                                ->whereDoesntHave('d_settlement_desc', function ($query) {
                                    $query->where('against', 'Challan');
                                })
                                ->whereBetween('challan_date', [$from_date, $till_date])
                                ->where(function ($query) use ($vehicle_id, $driver_id) {
                                    if ($vehicle_id) {
                                        $query->where('vehicle_id', $vehicle_id);
                                    }
                                    if ($driver_id) {
                                        $query->where('driver_id', $driver_id);
                                    }
                                })
                                ->whereHas('vehicle', function($query) use ($company_name) {
                                    $query->where('self_market','SELF');
                                })
                                ->get();

             
            
            foreach($against_details as $ad){
                $data[] = [
                    'against_id' => $ad->id,
                    'against'=>'Challan',
                    'against_number'=>$ad->challan_number,
                    'date' => $ad->challan_date, 
                    'vehicle_number' => $ad->vehicle ? $ad->vehicle->vehicle_number : null,
                    'driver' => $ad->driver ? $ad->driver->emp_name : null,
                    'source'=> $ad->source ? $ad->source->station_name : null,
                    'destination'=> $ad->destination ? $ad->destination->station_name : null,
                ];
            }
        }

        if($against=='All'){
            $data = collect($data)->sortBy('date')->values()->all();


        }


    
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $d_settlement = DriverSettlement::with([
            'branch:id,branch_name,branch_code',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name,account_id',
            'd_settlement_desc'
        ])->find($pk);

        
            $d_settlement->d_settlement_desc->transform(function ($desc) {
                if($desc->against == 'GR'){
                    $book_gr = BookGr::select('gr_date','acc_party_id','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                    $date = $book_gr->gr_date;
                    $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : null;
                    $source = $book_gr->source ? $book_gr->source->station_name : null;
                    $destination = $book_gr->destination ? $book_gr->destination->station_name : null;
                }
                elseif($desc->against == 'Loading Slip'){
                    $l_slip = LoadingSlip::select('ls_date','acc_party_id','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                    $date = $l_slip->ls_date;
                    $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : null;
                    $source = $l_slip->source ? $l_slip->source->station_name : null;
                    $destination = $l_slip->destination ? $l_slip->destination->station_name : null;
                }
                elseif($desc->against == 'Challan'){
                    $challan = Challan::select('challan_date','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name'])->find($desc->against_id);
                    $date = $challan->challan_date;
                    $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : null;
                    $source = $challan->source ? $challan->source->station_name : null;
                    $destination = $challan->destination ? $challan->destination->station_name : null;
                }
                    
                    return [
                        'd_settlement_desc_id' => $desc->id,
                        'against' => $desc->against,
                        'against_number' => $desc->against_number,
                        'against_id' => $desc->against_id,
                        'vehicle_number' => $vehicle,
                        'source' => $source,
                        'destination' => $destination,
                        'date' => $date,
                        'diesel_receipt_amt' => $desc->diesel_receipt_amt,
                        'diesel_receipt_qty' => $desc->diesel_receipt_qty,
                        'alloted_dsl' => $desc->alloted_dsl,
                        'dsl_diff' => $desc->dsl_diff,
                        'dsl_rate' => $desc->dsl_rate,
                        'dsl_diff_amount' => $desc->dsl_diff_amount,
                        'freight' => $desc->freight,
                        'advance' => $desc->advance,
                        'reading' => $desc->reading,
                        'return_freight' => $desc->return_freight,
                        'green_tax' => $desc->green_tax,
                        'diesel_amt' => $desc->diesel_amt,
                        'toll' => $desc->toll,
                        'state_entry' => $desc->state_entry,
                        'legal_exp' => $desc->legal_exp,
                        'food' => $desc->food,
                        'maintainance' => $desc->maintainance,
                        'parking' => $desc->parking,
                        'dalla' => $desc->dalla,
                        'other1' => $desc->other1,
                        'other2' => $desc->other2,
                        'other3' => $desc->other3,
                        'total' => $desc->total
                    ];
            });

        return ['driver_settlement' =>$d_settlement];
    }


    public function get_custom_fields(Request $request){
        $d_settlement = DriverSettlement::select($request->custom_fields)->with($request->with_relations)->find($request->lh_id);
        return response()->json(['driver_settlement' => $d_settlement]);
    }

    protected function get_max_number_from_ds_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected  function  record_transaction_in_accounts($request,$d_settlement){
        $t_data = [];
        $t_data['t_against'] = 'Driver Settlement';
        $t_data['t_against_id'] = $d_settlement->id;
        $t_data['t_against_no'] = $d_settlement->ds_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->settle_date;
        $t_data['voucher_time'] = null;
        
        $t_data['t_against_narration'] = $request->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($d_settlement->driver_expense!=0 && $d_settlement->driver_account_id){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT

            if($d_settlement->driver_expense>0){
            // TRIP EXPENSE ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>19, 'nature'=>'dr' , 'dr_amount'=>$d_settlement->driver_expense, 'cr_amount'=>0 ];
            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$d_settlement->driver_account_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$d_settlement->driver_expense ];
            }
            else if($d_settlement->driver_expense<0){
                // TRIP EXPENSE ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>19, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>abs($d_settlement->driver_expense) ];
                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$d_settlement->driver_account_id, 'nature'=>'dr' , 'dr_amount'=>abs($d_settlement->driver_expense), 'cr_amount'=>0 ];
            }
            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }


    }

}
