<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\Master\IssueBooks;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class LoadingSlipController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'LoadingSlip');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $loading_slips = LoadingSlip::with([
                    'branch:id,branch_name,branch_code',
                    'source:id,station_name',
                    'destination:id,station_name',
                    'destination1:id,station_name',
                    'destination2:id,station_name',
                    'vehicle:id,vehicle_number',
                    'acc_party:id,account_name',
                    'acc_party_branch:id,branch_name',
                    'driver:id,account_name',
                    'driver_payment_from:id,account_name',
                    'advance_received_in:id,account_name',
                    'balance_received_in:id,account_name',
                    'income:id,account_name'
                    ])->where('fin_year',$request->current_year)->get();
        return response()->json($loading_slips);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $ls = LoadingSlip::with([
                    'branch:id,branch_name,branch_code',
                    'source:id,station_name',
                    'destination:id,station_name',
                    'destination1:id,station_name',
                    'destination2:id,station_name',
                    'vehicle:id,vehicle_number',
                    'acc_party:id,account_name',
                    'acc_party_branch:id,branch_name',
                    'acc_party.party:id,account_id,party_name',
                    'driver:id,account_name',
                    'driver_payment_from:id,account_name',
                    'advance_received_in:id,account_name',
                    'balance_received_in:id,account_name',
                    'income:id,account_name'
                    ])->find($pk);

        return response()->json(['loading_slip' => $ls]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'LoadingSlip');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'ls_type' => 'required',
            'ls_number' => 'required|unique:loading_slips,ls_number',
            'source' => 'required',
            'destination' => 'required',
            'acc_party' => 'required',
            'vehicle' => 'required',
        ], [
            'branch.required' => 'Required',
            'ls_type.required' => 'Required',
            'ls_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'ls_number.unique' => 'Already Exist !',
            'acc_party.required' => 'required',
            'vehicle.required' => 'required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            $max_number = $this->get_max_number_from_ls_number($request->ls_number);        
            
            // SAVING A NEW LS
            $ls = new LoadingSlip();
            $ls->branch_id = $request->branch['id'];
            $ls->ls_type = $request->ls_type;
            $ls->prefix = $request->prefix;
            $ls->max_number = $max_number;
            $ls->ls_number = $request->ls_number;
            $ls->acc_party_id = $request->acc_party['id'];
            $ls->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;

            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $ls->vehicle_id = $vehicle_id;
            $ls->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;

            $ls->ls_date = $request->ls_date;
            $ls->ls_time = $request->ls_time;
            $ls->income = $request->income ? $request->income['id'] : null ;
            $ls->source_id = $request->source['id'];
            $ls->destination1_id = $request->destination1?$request->destination1['id']:null;
            $ls->destination2_id = $request->destination2?$request->destination2['id']:null;
            $ls->destination_id = $request->destination['id'];
            $ls->gr_number = $request->gr_number;
            $ls->party_name = $request->party_name;
            $ls->start_reading = $request->start_reading;
            $ls->end_reading = $request->end_reading;
            $ls->mileage = $request->mileage;
            $ls->net_km = $request->net_km;
            
            $ls->bill_status = false;
            
            $ls->bilty_type = $request->bilty_type;
            $ls->delivery_pt = $request->delivery_pt;
            $ls->pay_mode = $request->pay_mode;
            $ls->material_name = $request->material_name;
            $ls->chg_unit = $request->chg_unit;
            $ls->material_weight = $request->material_weight;
            $ls->pkg = $request->pkg;
            $ls->rate = $request->rate;
            $ls->to_pay_amt = $request->to_pay_amt;
            $ls->reporting_date = $request->reporting_date;
            $ls->unload_date = $request->unload_date;
            $ls->pod_receiving_date = $request->pod_receiving_date;
            $ls->pod_submission_date = $request->pod_submission_date;
            $ls->remarks = $request->remarks;

            $ls->freight = $request->freight;
            $ls->munshiana = $request->munshiana;
            $ls->dalla = $request->dalla;
            $ls->advance_received_in = $request->advance_received_in ? $request->advance_received_in['id'] : null;
            $ls->advance_amt = $request->advance_amt;
            $ls->advance_date = $request->advance_date;
            $ls->balance_date = $request->balance_date;
            $ls->balance_received_in = $request->balance_received_in ?  $request->balance_received_in['id'] : null;
            $ls->balance_received_in_amt = $request->balance_received_in_amt;
            $ls->diesel_rate = $request->diesel_rate;
            $ls->diesel_qty = $request->diesel_qty;
            $ls->diesel_amt = $request->diesel_amt;
            $ls->labour = $request->labour;
            $ls->other_plus = $request->other_plus;
            $ls->detention = $request->detention;
            $ls->detention_days = $request->detention_days;
            $ls->unload_detention = $request->unload_detention;
            $ls->balance_receivable = $request->balance_receivable;
            $ls->total_without_gst = $request->total_without_gst;

            $ls->tds_percent = $request->tds_percent;
            $ls->tds_amount = $request->tds_amount;
            $ls->igst_percent = $request->igst_percent;
            $ls->cgst_percent = $request->cgst_percent;
            $ls->sgst_percent = $request->sgst_percent;
            $ls->igst_amount = $request->igst_amount;
            $ls->cgst_amount = $request->cgst_amount;
            $ls->sgst_amount = $request->sgst_amount;
            $ls->grand_total = $request->grand_total;

            $ls->unloaded_wt = $request->unloaded_wt;
            $ls->gate_pass = $request->gate_pass;
            $ls->pod_pic = $request->pod_pic;


            $ls->driver_id = $request->driver ? $request->driver['id'] : null;
            $ls->driver_payment_from = $request->driver_payment_from ? $request->driver_payment_from['id'] :null;
            $ls->driver_payment_amt = $request->driver_payment_amt;

            $ls->gr_pic1 = $request->gr_pic1;
            $ls->other_pic = $request->other_pic;

            $ls->fin_year = $request->current_year; 

            $ls->created_by = $request->user->id;

            // Save the model to the database
            $ls->save();

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $ls);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($ls->id);
            $this->createEditLog('created', 'loading_slips', $ls->id, $ls->ls_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$ls->id, 'ls_number'=>$ls->ls_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'LoadingSlip');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $ls = LoadingSlip::find($pk);

        $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'ls_id', $pk);
        $voucher_check  = $this->check_in_voucher_desc('voucher_number','voucher_number','Loading Slip',$ls->ls_number);
       // $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','Loading Slip',$ls->ls_number);

       $lorry_hire_check['status'] = false;
       
        $diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','Loading Slip',$ls->ls_number);
        $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','Loading Slip',$ls->ls_number);
                
        if($bill_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status']){
            $status = $bill_check['status'] ? $bill_check : ($voucher_check['status'] ? $voucher_check : ($lorry_hire_check['status'] ? $lorry_hire_check : ($diesel_receipt_check['status'] ? $diesel_receipt_check : $driver_settlement_desc_check)));
            return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($ls->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'ls_type' => 'required',
            'ls_number' => 'required',
            'source' => 'required',
            'destination' => 'required',
            'acc_party' => 'required',
            'vehicle' => 'required',
        ], [
            'branch.required' => 'Required',
            'ls_type.required' => 'Required',
            'ls_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'acc_party.required' => 'required',
            'vehicle.required' => 'required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW LS
            $ls->acc_party_id = $request->acc_party['id'];
            $ls->acc_party_branch_id = $request->acc_party_branch ? $request->acc_party_branch['id'] : null;

            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $ls->vehicle_id = $vehicle_id;
            $ls->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            
            $ls->ls_date = $request->ls_date;
            $ls->ls_time = $request->ls_time;
            $ls->income = $request->income ? $request->income['id'] : null;
            $ls->source_id = $request->source['id'];
            $ls->destination1_id = $request->destination1?$request->destination1['id']:null;
            $ls->destination2_id = $request->destination2?$request->destination2['id']:null;
            $ls->destination_id = $request->destination['id'];
            $ls->gr_number = $request->gr_number;
            $ls->party_name = $request->party_name;
            $ls->start_reading = $request->start_reading;
            $ls->end_reading = $request->end_reading;
            $ls->mileage = $request->mileage;
            $ls->net_km = $request->net_km;

            $ls->bilty_type = $request->bilty_type;
            $ls->delivery_pt = $request->delivery_pt;
            $ls->pay_mode = $request->pay_mode;
            $ls->material_name = $request->material_name;
            $ls->chg_unit = $request->chg_unit;
            $ls->material_weight = $request->material_weight;
            $ls->pkg = $request->pkg;
            $ls->rate = $request->rate;
            $ls->to_pay_amt = $request->to_pay_amt;
            $ls->reporting_date = $request->reporting_date;
            $ls->unload_date = $request->unload_date;
            $ls->pod_receiving_date = $request->pod_receiving_date;
            $ls->pod_submission_date = $request->pod_submission_date;
            $ls->remarks = $request->remarks;

            $ls->freight = $request->freight;
            $ls->munshiana = $request->munshiana;
            $ls->dalla = $request->dalla;
            $ls->advance_received_in = $request->advance_received_in ? $request->advance_received_in['id'] : null;
            $ls->advance_amt = $request->advance_amt;
            $ls->advance_date = $request->advance_date;
            $ls->balance_date = $request->balance_date;
            $ls->balance_received_in = $request->balance_received_in ? $request->balance_received_in['id'] : null;
            $ls->balance_received_in_amt = $request->balance_received_in_amt;
            $ls->diesel_rate = $request->diesel_rate;
            $ls->diesel_qty = $request->diesel_qty;
            $ls->diesel_amt = $request->diesel_amt;
            $ls->labour = $request->labour;
            $ls->other_plus = $request->other_plus;
            $ls->detention = $request->detention;
            $ls->detention_days = $request->detention_days;
            $ls->unload_detention = $request->unload_detention;
            $ls->balance_receivable = $request->balance_receivable;
            $ls->total_without_gst = $request->total_without_gst;

            $ls->tds_percent = $request->tds_percent;
            $ls->tds_amount = $request->tds_amount;
            $ls->igst_percent = $request->igst_percent;
            $ls->cgst_percent = $request->cgst_percent;
            $ls->sgst_percent = $request->sgst_percent;
            $ls->igst_amount = $request->igst_amount;
            $ls->cgst_amount = $request->cgst_amount;
            $ls->sgst_amount = $request->sgst_amount;
            $ls->grand_total = $request->grand_total;


            $ls->driver_id = $request->driver ? $request->driver['id'] : null;
            $ls->driver_payment_from = $request->driver_payment_from ? $request->driver_payment_from['id'] :null;
            $ls->driver_payment_amt = $request->driver_payment_amt;

            $ls->unloaded_wt = $request->unloaded_wt;
            $ls->gate_pass = $request->gate_pass;
            $ls->pod_pic = $request->pod_pic;
            

            $ls->gr_pic1 = $request->gr_pic1;
            $ls->other_pic = $request->other_pic;

            $ls->updated_by = $request->user->id;

            $ls->save();
            // NEW LS SAVED

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $ls);
            // TRANSACTION RECORDED IN ACCOUNTS
            
            
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($ls->id);
            $this->createEditLog('updated', 'loading_slips', $ls->id, $ls->ls_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'LoadingSlip');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
        
        $ls = LoadingSlip::find($pk);

        $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'ls_id', $pk);
        $voucher_check  = $this->check_in_voucher_desc('voucher_number','voucher_number','Loading Slip',$ls->ls_number);
        $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','Loading Slip',$ls->ls_number);
        $diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','Loading Slip',$ls->ls_number);
        $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','Loading Slip',$ls->ls_number);
                
        if($bill_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status']){
            $status = $bill_check['status'] ? $bill_check : ($voucher_check['status'] ? $voucher_check : ($lorry_hire_check['status'] ? $lorry_hire_check : ($diesel_receipt_check['status'] ? $diesel_receipt_check : $driver_settlement_desc_check)));
            return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($ls->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'LOADING SLIP';
        $t_data['t_against_no'] = $ls->ls_number;
        $t_data['t_against_id'] = $ls->id;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        //FIND AND DELETE THE LS
        $ls->delete();

        $this->createEditLog('deleted', 'loading_slips', $ls->id, $ls->ls_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = LoadingSlip::select('id', 'ls_number','vehicle_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

       // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('ls_number', 'like', '%'. $searchVariable . '%')
            ->orWhere('vehicle_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $loading_slips = $query->get();
        return response(['loading_slips' => $loading_slips, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'LoadingSlip');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = LoadingSlip::select('id', 'ls_number', 'acc_party_id', 'ls_date' , 'branch_id','source_id', 'destination_id', 'vehicle_id', 'driver_id', 'driver_payment_from', 'advance_received_in', 'balance_received_in', 'income')
                ->with(['branch:id,branch_name,branch_code','source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number','acc_party:id,account_name','acc_party_branch:id,branch_name','driver:id,account_name','driver_payment_from:id,account_name','advance_received_in:id,account_name','balance_received_in:id,account_name','income:id,account_name']) 
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('ls_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = LoadingSlip::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $loadingSlipsQuery = LoadingSlip::select('id', 'ls_number')->where('fin_year',$request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $loadingSlipsQuery->whereIn('branch_id', $request->user_branch_ids);
        }
    
        $loadingSlips = $loadingSlipsQuery->orderBy('id', 'desc')->take(15)->get();
    
        return response()->json(['loading_slips' => $loadingSlips]);
    }
    


    public function get_next_ls_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        if($type=='SYSTEM'){
            $max = LoadingSlip::where('branch_id',$branch_id)->where('ls_type', $type)->max('max_number');
            $ls_number = $branch_code.'-'.sprintf('%06d', $max+1);
            return response()->json(['ls_number' => strtoupper($ls_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = LoadingSlip::where('prefix',$prefix)->where('ls_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','LoadingSlip')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $ls_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['ls_number' => strtoupper($ls_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','LoadingSlip')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $ls_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['ls_number' => strtoupper($ls_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['ls_number' => ['No Valid LS Number']]],422);
            }
        }
    }

    public function get_loading_slip_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = LoadingSlip::with(['branch:id,branch_name,branch_code','vehicle:id,vehicle_number','acc_party:id,account_name','acc_party_branch:id,branch_name','driver:id,account_name','source:id,station_name','destination1:id,station_name',
                    'destination2:id,station_name','destination:id,station_name','voucher_desc'=>function($query){
            $query->where('against','Loading Slip')->where('voucher_name','Receipt Voucher');
        },
        'lorry_hire_desc',
        'lorry_hire_desc.lorry_hire:id,lh_number,net_freight',
        ])->where('fin_year',$request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['acc_party_branch']) ){
            $query->where('acc_party_branch_id', $report_data['acc_party_branch']['id']);
        }

        if(isset($report_data['ls_num_from']) && isset($report_data['ls_num_to'])){
            $query->where('ls_number','>=',$report_data['ls_num_from']['ls_number'])->where('ls_number','<=',$report_data['ls_num_to']['ls_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('ls_date','>=',$report_data['from_date'])->where('ls_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['paid_status']) ){
            if($report_data['paid_status']=='Paid')
            $query->where('paid_status', 1)->orWhere('balance_receivable', 0);
            else if($report_data['paid_status']=='Outstanding')
            $query->where('paid_status', 0)->where('balance_receivable', '>',0);
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('ls_number', 'like','%'. $searchValue . '%');
            });
        }

        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $ls = LoadingSlip::with([
                    'branch:id,branch_name,branch_code',
                    'source:id,station_name',
                    'destination:id,station_name',
                    'destination1:id,station_name',
                    'destination2:id,station_name',
                    'vehicle:id,vehicle_number',
                    'acc_party:id,account_name',
                    'acc_party_branch:id,branch_name',
                    'driver:id,account_name',
                    'driver_payment_from:id,account_name',
                    'advance_received_in:id,account_name',
                    'balance_received_in:id,account_name',
                    'income:id,account_name'
            ])->find($pk);

        return ['loading_slip' => $ls];
    }

    protected function get_max_number_from_ls_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function record_transaction_in_accounts($request, $ls){
        $t_data = [];
        $t_data['t_against'] = 'LOADING SLIP';
        $t_data['t_against_id'] = $ls->id;
        $t_data['t_against_no'] = $ls->ls_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->ls_date;
        $t_data['voucher_time'] = $request->ls_time;
        $t_data['t_against_narration'] = $ls->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($ls->acc_party_id && $ls->income && $request->grand_total>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$request->grand_total + $request->tds_amount, 'cr_amount'=>0 ];
            
            // CREDIT SIDE
            if($request->cgst_amount > 0){
                // CREDIT CGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->cgst_amount ];
            }
            if($request->sgst_amount > 0){
                // CREDIT SGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->sgst_amount ];
            }
            if($request->igst_amount > 0){
                // CREDIT IGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->igst_amount ];
            }

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->income, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->total_without_gst ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 0;
        if($ls->acc_party_id && $ls->tds_amount>0){
            $t_data['voucher_date'] = $ls->advance_date??$request->ls_date;

            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>3, 'nature'=>'dr' , 'dr_amount'=>$request->tds_amount, 'cr_amount'=>0 ];

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->tds_amount ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }
        

        $t_data['cancellable'] = 0;
        if($ls->advance_received_in && $ls->acc_party_id && $ls->advance_amt >0){
            $t_data['voucher_date'] = $ls->advance_date??$request->ls_date;
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->advance_received_in, 'nature'=>'dr' , 'dr_amount'=>$ls->advance_amt, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$ls->advance_amt ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);
                
                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }
        $t_data['cancellable'] = 0;
        if($ls->balance_received_in && $ls->acc_party_id && $ls->balance_received_in_amt >0){
            $t_data['voucher_date'] = $ls->balance_date??$request->ls_date;


            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->balance_received_in, 'nature'=>'dr' , 'dr_amount'=>$ls->balance_received_in_amt, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$ls->balance_received_in_amt ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);
                
                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }
        $t_data['cancellable'] = 0;
        if($ls->driver_id && $ls->driver_payment_from && $ls->driver_payment_amt >0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->driver_id, 'nature'=>'dr' , 'dr_amount'=>$ls->driver_payment_amt, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->driver_payment_from, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$ls->driver_payment_amt ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);
                
                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 0;
        if($ls->diesel_amt>0 && $ls->acc_party_id ){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>18, 'nature'=>'dr' , 'dr_amount'=>$ls->diesel_amt, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$ls->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$ls->diesel_amt ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);
                
                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }
    }
}
