<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\LorryHireBill;
use App\Models\FleetOwner\LorryHireBillDesc;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\GoodsMove\Challan;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class LorryHireBillController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'LorryHireBill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $lorry_hire_bill = LorryHireBill::with([
            'branch:id,branch_name,branch_code',
            'lorry_hire_from:id,account_name',
            'vehicle:id,vehicle_number',
            'lhb_desc',
            'lhb_desc.challan:id,challan_number,challan_date,vehicle_number,lh_net_freight',
            'lhb_desc.challan_desc',
            'lhb_desc.challan_desc.bookGr:id,gr_number,gr_date',
        ])->where('fin_year',$request->current_year)->get();

        $lorry_hire_bill->lhb_desc->transform(function ($desc) use ($lorry_hire_bill)  {  
            return [
                'lhb_desc_id' => $desc->id,
                'lhb_id' => $lorry_hire_bill->id,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_date' => $desc->challan->challan_date,

                'gr_number' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_number : '',
                'gr_date' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_date : '',
                'vehicle_number' => $desc->challan ? $desc->challan->vehicle_number : '',

                'lh_freight' => $desc->lh_freight,
                'balance' => $desc->balance,
                
                'lh_net_freight' => $desc->lh_net_freight,
                'total_lh_deduction' => $desc->total_lh_deduction,
                'total_without_gst'=> $desc->total_without_gst
            ];
        });

        return response()->json($lorry_hire_bill);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $lorry_hire_bill = LorryHireBill::with([
            'branch:id,branch_name,branch_code',
            'lorry_hire_from:id,account_name',
            'vehicle:id,vehicle_number',
            'lhb_desc',
            'lhb_desc.challan:id,challan_number,challan_date,vehicle_number,lh_net_freight',
            'lhb_desc.challan_desc',
            'lhb_desc.challan_desc.bookGr:id,gr_number,gr_date',
        ])->find($pk);

        $lorry_hire_bill->lhb_desc->transform(function ($desc) use ($lorry_hire_bill)  {  
            return [
                'lhb_desc_id' => $desc->id,
                'lhb_id' => $lorry_hire_bill->id,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_date' => $desc->challan->challan_date,
                'lh_net_freight' => $desc->lh_net_freight,
                'total_lh_deduction' => $desc->total_lh_deduction,
                'total_without_gst'=> $desc->total_without_gst,
                'lh_freight' => $desc->lh_freight,
                'balance' => $desc->balance,

                'gr_number' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_number : '',
                'gr_date' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_date : '',
                'vehicle_number' => $desc->challan ? $desc->challan->vehicle_number : '',

            ];
        });


        return response()->json(['lorry_hire_bill' =>$lorry_hire_bill]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'LorryHireBill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'lhb_number' => 'required|unique:lorry_hire_bills,lhb_number',
            'lorry_hire_from' => 'required',
        ], [
            'branch.required' => 'Required',
            'lhb_number.required' => 'Required',
            'lorry_hire_from.required' => 'Required',
            'lhb_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE COVER TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_lhb_number($request->lhb_number);
            
            
            // SAVING A NEW GR
            $lorry_hire_bill = new LorryHireBill();
            $lorry_hire_bill->branch_id = $request->branch['id'];
            $lorry_hire_bill->max_number = $max_number;
            $lorry_hire_bill->lhb_number = $request->lhb_number;
            $lorry_hire_bill->invoice_number = $request->invoice_number;
            $lorry_hire_bill->lorry_hire_from_id = $request->lorry_hire_from['id'];
            $lorry_hire_bill->vehicle_id = $request->vehicle?$request->vehicle['id']:null;
            $lorry_hire_bill->lhb_date = $request->lhb_date;
            $lorry_hire_bill->from_date = $request->from_date;
            $lorry_hire_bill->to_date = $request->to_date;
            $lorry_hire_bill->total_lh_freight = $request->total_lh_freight;
            $lorry_hire_bill->total_lh_balance = $request->total_lh_balance;
            $lorry_hire_bill->total_lh_net_freight = $request->total_lh_net_freight;
            $lorry_hire_bill->total_lh_deduction = $request->total_lh_deduction;
            $lorry_hire_bill->total_without_gst = $request->total_without_gst;
            $lorry_hire_bill->igst_percent = $request->igst_percent;
            $lorry_hire_bill->igst_amount = $request->igst_amount;
            $lorry_hire_bill->cgst_percent = $request->cgst_percent;
            $lorry_hire_bill->cgst_amount = $request->cgst_amount;
            $lorry_hire_bill->sgst_percent = $request->sgst_percent;
            $lorry_hire_bill->sgst_amount = $request->sgst_amount;
            $lorry_hire_bill->tds_percent = $request->tds_percent;
            $lorry_hire_bill->tds_amount = $request->tds_amount;
            $lorry_hire_bill->grand_total_amount = $request->grand_total_amount;
            $lorry_hire_bill->grand_total_balance = $request->grand_total_balance;

            $lorry_hire_bill->fin_year = $request->current_year;

            $lorry_hire_bill->created_by = $request->user->id;

            // Save the model to the database
            $lorry_hire_bill->save();

            

            // SAVING PACKAGE DETAILS
            $lhb_desc = $request->lhb_desc;
            foreach ($lhb_desc as $cld) {
                
                $lhb_desc = new LorryHireBillDesc();

                $lhb_desc->lhb_id = $lorry_hire_bill->id;
                $lhb_desc->lhb_number = $lorry_hire_bill->lhb_number;
                $lhb_desc->challan_id = $cld['challan_id'];
                $lhb_desc->challan_number = $cld['challan_number'];
                
                $lhb_desc->lh_freight = $cld['lh_freight'];
                $lhb_desc->balance = $cld['balance'];

                $lhb_desc->lh_net_freight = $cld['lh_net_freight'];
                $lhb_desc->total_lh_deduction = $cld['total_lh_deduction'];

                $lhb_desc->total_without_gst = $cld['total_without_gst'];
            
                $lhb_desc->save();
            }
            // PACKAGE DETAILS SAVED

           
            $this->record_transaction_in_accounts($request, $lorry_hire_bill);

             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($lorry_hire_bill->id);

            $this->createEditLog('created', 'lorry_hire_bill', $lorry_hire_bill->id, $lorry_hire_bill->lhb_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$lorry_hire_bill->id, 'lhb_number'=>$lorry_hire_bill->lhb_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'LorryHireBill');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $lorry_hire_bill = LorryHireBill::find($pk);

        $voucher_check2  = $this->check_in_voucher_desc('voucher_number','voucher_number','Lorry Hire Bill',$lorry_hire_bill->lhb_number);
        $voucher_check['status'] = false;



        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($lorry_hire_bill->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'lhb_number' => 'required',
            'lorry_hire_from' => 'required',
        ], [
            'branch.required' => 'Required',
            'lhb_number.required' => 'Required',
            'lorry_hire_from.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            

            // SAVING A NEW COVER
            $lorry_hire_bill->invoice_number = $request->invoice_number;
            $lorry_hire_bill->lhb_date = $request->lhb_date;
            $lorry_hire_bill->total_lh_freight = $request->total_lh_freight;
            $lorry_hire_bill->total_lh_balance = $request->total_lh_balance;
            $lorry_hire_bill->total_lh_net_freight = $request->total_lh_net_freight;
            $lorry_hire_bill->total_lh_deduction = $request->total_lh_deduction;
            $lorry_hire_bill->total_without_gst = $request->total_without_gst;
            $lorry_hire_bill->igst_percent = $request->igst_percent;
            $lorry_hire_bill->igst_amount = $request->igst_amount;
            $lorry_hire_bill->cgst_percent = $request->cgst_percent;
            $lorry_hire_bill->cgst_amount = $request->cgst_amount;
            $lorry_hire_bill->sgst_percent = $request->sgst_percent;
            $lorry_hire_bill->sgst_amount = $request->sgst_amount;
            $lorry_hire_bill->tds_percent = $request->tds_percent;
            $lorry_hire_bill->tds_amount = $request->tds_amount;
            $lorry_hire_bill->grand_total_amount = $request->grand_total_amount;
            $lorry_hire_bill->grand_total_balance = $request->grand_total_balance;

            $lorry_hire_bill->updated_by = $request->user->id;

            $lorry_hire_bill->save();
            // NEW COVER SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_lhb_desc = $request->newly_added_lhb_desc;
            foreach ($newly_added_lhb_desc as $nald) {
                $lhb_desc = new LorryHireBillDesc();

                $lhb_desc->lhb_id = $lorry_hire_bill->id;
                $lhb_desc->lhb_number = $lorry_hire_bill->lhb_number;
                $lhb_desc->challan_id = $nald['challan_id'];
                $lhb_desc->challan_number = $nald['challan_number'];
                $lhb_desc->lh_freight = $nald['lh_freight'];
                $lhb_desc->lh_net_freight = $nald['lh_net_freight'];
                $lhb_desc->total_lh_deduction = $nald['total_lh_deduction'];
                $lhb_desc->total_without_gst = $nald['total_without_gst'];
                $lhb_desc->balance = $nald['balance'];

                $lhb_desc->save();
            }
            // PACKAGE DETAILS SAVED

            // SAVING PACKAGE DETAILS
            $updated_lhb_desc = $request->updated_lhb_desc;
            foreach ($updated_lhb_desc as $udd) {
                
                $lhb_desc = LorryHireBillDesc::find($udd['lhb_desc_id']);

                // Write code for edit in future

                $lhb_desc->save();
            }
            // PACKAGE DETAILS SAVED

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_lhb_desc = $request->deleted_lhb_desc;
            if( !empty($deleted_lhb_desc) ){
                foreach ($deleted_lhb_desc as $dcd) {
                    $cld = LorryHireBillDesc::find($dcd);
                    $cld->delete();

                    $t_data2['t_against'] = 'Challan';
                    $t_data2['t_against_id'] = $cld->challan_id;
                    $t_data2['t_against_no'] = $cld->challan_number;
                    $t_data2['fin_year'] = $request->current_year;
                    //$this->set_transaction_status($t_data2, true);
                }
                    
            }
            //DELETED EXISTING PACKAGE DETAILS

            $this->record_transaction_in_accounts($request, $lorry_hire_bill);

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($lorry_hire_bill->id);
            $this->createEditLog('updated', 'lorry_hire_bill', $lorry_hire_bill->id, $lorry_hire_bill->lhb_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'LorryHireBill');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $lorry_hire_bill = LorryHireBill::find($pk);

        $voucher_check2  = $this->check_in_voucher_desc('voucher_number','voucher_number','Lorry Hire Bill',$lorry_hire_bill->lhb_number);

        if($voucher_check2['status'] ){
            $status= $voucher_check2['status'];
            return response()->json(['error'=>'Document in use', 'data'=>$voucher_check2],500);
        };


        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($lorry_hire_bill->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            $lhb_desc_ids = LorryHireBillDesc::select('id')->where('lhb_id', $pk)->pluck('id')->toArray();

            // DELETING CHALLAN DESCRIPTIONS
            foreach ($lhb_desc_ids as $pdi) {
                // MAKE Challan TRANSACTION INACTIVE
                $bill_desc = LorryHireBillDesc::find($pdi);
                
                $t_data2['t_against'] = 'Challan';
                $t_data2['t_against_id'] = $bill_desc->challan_id;
                $t_data2['t_against_no'] = $bill_desc->challan_number;
                $t_data2['fin_year'] = $request->current_year;
                //$this->set_transaction_status($t_data2, true);
            }


            $lhb_desc = LorryHireBillDesc::whereIn('id', $lhb_desc_ids)->delete();
    
            //FIND AND DELETE THE BRANCH
            $lorry_hire_bill->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY
  

            // DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'LORRY HIRE BILL';
            $t_data['t_against_id'] = $lorry_hire_bill->id;
            $t_data['t_against_no'] = $lorry_hire_bill->lhb_number;
            $t_data['fin_year'] = $request->current_year;
            $this->delete_transaction_voucher($t_data);


            $this->createEditLog('deleted', 'lorry_hire_bill', $lorry_hire_bill->id, $lorry_hire_bill->lhb_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = LorryHireBill::select('id', 'lhb_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('lhb_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $lorry_hire_bill = $query->get();
        return response(['lorry_hire_bill' => $lorry_hire_bill, 'draw' => $request->draw]);
    }

    public function get_pending_lh_bill(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = Challan::with(
                    [
                        'source:id,station_name',
                        'destination:id,station_name',
                        'challan_desc:id,challan_id,gr_id,gr_number',
                        'vehicle:id,vehicle_number,self_market,owner_id',
                        'vehicle.party:id,party_name',
                    ]
                )
                ->where('fin_year',$request->current_year)
                ->whereDoesntHave('lhb_desc');

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('challan_date','>=',$report_data['from_date'])->where('challan_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['source']) ){
            $query->where('source_id', $report_data['source']['id']);
        }

        if(isset($report_data['destination']) ){
            $query->where('destination_id', $report_data['destination']['id']);
        }


        if(isset($report_data['owner']) ){
            $query->whereHas('vehicle', function ($query) use ($report_data) {
                $query->where('owner_id', $report_data['owner']['id']);
            });
        }

        $query->whereHas('vehicle', function ($query) {
            $query->where('self_market','MARKET');
        });

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('challan_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function fetch_against_details(Request $request){
        $lorry_hire_from_id = $request->lorry_hire_from['id'];
        $vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
        $from_date = $request->from_date;
        $to_date = $request->to_date;

        $data = [];
        

        // GET ALL BILLS AGAINST THE PARTY
        // IF IT HAS A credit_note against it, then add a special field 'has_credit_note' => true in my response
        $details = Challan::select('id','challan_number','challan_number','challan_date','lh_freight','lh_net_freight','lh_balance','lorry_hire_from','vehicle_id')
                ->with([
                    'challan_desc',
                    'challan_desc.bookGr:id,gr_number,gr_date',
                    'vehicle:id,vehicle_number',
                    'lorry_hire_from:id,account_name',
                ]) 
                ->where('lorry_hire_from', $lorry_hire_from_id)
                ->when($vehicle_id, function ($query, $vehicle_id) {
                    return $query->where('vehicle_id', $vehicle_id);
                })
                ->whereBetween('challan_date', [$from_date, $to_date])
                ->whereDoesntHave('lhb_desc')
                ->get();

                
    
        foreach ($details as $challan) {
            $balance = $this->check_for_balance($challan, $challan->lh_balance, 'Challan', []);

            // Add the diesel data to the response array
            $data[] = [
                'lhb_desc_id' => null,
                'lhb_id' => null,
                'challan_id' => $challan->id,
                'challan_number' => $challan->challan_number,
                'challan_date' => $challan->challan_date,
                'gr_number' => $challan->challan_desc ? $challan->challan_desc[0]->bookGr->gr_number : '',
                'gr_date' => $challan->challan_desc ? $challan->challan_desc[0]->bookGr->gr_date : '',
                'vehicle_number' => $challan->vehicle->vehicle_number,
                
                'lh_freight' => $challan->lh_freight,
                'lh_net_freight' => $challan->lh_net_freight,
                'total_lh_deduction' =>  0,
                'balance' => $balance,
                
                'total_without_gst'=> $challan->lh_net_freight,
                'igst_amount'=>0,
                'cgst_amount'=>0,
                'sgst_amount'=>0,
                'grand_total'=>0,

                'checked' => true,
            ];
        }

        return response()->json(['bills' => $data]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'LorryHireBill');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = LorryHireBill::with([
                    'branch:id,branch_name,branch_code',
                    'lorry_hire_from:id,account_name',
                ])
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('lhb_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = LorryHireBill::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $lorry_hire_billQuery = LorryHireBill::select('id', 'lhb_number')->where('fin_year',$request->current_year);
        
        if (!empty($request->user_branch_ids)) {
            $lorry_hire_billQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        
        $lorry_hire_bill = $lorry_hire_billQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['lorry_hire_bill' => $lorry_hire_bill]);
    }
    


    public function get_next_lhb_number(Request $request){
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        $max = LorryHireBill::where('branch_id',$branch_id)->max('max_number');
        $diesel_lhb_number = $branch_code.'-'.sprintf('%06d', $max+1);
        return response()->json(['lhb_number' => strtoupper($diesel_lhb_number), 'max_number' => $max+1]);
        
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $lorry_hire_bill = LorryHireBill::with([
            'branch:id,branch_name,branch_code',
            'lorry_hire_from:id,account_name',
            'vehicle:id,vehicle_number',
            'lhb_desc',
            'lhb_desc.challan:id,challan_number,challan_date,vehicle_number,lh_net_freight',
            'lhb_desc.challan_desc',
            'lhb_desc.challan_desc.bookGr:id,gr_number,gr_date',
        ])->find($pk);


        
        $lorry_hire_bill->lhb_desc->transform(function ($desc) use ($lorry_hire_bill)  {  
            return [
                'lhb_desc_id' => $desc->id,
                'lhb_id' => $lorry_hire_bill->id,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_date' => $desc->challan->challan_date,
                'lh_freight' => $desc->lh_freight,
                'lh_net_freight' => $desc->lh_net_freight,
                'total_lh_deduction' => $desc->total_lh_deduction,
                'total_without_gst'=> $desc->total_without_gst,
                'balance' => $desc->balance,
                'gr_number' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_number : '',
                'gr_date' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_date : '',
                'vehicle_number' => $desc->challan_desc ? $desc->challan->vehicle_number : '',

            ];
        });

        return ['lorry_hire_bill' =>$lorry_hire_bill];
    }


    public function get_custom_fields(Request $request){
        $lorry_hire_bill = LorryHireBill::select($request->custom_fields)->with($request->with_relations)->find($request->lh_id);
        return response()->json(['lorry_hire_bill' => $lorry_hire_bill]);
    }

    protected function get_max_number_from_lhb_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function check_for_balance($against,$default_balance,$selection, $soft_deleted_voucher_descs){
            $filteredVoucherDesc = $against->voucher_desc->where('against', $selection)->whereNotIn('id', $soft_deleted_voucher_descs);
            
            if ($filteredVoucherDesc->isEmpty()) {
                // No voucher_desc, return grand_total_amount as balance
                return $default_balance;
            }
            
            $recievedAmtSum = $filteredVoucherDesc->sum('receipt_amt');
            $deductionSum = 0;
            
            $balance = $default_balance - $recievedAmtSum - $deductionSum ;
            return $balance;
    }


    protected function record_transaction_in_accounts($request, $lorry_hire_bill){
        $t_data = [];
        $t_data['t_against'] = 'LORRY HIRE BILL';
        $t_data['t_against_id'] = $lorry_hire_bill->id;
        $t_data['t_against_no'] = $lorry_hire_bill->lhb_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->lhb_date;
        $t_data['voucher_time'] = Carbon::now()->format('H:i:s');
        $t_data['t_against_narration'] = "LORRY HIRE BILL No: ".$lorry_hire_bill->lhb_number."  Bill No: ".$lorry_hire_bill->invoice_number;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        

        // $t_data['cancellable'] = 1;
        // if($lorry_hire_bill->lorry_hire_from_id && ($lorry_hire_bill->cgst_amount>0 || $lorry_hire_bill->sgst_amount>0 || $lorry_hire_bill->igst_amount>0)){
           
        //     $t_data['voucher_accounting_details']=[];

            

        //     // DEBIT SIDE
        //     if($lorry_hire_bill->cgst_amount > 0){ 
        //         // DEBIT DEBIT ACCOUNT
        //        $t_data['voucher_accounting_details'][] = ['account_id'=>9, 'nature'=>'dr' , 'cr_amount'=>0, 'dr_amount'=>$lorry_hire_bill->cgst_amount ];
        //     }
        //     if($lorry_hire_bill->sgst_amount > 0){
        //         // DEBIT DEBIT ACCOUNT
        //         $t_data['voucher_accounting_details'][] = ['account_id'=>10, 'nature'=>'dr' , 'cr_amount'=>0, 'dr_amount'=>$lorry_hire_bill->sgst_amount ];
        //     }
        //     if($lorry_hire_bill->igst_amount> 0){
        //         // DEBIT DEBIT ACCOUNT
        //         $t_data['voucher_accounting_details'][] = ['account_id'=>11, 'nature'=>'dr' , 'cr_amount'=>0, 'dr_amount'=>$lorry_hire_bill->igst_amount ];
        //     }

        //     // CREDIT PARTY ACCOUNT
        //     $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire_bill->lorry_hire_from_id, 'nature'=>'cr' , 'cr_amount'=>$lorry_hire_bill->igst_amount + $lorry_hire_bill->sgst_amount + $lorry_hire_bill->cgst_amount , 'dr_amount'=>0 ];
          
            
        //     //CHECK TOTAL DR AND CR
        //     $total_dr = 0;
        //     $total_cr = 0;

        //     foreach ($t_data['voucher_accounting_details'] as $vad) {
        //         $total_dr += $vad['dr_amount'];
        //         $total_cr += $vad['cr_amount'];
        //     }

        //     // Round both $total_dr and $total_cr to two decimal places
        //     $total_dr_rounded = round(floatval($total_dr), 2);
        //     $total_cr_rounded = round(floatval($total_cr), 2);

        //     // Check if they are not equal
        //     if ($total_dr_rounded != $total_cr_rounded) {
        //         // Output the values for debugging
        //         dd($total_dr_rounded, $total_cr_rounded);
                
        //         // Return response with an error message
        //         return response()->json(['message' => 'Error In Accounting Entries'], 422);
        //     }


        //     $t_data['total_dr_amount'] = $total_dr;
        //     $t_data['total_cr_amount'] = $total_cr; 

            

        //     $this->create_transaction_voucher( $t_data );
        // }

        $t_data['cancellable'] = 1;
        $lorry_hire_exp = $this->get_acc_id('Lorry Hire Freight Exp.');
        if($lorry_hire_bill->lorry_hire_from_id && ($lorry_hire_bill->tds_amount>0) ){   
            $t_data['voucher_accounting_details']=[];
        
            // DEBIT SIDE
            if($lorry_hire_bill->tds_amount > 0){ 
                // DEBIT DEBIT ACCOUNT
               $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire_exp, 'nature'=>'dr' , 'cr_amount'=>0, 'dr_amount'=>$lorry_hire_bill->tds_amount ];
            }

            // CREDIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire_bill->lorry_hire_from_id, 'nature'=>'cr' , 'cr_amount'=>$lorry_hire_bill->tds_amount , 'dr_amount'=>0 ];
          
            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 

            

            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 1;
        $tds_payable_id = $this->get_acc_id('TDS Payable');
        if($lorry_hire_bill->lorry_hire_from_id && ($lorry_hire_bill->tds_amount>0) ){
           
            $t_data['voucher_accounting_details']=[];

            

            // DEBIT SIDE
            if($lorry_hire_bill->tds_amount > 0){ 
                // DEBIT DEBIT ACCOUNT
               $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire_bill->lorry_hire_from_id, 'nature'=>'dr' , 'cr_amount'=>0, 'dr_amount'=>$lorry_hire_bill->tds_amount ];
            }

            // CREDIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$tds_payable_id, 'nature'=>'cr' , 'cr_amount'=>$lorry_hire_bill->tds_amount , 'dr_amount'=>0 ];
          
            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 

            

            $this->create_transaction_voucher( $t_data );
        }

    }

    protected function sum_common_accounts($array){
        $result = array();

        $set_of_used_account_id = [];

        foreach ($array as $sub) {
            // Find account_id and sum it
            $account_id = $sub['account_id'];

            // Loop in array and find sum of the account_id
            if(in_array($account_id, $set_of_used_account_id)){
                continue;
            }

            $sum = 0;
            foreach ($array as $sub2) {
                if ($sub2['account_id'] == $account_id) {
                    $sum += $sub2['amount'];
                }
            }


            $set_of_used_account_id[] = $account_id;


            $result[] = ['account_id' => $account_id, 'amount' => $sum, 'account_name'=>$this->get_acc_name($account_id)];
            
        }
        return $result;
    }


}
