<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\TruckStatus;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\GoodsMove\Challan;
use App\Models\Master\Vehicle;
use App\Models\Master\Station;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class TruckStatusController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    // public function show_truck_status(Request $request){
    //     $query = BookGr::with(['branch:id,branch_name,branch_code','vehicle:id,vehicle_number','challan_desc','pod_desc','challan_desc.challan:id,challan_number,lorry_hire_from,vehicle_id','challan_desc.challan.lorry_hire_from:id,account_name','challan_desc.challan.lorry_hire_from.party:id,party_name,account_id,mob_number'])->where('fin_year',$request->current_year);

    //     if($request->vehicle){
    //         $query->where('vehicle_id',$request->vehicle['id']);
    //     }

    //     if($request->source){
    //         $query->where('source_id', $request->source['id']);
    //     }

    //     if($request->destination){
    //         $query->where('destination_id',$request->destination['id']);
    //     }

    //     if($request->from_date){
    //         $query->where('gr_date','>=',$request->from_date);
    //     }

    //     if($request->to_date){
    //         $query->where('gr_date','<=',$request->to_date);
    //     }

    //     if ($request->truck_status) {
    //         $query->where(function($subQuery) use ($request) {
    //             if ($request->truck_status == 'IN TRANSIT') {
    //                 $subQuery->where('truck_status', $request->truck_status)
    //                          ->orWhereNull('truck_status');
    //             } elseif ($request->truck_status == 'NOT UNLOADED') {
    //                 $subQuery->where('truck_status', '!=', 'UNLOADED')
    //                          ->orWhereNull('truck_status');
    //             } else {
    //                 $subQuery->where('truck_status', $request->truck_status);
    //             }
    //         });
    //     }

    //     if ($request->lorry_hire_from) {
    //         $query->whereHas('challan_desc.challan.lorry_hire_from', function ($q) use ($request) {
    //             $q->where('id', $request->lorry_hire_from['id']);
    //         });
    //     }

    
    //     $result = $query->orderBy('gr_date')->get();

    //     // Map booking_number
    //     $resp_data = [];

    //     foreach ($result as $item) {
    //         $copy_of_challan = count($item->challan_desc) ? json_decode(json_encode($item->challan_desc[0])) : null;
    //         $resp_data[] = [
    //             'gr_id' => $item->id,
    //             'gr_number' => $item->gr_number,
    //             'gr_date' => $item->gr_date,
    //             'source_name' => $item->source_name,
    //             'destination_name' => $item->destination_name,
    //             'driver_name' => $item->driver_name,
    //             'driver_mob_number' => $item->driver_mob_number,
    //             'vehicle' => $item->vehicle?$item->vehicle->vehicle_number:'',
    //             'lorry_hire_from' =>  $copy_of_challan ? $copy_of_challan->challan->lorry_hire_from : '',
    //             'truck_status' => $item->truck_status??'IN TRANSIT',
    //             'location' => $item->location,
    //             'status_date' => $item->status_date?? Carbon::now()->format('Y-m-d'),
    //             'arrival_date' => $item->arrival_date,
    //             'unloading_date' => $item->unloading_date,
    //             'halting_days' => $item->halting_days,
    //             'pod_status' => $item->pod_desc ? 'YES' : 'NO'
    //         ];
    //     }

    //     return response()->json(['truck_status'=>$resp_data]);
    // }

    public function index(Request $request){
        
        // Fetch all the price lists with the relationships on acc_party, source, destination, consignor, consignee
        $truck_status = TruckStatus::with([
            'acc_party' => function ($query) {
                $query->select('id', 'account_name');
            },
            'source' => function ($query) {
                $query->select('id', 'station_name');
            },
            'destination' => function ($query) {
                $query->select('id', 'station_name');
            },
            'vehicle' => function ($query) {
                $query->select('id', 'vehicle_number');
            },
        ])
        ->when($request->acc_party, function ($query) use ($request) {
            $query->where('acc_party_id', $request->acc_party['id']);
        })
        ->when($request->destination, function ($query) use ($request) {
            $query->where('destination_id', $request->destination['id']);
        })
        ->when($request->source, function ($query) use ($request) {
            $query->where('source_id', $request->source['id']);
        })
        ->when($request->vehicle, function ($query) use ($request) {
            $query->where('vehicle_id', $request->vehicle['id']);
        })
        ->where(function ($query) use ($request) {
            // Check if the dates fall within the range
            if ($request->eff_from && $request->eff_till) {
                $query->whereDate('eff_from', '<=', $request->eff_till)
                    ->whereDate('eff_till', '>=', $request->eff_from);
            } elseif ($request->eff_from) {
                $query->whereDate('eff_till', '>=', $request->eff_from);
            } elseif ($request->eff_till) {
                $query->whereDate('eff_from', '<=', $request->eff_till);
            }
        })
        ->get();
        

        return response()->json(['truck_status' => $truck_status]);

    }

    public function show(Request $request, $pk){
            $truck_status = TruckStatus::with([
                'acc_party' => function ($query) {
                    $query->select('id', 'account_name');
                },
                'source' => function ($query) {
                    $query->select('id', 'station_name');
                },
                'destination' => function ($query) {
                    $query->select('id', 'station_name');
                },
                'vehicle' => function ($query) {
                    $query->select('id', 'vehicle_number');
                },
            ])->find($pk);

            
            return response()->json(['truck_status' => $truck_status]);
    }

    public function store(Request $request){
        try {
          
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            
            foreach ($request->truck_status as $index => $pl) {
                if( !$pl['vehicle'] )
                continue;

                
                $pri_lis = new TruckStatus();

                $pri_lis->acc_party_id = $pl['acc_party'] ? $pl['acc_party']['id'] : null;
                $pri_lis->vehicle_id = $pl['vehicle']['id'];
                $pri_lis->vehicle_no = $pl['vehicle']['vehicle_number'];
                $pri_lis->source_id = $pl['source'] ? $pl['source']['id'] : null;
                $pri_lis->source = $pl['source'] ? $pl['source']['station_name'] : null;
                $pri_lis->destination_id = $pl['destination'] ? $pl['destination']['id'] : null;
                $pri_lis->destination = $pl['destination'] ? $pl['destination']['station_name'] : null;
                $pri_lis->date = $pl['date'] ? $pl['date'] : null;
                $pri_lis->location = $pl['location'] ? $pl['location'] : null;
                $pri_lis->truck_status = $pl['truck_status'] ? $pl['truck_status'] : null;
                $pri_lis->created_by = $request->user->id;
                $pri_lis->save();
                

                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('created', 'truck_status', $pri_lis->id,  $pri_lis->vehicle_no,  null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
         
            }
            
            DB::commit();
            
            return response()->json(['success' => 'Customer Code Created Successfully']);

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['truck_status' => [$e->getMessage()]]], 422);
        }    



    }

    public function update(Request $request){

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            // Loop through $request->price_lists and create new PriceList for each of them.
            foreach ($request->newly_added_truck_status as $index => $napl) {
            
                $pri_lis = new TruckStatus();

                $pri_lis->acc_party_id = $napl['acc_party'] ? $napl['acc_party']['id'] : null;
                $pri_lis->vehicle_id = $napl['vehicle']['id'];
                $pri_lis->vehicle_no = $napl['vehicle']['vehicle_number'];
                $pri_lis->source_id = $napl['source'] ? $napl['source']['id'] : null;
                $pri_lis->source = $napl['source'] ? $napl['source']['station_name'] : null;
                $pri_lis->destination_id = $napl['destination'] ? $napl['destination']['id'] : null;
                $pri_lis->destination = $napl['destination'] ? $napl['destination']['station_name'] : null;
                $pri_lis->date = $napl['date'] ? $napl['date'] : null;
                $pri_lis->location = $napl['location'] ? $napl['location'] : null;
                $pri_lis->truck_status = $napl['truck_status'] ? $napl['truck_status'] : null;
                $pri_lis->created_by = $request->user->id;
                $pri_lis->save();



                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('created', 'truck_status', $pri_lis->id, $pri_lis->vehicle_no , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }
             
            // Loop through $request->updated_price_list and update PriceList for each of them.
            foreach ($request->updated_truck_status as $index => $upl) {
                

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($upl['id']);

                $pri_lis = TruckStatus::find($upl['id']);
                $pri_lis->acc_party_id = $upl['acc_party'] ? $upl['acc_party']['id'] : null;
                $pri_lis->vehicle_id = $upl['vehicle']['id'];
                $pri_lis->vehicle_no = $upl['vehicle']['vehicle_number'];
                $pri_lis->source_id = $upl['source'] ? $upl['source']['id'] : null;
                $pri_lis->source = $upl['source'] ? $upl['source']['station_name'] : null;
                $pri_lis->destination_id = $upl['destination'] ? $upl['destination']['id'] : null;
                $pri_lis->destination = $upl['destination'] ? $upl['destination']['station_name'] : null;
                $pri_lis->date = $upl['date'] ? $upl['date'] : null;
                $pri_lis->location = $upl['location'] ? $upl['location'] : null;
                $pri_lis->truck_status = $upl['truck_status'] ? $upl['truck_status'] : null;
                $pri_lis->updated_by = $request->user->id;
                $pri_lis->save();

                

                // Save Edit Log
                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('updated', 'truck_status', $pri_lis->id, $pri_lis->vehicle_no , $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            // Loop through $request->deleted_price_list and delete PriceList for each of them.
            foreach ($request->deleted_truck_status as $dpl) {
                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);


                $pri_lis = TruckStatus::find($dpl);
                $pri_lis->delete();


                // Save Edit Log
                $this->createEditLog('deleted', 'truck_status', $pri_lis->id, $pri_lis->vehicle_no, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();

            return response()->json(['success' => 'Truck Status  Updated Successfully']);
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['truck_status' => [$e->getMessage()]]], 422);
        }
    
    }


    public function destroy(Request $request){
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
        
            // Loop through $request->deleted_price_list and delete PriceList for each of them.
            foreach ($request->deleted_truck_status as $dpl) {

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);

                $pri_lis = TruckStatus::find($dpl);
                $pri_lis->delete();

                // Save Edit Log
                $this->createEditLog('deleted', 'truck_status', $pri_lis->id, $pri_lis->vehicle_no, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();
            return response()->json(['success' => 'Truck Status Deleted Successfully']);
            
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['truck_status' => [$e->getMessage()]]], 422);
        }
    }

    


    protected function get_data_for_obj($pk){
        $truck_status = TruckStatus::with([
            'acc_party' => function ($query) {
                $query->select('id', 'account_name');
            },
            'source' => function ($query) {
                $query->select('id', 'station_name');
            },
            'destination' => function ($query) {
                $query->select('id', 'station_name');
            },
            'vehicle' => function ($query) {
                $query->select('id', 'vehicle_number');
            },
            
        ])->find($pk);


            
        return ['truck_status' => $truck_status];
    }
}
