<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\GoodsMove\Challan;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class TruckStatusController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function show_truck_status(Request $request)
    {
        
        $query = BookGr::with(['branch:id,branch_name,branch_code','lorry_hire_from','eway_details:id,gr_id,eway_number,eway_exp_date','acc_party','acc_party.party:id,account_id,party_name','lorry_hire_from.party:id,account_id,party_name,mob_number','vehicle:id,vehicle_number','challan_desc','pod_desc','challan_desc.challan:id,challan_number,lorry_hire_from,vehicle_id','challan_desc.challan.lorry_hire_from:id,account_name','challan_desc.challan.lorry_hire_from.party:id,party_name,account_id,mob_number'])->where('fin_year',$request->current_year);
    
        if ($request->truck_status) {
            $query->where(function($subQuery) use ($request) {
                if ($request->truck_status == 'IN TRANSIT') {
                    $subQuery->where('truck_status', $request->truck_status)
                             ->orWhereNull('truck_status');
                } elseif ($request->truck_status == 'NOT UNLOADED') {
                    $subQuery->where('truck_status', '!=', 'UNLOADED')
                             ->orWhereNull('truck_status');
                } else {
                    $subQuery->where('truck_status', $request->truck_status);
                }
            });
        }
    
        $result = $query->orderBy('gr_date')->get();
    
        // Calculate min and max dates from gr_date
        $min_date = $result->min('gr_date');
        $max_date = $result->max('gr_date');
        
        // Map booking_number
        $resp_data = [];
    
        foreach ($result as $item) {
            $copy_of_challan = count($item->challan_desc) ? json_decode(json_encode($item->challan_desc[0])) : null;
            $copy_of_gr = json_decode(json_encode($item));
            $copy_of_acc_party = json_decode(json_encode($item->acc_party));
            $copy_of_eway_details = json_decode(json_encode($item->eway_details));
            $resp_data[] = [
                'gr_id' => $item->id,
                'gr_number' => $item->gr_number,
                'gr_date' => $item->gr_date,
                'source_name' => $item->source_name,
                'destination_name' => $item->destination_name,
                'driver_name' => $item->driver_name,
                'driver_mob_number' => $item->driver_mob_number,
                'vehicle' => $item->vehicle?$item->vehicle->vehicle_number:'',
                'lorry_hire_from' =>  $copy_of_challan ? $copy_of_challan->challan->lorry_hire_from : '',
                'truck_status' => $item->truck_status??'IN TRANSIT',
                'current_location' => $item->current_location,
                'status_date' => $item->status_date?? Carbon::now()->format('Y-m-d'),
                'arrival_date' => $item->arrival_date,
                'unloading_date' => $item->unloading_date,
                'halting_days' => $item->halting_days,
                'pod_status' => $item->pod_desc ? 'YES' : 'NO',
                'date_locations' => json_decode($item->date_json ?? '{}'), // Decode the stored JSON date locations
                'lorry_hire_from_party' => $copy_of_gr->lorry_hire_from ? $copy_of_gr->lorry_hire_from->party: '',
                'acc_party' => $copy_of_acc_party ? $copy_of_acc_party->party : '',
                'eway_details' => $copy_of_eway_details,
            ];
        }
    
        return response()->json([
            'truck_status' => $resp_data,
            'min_date' => $min_date,
            'max_date' => $max_date
        ]);
    }

    public function update_truck_status(Request $request)
    {
        $test = $this->is_edit_permitted($request->user->id, 'TruckStatus');
        $test_edit = $test->edit;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $updated_truck_statuses = $request->updated_truck_statuses;

            foreach ($updated_truck_statuses as $uts) {
                $book = BookGr::find($uts['gr_id']);

                $book->truck_status = $uts['truck_status'];
                if($uts['truck_status'] == 'UNLOADED') {
                    $book->truck_status_value = 1;
                }
                else
                {
                    $book->truck_status_value = 0;
                }
                $book->current_location = $uts['current_location'];
                $book->status_date = $uts['status_date'];
                $book->arrival_date = $uts['arrival_date'];
                $book->unloading_date = $uts['unloading_date'];
                $book->halting_days = $uts['halting_days'];
                
                // Store date locations as JSON
                if (isset($uts['date_locations'])) {
                    $book->date_json = json_encode($uts['date_locations']);
                }

                $book->save();
            }
            
            DB::commit();

            return response()->json(['success' => 'Truck Status Updated Successfully']);
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['truck_status' => [$e->getMessage()]]], 422);
        }
    }
}