<?php

namespace App\Http\Controllers\GoodsMove;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\GrDesc;
use App\Models\Booking\PackagePosition;
use App\Models\Booking\PkgPosRecovery;
use App\Models\GoodsMove\ChallanDesc;
use App\Models\GoodsMove\ChallanReceipt;
use App\Models\GoodsMove\ChallanReceiptDesc;
use App\Models\Master\IssueBooks;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ChallanReceiptController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'ChallanReceipt');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $challan_rec = ChallanReceipt::with([
            'branch:id,branch_name,branch_code',
            'arrival_from:id,branch_name,branch_code',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'challan:id,challan_number',
            'challan_rec_desc',
            'challan_rec_desc.challan_desc:id,loaded_pkgs',
            'challan_rec_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
            'challan_rec_desc.gr_desc.material:id,material_name',
            'challan_rec_desc.gr_desc.bookGr:id,gr_number,consignor_id,consignee_id,destination_id,consignor_name,consignee_name,destination_name,pay_mode,grand_total',
            'challan_rec_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.destination:id,station_name',
        ])->where('fin_year',$request->current_year)->get();

        $challan_rec->challan_rec_desc->transform(function ($desc) {
            return [
                'challan_rec_id' => $desc->id,
                'challan_rec_number' => $desc->challan_rec_number,
                'challan_rec_desc_id' => $desc->id,
                'gr_id' => $desc->gr_desc->gr_id,
                'gr_number' => $desc->gr_desc->gr_number,
                'gr_desc_id' => $desc->gr_desc_id,

                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_desc_id' => $desc->challan_desc_id,
                'pkg' => $desc->gr_desc->pkg,
                'wt' => $desc->gr_desc->material_weight,
                'free_pkgs' => $desc->gr_desc->free_pkgs,
                'loaded_pkgs' => $desc->challan_desc->loaded_pkgs,
                'unloaded_pkgs' => $desc->unloaded_pkgs,
                'material' => $desc->gr_desc->material['material_name'],
                'consignor' => $desc->gr_desc->bookGr->consignor_id  ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                'consignee' => $desc->gr_desc->bookGr->consignee_id  ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                'grand_total' => $desc->gr_desc->bookGr->grand_total,
                'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
            ];
        });

        return response()->json($challan_rec);
    }

    public function show(Request $request, $pk){
        $challan_rec = ChallanReceipt::with([
            'branch:id,branch_name,branch_code',
            'arrival_from:id,branch_name,branch_code',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'challan:id,challan_number',
            'challan_rec_desc',
            'challan_rec_desc.challan_desc:id,loaded_pkgs',
            'challan_rec_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
            'challan_rec_desc.gr_desc.material:id,material_name',
            'challan_rec_desc.gr_desc.bookGr:id,gr_number,consignor_id,consignee_id,destination_id,consignor_name,consignee_name,destination_name,pay_mode,grand_total',
            'challan_rec_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.destination:id,station_name',
        ])->find($pk);

        $challan_rec->challan_rec_desc->transform(function ($desc) {
            return [

                'challan_rec_id' => $desc->id,
                'challan_rec_number' => $desc->challan_rec_number,
                'challan_rec_desc_id' => $desc->id,
                'gr_id' => $desc->gr_desc->gr_id,
                'gr_number' => $desc->gr_desc->gr_number,
                'gr_desc_id' => $desc->gr_desc_id,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_desc_id' => $desc->challan_desc_id,
                'pkg' => $desc->gr_desc->pkg,
                'wt' => $desc->gr_desc->material_weight,
                'free_pkgs' => $desc->gr_desc->free_pkgs,
                'loaded_pkgs' => $desc->challan_desc->loaded_pkgs,
                'unloaded_pkgs' => $desc->unloaded_pkgs,
                'material' => $desc->gr_desc->material['material_name'],
                'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                'grand_total' => $desc->gr_desc->bookGr->grand_total,
                'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
            ];
        });

        return response()->json(['challan_rec'=>$challan_rec]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'ChallanReceipt');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'challan_rec_type' => 'required',
            'challan_rec_number' => 'required|unique:challan_receipts,challan_rec_number',
            'challan' => 'required',
            'arrival_from' => 'required',
        ], [
            'branch.required' => 'Required',
            'challan_rec_type.required' => 'Required',
            'challan.required' => 'Required',
            'challan_rec_type.required' => 'Required',
            'arrival_from.required' => 'Required',
            'challan_rec_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE CHALLAN TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_challan_rec_number($request->challan_rec_number);
            if($request->challan_rec_type == 'PRINTED'){
                $check = IssueBooks::select('prefix','party_id')->where('prefix',$request->prefix)->where('branch_id', $request->branch['id'])->where('book_type','ChallanReceipt')->where('issue_from', '<=', $max_number)->where('issue_to', '>=', $max_number)->first();
                if(!$check){
                    return response()->json(['message'=>'Book Not Issued To The Branch'], 422);
                }
            }

            
            
            
            // SAVING A NEW GR
            $challan_rec = new ChallanReceipt();
            $challan_rec->branch_id = $request->branch['id'];
            $challan_rec->challan_rec_type = $request->challan_rec_type;
            $challan_rec->prefix = $request->prefix;
            $challan_rec->max_number = $request->max_number;
            $challan_rec->challan_rec_number = $request->challan_rec_number;

            $challan_rec->arrival_date = $request->arrival_date;
            $challan_rec->arrival_time = $request->arrival_time;

            $challan_rec->arrival_from = $request->arrival_from['id'];

            $challan_rec->filter_challan_type = $request->filter_challan_type;
            $challan_rec->challan_id = $request->challan['id'];
            $challan_rec->challan_number = $request->challan['challan_number'];
            $challan_rec->challan_date = $request->challan_date;
            $challan_rec->challan_time = $request->challan_time;

            $challan_rec->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $challan_rec->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $challan_rec->vehicle_type = $request->vehicle_type;
            $challan_rec->driver_id = $request->driver ? $request->driver['id'] : null;

            $challan_rec->seal_number = $request->seal_number;
            $challan_rec->status = $request->status;
            $challan_rec->remarks = $request->remarks;

            $challan_rec->total_to_pay = $request->total_to_pay;
            $challan_rec->del_comm = $request->del_comm;
            $challan_rec->kaat = $request->kaat;
            $challan_rec->door_del = $request->door_del;
            $challan_rec->driver_amt_type = $request->driver_amt_type;
            $challan_rec->driver_amt = $request->driver_amt;
            $challan_rec->branch_balance = $request->branch_balance;
           
            $challan_rec->fin_year = $request->current_year;

            $challan_rec->created_by = $request->user->id;

            // Save the model to the database
            $challan_rec->save();

            // SAVING PACKAGE DETAILS
            $challan_rec_desc = $request->challan_rec_desc;
            foreach ($challan_rec_desc as $cd) {
                if(!$cd['unloaded_pkgs'] > 0)
                continue;
                
                $ch_dc = new ChallanReceiptDesc();
                $ch_dc->challan_receipt_id = $challan_rec->id;
                $ch_dc->challan_rec_number = $challan_rec->challan_rec_number;

                $ch_dc->challan_id = $cd['challan_id'];
                $ch_dc->challan_number = $cd['challan_number'];
                $ch_dc->challan_desc_id = $cd['challan_desc_id'];

                $ch_dc->gr_id = $cd['gr_id'];
                $ch_dc->gr_number = $cd['gr_number'];
                $ch_dc->gr_desc_id = $cd['gr_desc_id'];
                $ch_dc->unloaded_pkgs = $cd['unloaded_pkgs'];
                $ch_dc->fin_year = $request->current_year;
                $ch_dc->save();

                // SAVING PACKAGE POSITIONS
                $cnt = PackagePosition::where('gr_desc_id', $cd['gr_desc_id'])->where('position',$request->branch['id'])->first();
                if($cnt){
                    $to_backup = PackagePosition::where('id',$cd['pkg_pos_id'])->where('gr_desc_id', $cd['gr_desc_id'])->where('position','IN TRANSIT')->first();
                    // Backup in PkgPosRecovery
                    $pkg_pos_recovery = new PkgPosRecovery();
                    $pkg_pos_recovery->gr_id = $to_backup->gr_id;
                    $pkg_pos_recovery->gr_number = $to_backup->gr_number;
                    $pkg_pos_recovery->gr_desc_id = $to_backup->gr_desc_id;
                    $pkg_pos_recovery->position = $to_backup->position;
                    $pkg_pos_recovery->pkg = $to_backup->pkg;
                    $pkg_pos_recovery->challan_desc_id = $to_backup->challan_desc_id;
                    $pkg_pos_recovery->origin = $to_backup->origin;
                    $pkg_pos_recovery->is_delivered = $to_backup->is_delivered;
                    $pkg_pos_recovery->save();
                    // Backup in PkgPosRecovery
                    $to_backup->delete();


                    $cnt->pkg = $cnt->pkg + $cd['unloaded_pkgs'];
                    $cnt->save();
                }
                else{

                    $existing_pkg_position = PackagePosition::where('id',$cd['pkg_pos_id'])->where('gr_desc_id', $cd['gr_desc_id'])->where('position','IN TRANSIT')->first();
                
                    if($existing_pkg_position->pkg == $cd['unloaded_pkgs']){
                        $existing_pkg_position->position = $request->branch['id'];
                        $existing_pkg_position->challan_desc_id = null;
                        $existing_pkg_position->save();
                    }

                }

                
                

                // else{
                //     $existing_pkg_position->pkg = $existing_pkg_position->pkg - $cd['unloaded_pkgs'];
                //     $existing_pkg_position->save();

                //     $pkg_position = new PackagePosition();
                //     $pkg_position->gr_desc_id = $existing_pkg_position['gr_desc_id'];
                //     $pkg_position->gr_id = $existing_pkg_position['gr_id'];
                //     $pkg_position->gr_number = $existing_pkg_position['gr_number'];
                //     $pkg_position->position = $request->branch['id'];
                //     $pkg_position->pkg = $cd['unloaded_pkgs'];
                //     $pkg_position->origin = $existing_pkg_position['origin'];
                //     $pkg_position->challan_desc_id = null;
                //     $pkg_position->save();
                // }

                // PACKAGE POSITIONS SAVED

                // GR DESC FREE PACKETS
                // SUM OF ALL THE PKG WHERE
                $sum_of_free_pkg = PackagePosition::where('gr_desc_id', $cd['gr_desc_id'])->where('position','!=','IN TRANSIT')->sum('pkg');

                $gr_desc = GrDesc::find($cd['gr_desc_id']);
                $gr_desc->free_pkgs = $sum_of_free_pkg;
                $gr_desc->save();

            }
            // PACKAGE DETAILS SAVED



             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($challan_rec->id);
            $this->createEditLog('created', 'challan_receipts', $challan_rec->id, $challan_rec->challan_rec_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$challan_rec->id, 'challan_rec_number'=>$challan_rec->challan_rec_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'ChallanReceipt');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $challan_rec = ChallanReceipt::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($challan_rec->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'challan_rec_type' => 'required',
            'challan_rec_number' => 'required',
            'challan' => 'required',
            'arrival_from' => 'required',
        ], [
            'branch.required' => 'Required',
            'challan_rec_type.required' => 'Required',
            'challan.required' => 'Required',
            'challan_rec_type.required' => 'Required',
            'arrival_from.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW CHALLAN RECEIPT
            $challan_rec->arrival_date = $request->arrival_date;
            $challan_rec->arrival_time = $request->arrival_time;

            $challan_rec->arrival_from = $request->arrival_from['id'];

            $challan_rec->filter_challan_type = $request->filter_challan_type;
            $challan_rec->challan_id = $request->challan['id'];
            $challan_rec->challan_number = $request->challan['challan_number'];
            $challan_rec->challan_date = $request->challan_date;
            $challan_rec->challan_time = $request->challan_time;

            $challan_rec->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $challan_rec->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $challan_rec->vehicle_type = $request->vehicle_type;
            $challan_rec->driver_id = $request->driver ? $request->driver['id'] : null;

            $challan_rec->seal_number = $request->seal_number;
            $challan_rec->status = $request->status;
            $challan_rec->remarks = $request->remarks;

            $challan_rec->total_to_pay = $request->total_to_pay;
            $challan_rec->del_comm = $request->del_comm;
            $challan_rec->kaat = $request->kaat;
            $challan_rec->door_del = $request->door_del;
            $challan_rec->driver_amt_type = $request->driver_amt_type;
            $challan_rec->driver_amt = $request->driver_amt;
            $challan_rec->branch_balance = $request->branch_balance;

            $challan_rec->updated_by = $request->user->id;
            $challan_rec->save();
            // NEW CHALLAN RECEIPT SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_challan_rec_desc = $request->newly_added_challan_rec_desc;
            foreach ($newly_added_challan_rec_desc as $nacd) {
                if(!$nacd['unloaded_pkgs'] > 0)
                continue;

                $ch_dc = new ChallanReceiptDesc();
                $ch_dc->challan_receipt_id = $challan_rec->id;
                $ch_dc->challan_rec_number = $challan_rec->challan_rec_number;

                $ch_dc->challan_id = $nacd['challan_id'];
                $ch_dc->challan_number = $nacd['challan_number'];
                $ch_dc->challan_desc_id = $nacd['challan_desc_id'];

                $ch_dc->gr_id = $nacd['gr_id'];
                $ch_dc->gr_number = $nacd['gr_number'];
                $ch_dc->gr_desc_id = $nacd['gr_desc_id'];
                $ch_dc->unloaded_pkgs = $nacd['unloaded_pkgs'];
                $ch_dc->fin_year = $request->current_year;
                $ch_dc->save();

                // SAVING PACKAGE POSITIONS
                // SAVING PACKAGE POSITIONS
                $cnt = PackagePosition::where('gr_desc_id', $nacd['gr_desc_id'])->where('position',$request->branch['id'])->first();
                if($cnt){
                    $to_backup = PackagePosition::where('id',$nacd['pkg_pos_id'])->where('gr_desc_id', $nacd['gr_desc_id'])->where('position','IN TRANSIT')->first();
                    // Backup in PkgPosRecovery
                    $pkg_pos_recovery = new PkgPosRecovery();
                    $pkg_pos_recovery->gr_id = $to_backup->gr_id;
                    $pkg_pos_recovery->gr_number = $to_backup->gr_number;
                    $pkg_pos_recovery->gr_desc_id = $to_backup->gr_desc_id;
                    $pkg_pos_recovery->position = $to_backup->position;
                    $pkg_pos_recovery->pkg = $to_backup->pkg;
                    $pkg_pos_recovery->challan_desc_id = $to_backup->challan_desc_id;
                    $pkg_pos_recovery->origin = $to_backup->origin;
                    $pkg_pos_recovery->is_delivered = $to_backup->is_delivered;
                    $pkg_pos_recovery->save();
                    // Backup in PkgPosRecovery
                    $to_backup->delete();


                    $cnt->pkg = $cnt->pkg + $nacd['unloaded_pkgs'];
                    $cnt->save();
                }
                else{

                    $existing_pkg_position = PackagePosition::where('id',$nacd['pkg_pos_id'])->where('gr_desc_id', $nacd['gr_desc_id'])->where('position','IN TRANSIT')->first();
                
                    if($existing_pkg_position->pkg == $nacd['unloaded_pkgs']){
                        $existing_pkg_position->position = $request->branch['id'];
                        $existing_pkg_position->challan_desc_id = null;
                        $existing_pkg_position->save();
                    }

                }

                // else{
                //     $existing_pkg_position->pkg = $existing_pkg_position->pkg - $nacd['unloaded_pkgs'];
                //     $existing_pkg_position->save();

                //     $pkg_position = new PackagePosition();
                //     $pkg_position->gr_desc_id = $existing_pkg_position['gr_desc_id'];
                //     $pkg_position->gr_id = $existing_pkg_position['gr_id'];
                //     $pkg_position->gr_number = $existing_pkg_position['gr_number'];
                //     $pkg_position->position = $request->branch['id'];
                //     $pkg_position->pkg = $nacd['unloaded_pkgs'];
                //     $pkg_position->origin = $existing_pkg_position['origin'];
                //     $pkg_position->challan_desc_id = null;
                //     $pkg_position->save();
                // }
                // PACKAGE POSITIONS SAVED

                $sum_of_free_pkg = PackagePosition::where('gr_desc_id', $existing_pkg_position['gr_desc_id'])->where('position','!=','IN TRANSIT')->sum('pkg');

                $gr_desc = GrDesc::find($existing_pkg_position['gr_desc_id']);
                $gr_desc->free_pkgs = $sum_of_free_pkg;
                $gr_desc->save();

            }
            // PACKAGE DETAILS SAVED

            // SAVING UPDATES ON EXISTING  PACKAGE DETAILS
            // $updated_challan_rec_desc = $request->updated_challan_rec_desc;
            // if( !empty($updated_challan_rec_desc) ){
            //     foreach ($updated_challan_rec_desc as $ucd) {
            //         if(!$ucd['unloaded_pkgs'] > 0)
            //         continue;

            //         $ch_dc = ChallanReceiptDesc::find($ucd['challan_rec_desc_id']);
            //         $ch_dc->unloaded_pkgs = $ucd['unloaded_pkgs'];
            //         $ch_dc->save();

            //         // SAVING PACKAGE POSITIONS
            //         $pkg_position = PackagePosition::where('gr_desc_id', $ch_dc->gr_desc_id)->where('position',$challan_rec->branch_id)->first();
            //         $diff = $pkg_position->pkg - $ucd['unloaded_pkgs'];
            //         $pkg_position->pkg = $ucd['unloaded_pkgs'];
            //         $pkg_position->save();

            //         $pkg_position = PackagePosition::where('gr_desc_id', $ch_dc->gr_desc_id)->where('position','IN TRANSIT')->first();
            //         if($pkg_position){
            //             $pkg_position->pkg = $pkg_position->pkg + $diff;

            //             if($pkg_position->pkg == 0){
            //                 $pkg_position->delete();
            //             }
            //             else{
            //                 $pkg_position->save();
            //             }
            //         }
            //         // PACKAGE POSITIONS SAVED

            //         $sum_of_free_pkg = PackagePosition::where('gr_desc_id', $pkg_position['gr_desc_id'])->where('position','!=','IN TRANSIT')->sum('pkg');

            //         $gr_desc = GrDesc::find($pkg_position['gr_desc_id']);
            //         $gr_desc->free_pkgs = $sum_of_free_pkg;
            //         $gr_desc->save();
            //     }
            // }
            // UPDATES SAVED ON EXISTING PACKAGE DETAILS

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_challan_rec_desc = $request->deleted_challan_rec_desc;
            if( !empty($deleted_challan_rec_desc) ){
                foreach ($deleted_challan_rec_desc as $dcd) {
                    $crd = ChallanReceiptDesc::find( $dcd );
                    $crd->delete();

                    // DO NOT ALLOW DELETE IF THE GR HAS ALREADY BEEN SENT AHEAD BY THE RECEIVER BRANCH
                    $check_ahead = ChallanDesc::where('gr_id',$crd->gr_id)->where('created_at','>',$crd->created_at)->first();
                    if($check_ahead){
                      $status = ['status'=>true, 'message'=>'Stopped By Challan Number '.$check_ahead['challan_number'], 'exists_in'=>'challan_desc', 'index'=>$check_ahead['challan_number'] ];  
                      return response()->json(['error'=>'Document in use', 'data'=>$status],500);
                    }
                    

                    $pk_query = PackagePosition::where('gr_desc_id', $crd->gr_desc_id)->where('position',$challan_rec->branch_id)->where('unloaded_pkgs',$crd->unloaded_pkgs);

                    if($pk_query->exists()){
                      $pkg_position = $pk_query->first();
                    }
                    else{
                        $pkg_position = PackagePosition::where('gr_desc_id', $crd->gr_desc_id)->where('position',$challan_rec->branch_id)->first();
                    }
                    
                    if($pkg_position){

                        if($pkg_position->pkg == $crd->unloaded_pkgs){
                            $pkg_position->position = 'IN TRANSIT';
                            $pkg_position->challan_desc_id = $crd->challan_desc_id;
                            $pkg_position->save();
                        }
                        else{
                            $pkg_position->pkg = $pkg_position->pkg - $crd->unloaded_pkgs;
                            $pkg_position->save();

                            $pkg_posit = new PackagePosition();
                            $pkg_posit->gr_desc_id = $crd->gr_desc_id;
                            $pkg_posit->gr_id = $crd->gr_id;
                            $pkg_posit->gr_number = $crd->gr_number;
                            $pkg_posit->position = 'IN TRANSIT';
                            $pkg_posit->pkg = $crd->unloaded_pkgs;
                            $pkg_posit->origin = $pkg_position->origin;
                            $pkg_posit->challan_desc_id = $crd->challan_desc_id;
                            $pkg_posit->save();
                        }
                        
                    }
                    
                    $sum_of_free_pkg = PackagePosition::where('gr_desc_id', $crd->gr_desc_id)->where('position','!=','IN TRANSIT')->sum('pkg');

                    $gr_desc = GrDesc::find($crd->gr_desc_id);
                    $gr_desc->free_pkgs = $sum_of_free_pkg;
                    $gr_desc->save();
    
                }      
            }
            //DELETED EXISTING PACKAGE DETAILS
            


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($challan_rec->id);
            $this->createEditLog('updated', 'challan_receipts', $challan_rec->id, $challan_rec->challan_rec_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'ChallanReceipt');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $challan_rec = ChallanReceipt::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($challan_rec->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
        

        


        $challan_rec_desc_ids = ChallanReceiptDesc::select('id')->where('challan_receipt_id', $pk)->pluck('id')->toArray();

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
        
        // DELETING CHALLAN DESCRIPTIONS
        foreach ($challan_rec_desc_ids as $cdi) {
                    $crd = ChallanReceiptDesc::find( $cdi );
                    $crd->delete();

                    // DO NOT ALLOW DELETE IF THE GR HAS ALREADY BEEN SENT AHEAD BY THE RECEIVER BRANCH
                    $check_ahead = ChallanDesc::where('gr_id',$crd->gr_id)->where('created_at','>',$crd->created_at)->first();
                    if($check_ahead){
                      $status = ['status'=>true, 'message'=>'Stopped By Challan Number '.$check_ahead['challan_number'], 'exists_in'=>'challan_desc', 'index'=>$check_ahead['challan_number'] ];  
                      return response()->json(['error'=>'Document in use', 'data'=>$status],500);
                    }


                    $pk_query = PackagePosition::where('gr_desc_id', $crd->gr_desc_id)->where('position',$challan_rec->branch_id)->where('pkg',$crd->unloaded_pkgs);

                    if($pk_query->exists()){
                      $pkg_position = $pk_query->first();
                    }
                    else{
                        $pkg_position = PackagePosition::where('gr_desc_id', $crd->gr_desc_id)->where('position',$challan_rec->branch_id)->first();
                    }
                    
                    if($pkg_position){

                        if($pkg_position->pkg == $crd->unloaded_pkgs){
                            $pkg_position->position = 'IN TRANSIT';
                            $pkg_position->challan_desc_id = $crd->challan_desc_id;
                            $pkg_position->save();
                        }
                        else{
                            $pkg_position->pkg = $pkg_position->pkg - $crd->unloaded_pkgs;
                            $pkg_position->save();

                            $pkg_posit = new PackagePosition();
                            $pkg_posit->gr_desc_id = $crd->gr_desc_id;
                            $pkg_posit->gr_id = $crd->gr_id;
                            $pkg_posit->gr_number = $crd->gr_number;
                            $pkg_posit->position = 'IN TRANSIT';
                            $pkg_posit->pkg = $crd->unloaded_pkgs;
                            $pkg_posit->origin = $pkg_position->origin;
                            $pkg_posit->challan_desc_id = $crd->challan_desc_id;
                            $pkg_posit->save();
                        }
                        
                    }
                    
                    $sum_of_free_pkg = PackagePosition::where('gr_desc_id', $crd->gr_desc_id)->where('position','!=','IN TRANSIT')->sum('pkg');

                    $gr_desc = GrDesc::find($crd->gr_desc_id);
                    $gr_desc->free_pkgs = $sum_of_free_pkg;
                    $gr_desc->save();
        }

        //FIND AND DELETE THE BRANCH
        $challan_rec->delete();

        $this->createEditLog('deleted', 'challan_receipts', $challan_rec->id, $challan_rec->challan_rec_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = ChallanReceipt::select('id', 'challan_rec_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('challan_rec_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $challan_rec = $query->get();
        return response(['challan_receipts' => $challan_rec, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'ChallanReceipt');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = ChallanReceipt::select('id', 'challan_rec_number', 'arrival_from', 'arrival_date', 'arrival_time' ,'challan_number', 'challan_date','challan_time' , 'branch_id')
                ->with(['branch:id,branch_name,branch_code', 'arrival_from:id,branch_name,branch_code',])
                ->where('fin_year',$request->current_year) 
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('challan_rec_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = ChallanReceipt::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $challanRecQuery = ChallanReceipt::select('id', 'challan_rec_number')->where('fin_year',$request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $challanRecQuery->whereIn('branch_id', $request->user_branch_ids);
        }
    
        $challanRec = $challanRecQuery->orderBy('id', 'desc')->take(10)->get();
    
        return response()->json(['challan_receipts' => $challanRec]);
    }
    


    public function get_next_challan_rec_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        if($type=='SYSTEM'){
            $max = ChallanReceipt::where('branch_id',$branch_id)->where('challan_rec_type', $type)->max('max_number');
            $challan_rec_number = $branch_code.'-'.sprintf('%06d', $max+1);
            return response()->json(['challan_rec_number' => strtoupper($challan_rec_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = ChallanReceipt::where('prefix',$prefix)->where('challan_rec_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Challan Receipt')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $challan_rec_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['challan_rec_number' => strtoupper($challan_rec_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Challan Receipt')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $challan_rec_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['challan_rec_number' => strtoupper($challan_rec_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['challan_rec_number' => ['No Valid Ch Recipt Number']]],422);
            }
        }
    }

    public function get_challan_receipt_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = ChallanReceipt::with(['branch:id,branch_name,branch_code','challan_rec_desc','challan_rec_desc.challan:id,challan_number,movement_type','challan_rec_desc.gr_desc:id,material_id,gr_id,gr_number', 'challan_rec_desc.gr_desc.material:id,material_name'])
        ->where('fin_year',$request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['challan_rec_num_from']) && isset($report_data['challan_rec_num_to'])){
            $query->where('challan_rec_number','>=',$report_data['challan_rec_num_from']['challan_rec_number'])->where('challan_rec_number','<=',$report_data['challan_rec_num_to']['challan_rec_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('arrival_date','>=',$report_data['from_date'])->where('arrival_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['challan_number']) ){
            $query->whereHas('challan_rec_desc', function($q) use ($report_data){
                $q->where('challan_number', $report_data['challan_number']['challan_number']);
            });
        }

        if(isset($report_data['movement_type']) ){
            $query->whereHas('challan', function($q) use ($report_data){
                $q->where('movement_type', $report_data['movement_type']);
            });
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('challan_rec_number', 'like', '%'.$searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $challan_rec = ChallanReceipt::with([
            'branch:id,branch_name,branch_code',
            'arrival_from:id,branch_name,branch_code',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'challan:id,challan_number',
            'challan_rec_desc',
            'challan_rec_desc.challan_desc:id,loaded_pkgs',
            'challan_rec_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
            'challan_rec_desc.gr_desc.material:id,material_name',
            'challan_rec_desc.gr_desc.bookGr:id,gr_number,consignor_id,consignee_id,destination_id,consignor_name,consignee_name,destination_name,pay_mode,grand_total',
            'challan_rec_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.destination:id,station_name',
        ])->find($pk);

        $challan_rec->challan_rec_desc->transform(function ($desc) {
            return [
                'challan_rec_id' => $desc->id,
                'challan_rec_number' => $desc->challan_rec_number,
                'challan_rec_desc_id' => $desc->id,
                'gr_id' => $desc->gr_desc->gr_id,
                'gr_number' => $desc->gr_desc->gr_number,
                'gr_desc_id' => $desc->gr_desc_id,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_desc_id' => $desc->challan_desc_id,
                'pkg' => $desc->gr_desc->pkg,
                'wt' => $desc->gr_desc->material_weight,
                'free_pkgs' => $desc->gr_desc->free_pkgs,
                'loaded_pkgs' => $desc->challan_desc->loaded_pkgs,
                'unloaded_pkgs' => $desc->unloaded_pkgs,
                'material' => $desc->gr_desc->material['material_name'],
                'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                'consignee' => $desc->gr_desc->bookGr->consignee ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                'grand_total' => $desc->gr_desc->bookGr->grand_total,
                'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
            ];
        });

        return ['challan_rec'=>$challan_rec];
    }

    protected function get_max_number_from_challan_rec_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }
}
