<?php

namespace App\Http\Controllers\GoodsMove;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\Booking\GrDesc;
use App\Models\Booking\PackagePosition;
use App\Models\GoodsMove\Unloading;
use App\Models\GoodsMove\UnloadingDesc;
use App\Models\Master\IssueBooks;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class UnloadingController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Unloading');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $unloadings = Unloading::with([
            'branch:id,branch_name,branch_code',
            'arrival_from:id,branch_name,branch_code',
            'vehicle:id,vehicle_number',
            'unloading_desc',
            'unloading_desc.gr_desc:id,gr_id,gr_number,pkg,material_id,material_weight,packing_method,chg_unit,rate,total',
            'unloading_desc.gr_desc.material:id,material_name',
            'unloading_desc.bookGr',
            'unloading_desc.bookGr.branch:id,branch_name,branch_code',
            'unloading_desc.bookGr.consignor:id,party_name',
            'unloading_desc.bookGr.consignee:id,party_name',
            'unloading_desc.bookGr.source:id,station_name',
            'unloading_desc.bookGr.destination:id,station_name',
        ])->where('fin_year',$request->current_year)->get();

        $unloadings->unloading_desc->transform(function ($desc) {
            return [
                'unloading_desc_id' => $desc->id,
                'gr_id' => $desc->gr_id,
                'gr_number' => $desc->gr_number,
                'gr_desc_id' => $desc->gr_desc_id,
                'gr_date' => $desc->bookGr->gr_date,
                'branch' => $desc->bookGr->branch,
                'consignor' => $desc->bookGr->consignor ? $desc->bookGr->consignor : ['id'=>null, 'party_name'=>$desc->bookGr->consignor_name],
                'consignee' => $desc->bookGr->consignee ? $desc->bookGr->consignee : ['id'=>null, 'party_name'=>$desc->bookGr->consignee_name],
                'source' => $desc->bookGr->source ? $desc->bookGr->source : ['id'=>null, 'station_name'=>$desc->bookGr->source_name],
                'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=>null, 'station_name'=>$desc->bookGr->destination_name],
                'pay_mode' => $desc->bookGr->pay_mode,
                'delivery_pt' => $desc->bookGr->delivery_pt,
                'p_marka' => $desc->bookGr->p_marka,
                'material' => $desc->gr_desc->material,
                'pkg' => $desc->gr_desc->pkg,
                'material_weight' => $desc->gr_desc->material_weight,
                'packing_method' => $desc->gr_desc->packing_method,
                'chg_unit' => $desc->gr_desc->chg_unit,
                'rate' => $desc->gr_desc->rate,
                'desc_total' => $desc->gr_desc->total,
                'payment_dynamic_fields' => $desc->bookGr->payment_dynamic_fields,
                'grand_total' => $desc->bookGr->grand_total,
                'total_without_rounding' => $desc->bookGr->total_without_rounding,
                'rounded_off' => $desc->bookGr->rounded_off
            ];
        });

        return response()->json($unloadings);
    }

    public function show(Request $request, $pk){
        //FETCH AND SEND ALL
        $unloading = Unloading::with([
            'branch:id,branch_name,branch_code',
            'arrival_from:id,branch_name,branch_code,account_id',
            'vehicle:id,vehicle_number',
            'driver_amt_acc:id,account_name',
            'unloading_desc',
            'unloading_desc.gr_desc:id,gr_id,gr_number,pkg,material_id,material_weight,packing_method,chg_unit,rate,total',
            'unloading_desc.gr_desc.material:id,material_name',
            'unloading_desc.bookGr',
            'unloading_desc.bookGr.branch:id,branch_name,branch_code',
            'unloading_desc.bookGr.consignor:id,party_name',
            'unloading_desc.bookGr.consignee:id,party_name',
            'unloading_desc.bookGr.source:id,station_name',
            'unloading_desc.bookGr.destination:id,station_name',
        ])->find($pk);

        $unloading->unloading_desc->transform(function ($desc) {
            return [
                'unloading_desc_id' => $desc->id,
                'gr_id' => $desc->gr_id,
                'gr_number' => $desc->gr_number,
                'gr_desc_id' => $desc->gr_desc_id,
                'gr_date' => $desc->bookGr->gr_date,
                'branch' => $desc->bookGr->branch,
                'consignor' => $desc->bookGr->consignor ? $desc->bookGr->consignor : ['id'=>null, 'party_name'=>$desc->bookGr->consignor_name],
                'consignee' => $desc->bookGr->consignee ? $desc->bookGr->consignee : ['id'=>null, 'party_name'=>$desc->bookGr->consignee_name],
                'source' => $desc->bookGr->source ? $desc->bookGr->source : ['id'=>null, 'station_name'=>$desc->bookGr->source_name],
                'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=>null, 'station_name'=>$desc->bookGr->destination_name],
                'pay_mode' => $desc->bookGr->pay_mode,
                'delivery_pt' => $desc->bookGr->delivery_pt,
                'p_marka' => $desc->bookGr->p_marka,
                'material' => $desc->gr_desc->material,
                'pkg' => $desc->gr_desc->pkg,
                'material_weight' => $desc->gr_desc->material_weight,
                'packing_method' => $desc->gr_desc->packing_method,
                'chg_unit' => $desc->gr_desc->chg_unit,
                'rate' => $desc->gr_desc->rate,
                'desc_total' => $desc->gr_desc->total,
                'payment_dynamic_fields' => $desc->bookGr->payment_dynamic_fields,
                'grand_total' => $desc->bookGr->grand_total,
                'total_without_rounding' => $desc->bookGr->total_without_rounding,
                'rounded_off' => $desc->bookGr->rounded_off
            ];
        });

        return response()->json(['unloading'=>$unloading]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'Unloading');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'unloading_number' => 'required|unique:unloadings,unloading_number',
            'challan_number' => 'required',
            'arrival_from' => 'required',
        ], [
            'branch.required' => 'Required',
            'unloading_number.required' => 'Required',
            'challan_number.required' => 'Required',
            'arrival_from.required' => 'Required',
            'unloading_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_unloading_number($request->unloading_number);
            
            // SAVING A NEW Unloadin
            $unloading = new Unloading();
            $unloading->branch_id = $request->branch['id'];
            $unloading->unloading_type = $request->unloading_type;
            $unloading->prefix = $request->prefix;
            $unloading->max_number = $max_number;
            $unloading->unloading_number = $request->unloading_number;

            $unloading->arrival_date = $request->arrival_date;
            $unloading->arrival_time = $request->arrival_time;

            $unloading->arrival_from = $request->arrival_from['id'];

            $unloading->challan_number = $request->challan_number;
            $unloading->challan_date = $request->challan_date;
            $unloading->challan_time = $request->challan_time;

            
            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $unloading->vehicle_id = $vehicle_id;
            $unloading->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
                
            $unloading->vehicle_type = $request->vehicle_type;
            $unloading->driver_id = $request->driver ? $request->driver['id'] : null;
            $unloading->driver_name = $request->driver ? $request->driver['emp_name'] : null;

            $unloading->seal_number = $request->seal_number;
            $unloading->status = $request->status;
            $unloading->remarks = $request->remarks;

            $unloading->total_to_pay = $request->total_to_pay;
            $unloading->del_comm = $request->del_comm;
            $unloading->kaat = $request->kaat;
            $unloading->door_del = $request->door_del;
            $unloading->driver_amt_type = $request->driver_amt_type;
            $unloading->driver_amt = $request->driver_amt;
            $unloading->driver_amt_acc_id = $request->driver_amt_acc ? $request->driver_amt_acc['id'] : null;
            $unloading->branch_balance = $request->branch_balance;
           

            $unloading->created_by = $request->user->id;
            $unloading->fin_year = $request->current_year;

            // Save the model to the database
            $unloading->save();

            // SAVING PACKAGE DETAILS
            $unloading_desc = $request->unloading_desc;
            foreach ($unloading_desc as $ud) {
                if( !$ud['gr_number'] || !$ud['branch'] || !$ud['consignor'] || !$ud['consignee'] || !$ud['material'] || !$ud['pkg'] || !$ud['rate'] || !$ud['chg_unit'] )
                continue;

                $gr = new BookGr();
                $gr->branch_id = $ud['branch']['id'];
                $gr->gr_number = $request->arrival_from['branch_code'].'-'.$ud['gr_number'];
                $gr->gr_date = $ud['gr_date'];
                $gr->gr_time = $request->challan_time;
                $source_id = $this->add_if_not_in_master('Station', $request->source, $request->current_year);
                $gr->source_id = $source_id;
                $gr->source_name = $ud['source']['station_name'];

                $destination_id = $this->add_if_not_in_master('Station', $request->destination, $request->current_year);
                $gr->destination_id = $destination_id;
                $gr->destination_name = $ud['destination']['station_name'];

                
                $gr->tpt_mode = 'SURFACE';

                $gr->crossing = 1;

                $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
                $gr->vehicle_id = $vehicle_id;
                $gr->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
                
                $gr->vehicle_type = $request->vehicle_type;
                $gr->driver_id = $request->driver ? $request->driver['id'] : null;
                $gr->driver_name = $request->driver ? $request->driver['emp_name'] : null;

                $gr->pay_mode = $ud['pay_mode'];

                $gr->consignor_id = $ud['consignor'] ? $ud['consignor']['id'] : null;
                $gr->consignor_name = $ud['consignor'] ? $ud['consignor']['party_name'] : null;

                $gr->consignee_id = $ud['consignee'] ? $ud['consignee']['id'] : null;
                $gr->consignee_name = $ud['consignee'] ? $ud['consignee']['party_name'] : null;

                $gr->acc_party_id = $ud['acc_party'] ? $ud['acc_party']['id'] : null;
                
                $gr->delivery_pt = $ud['delivery_pt'];

                $gr->total_pkg = $ud['pkg'];
                $gr->total_material_weight = $ud['material_weight'];
                $gr->total_chg_qty = $ud['pkg'];
                $gr->total_desc_value = $ud['desc_total'];

                $gr->p_marka = $ud['p_marka'];
                 
                $gr->freight = $ud['desc_total'];
                $gr->payment_dynamic_fields = $ud['payment_dynamic_fields'];
                $gr->total_without_rounding = $ud['total_without_rounding'];
                $gr->rounded_off = $ud['rounded_off'];
                $gr->grand_total = $ud['grand_total'];
                $gr->created_by = $request->user->id;
                $gr->fin_year = $request->current_year;
                $gr->save();


                $pkg_det = new GrDesc();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;

                $material_id = $this->add_if_not_in_master('Material', $ud['material'], $request->current_year);

                $pkg_det->material_id = $material_id;
                
                $pkg_det->destination_id = $ud['destination']['id'];
                $pkg_det->packing_method = $ud['packing_method'];
                $pkg_det->pkg = $ud['pkg'];
                $pkg_det->material_weight = $ud['material_weight'];
                $pkg_det->chg_unit = $ud['chg_unit'];
                $pkg_det->chg_qty = $ud['pkg'];
                $pkg_det->rate = $ud['rate'];
                $pkg_det->total = $ud['desc_total'];

                $pkg_det->free_pkgs = $ud['pkg'];
                $pkg_det->fin_year = $request->current_year;
                $pkg_det->save();
                 
                
                // SAVING PACKAGE POSITIONS
                $pkg_position = new PackagePosition();
                $pkg_position->gr_id = $gr->id;
                $pkg_position->gr_number = $gr->gr_number;
                $pkg_position->gr_desc_id = $pkg_det->id;
                $pkg_position->position = $request->branch['id'];
                $pkg_position->pkg = $ud['pkg'];
                $pkg_position->origin = $request->branch['id'];
                $pkg_position->save();
                // PACKAGE POSITIONS SAVED

                $unld_desc = new UnloadingDesc();
                $unld_desc->unloading_id = $unloading->id;
                $unld_desc->gr_id = $gr->id;
                $unld_desc->gr_number = $gr->gr_number;
                $unld_desc->gr_desc_id = $pkg_det->id;
                $unld_desc->fin_year = $request->current_year;
                $unld_desc->save();
            }
            // PACKAGE DETAILS SAVED


            $this->record_transaction_in_accounts($request, $unloading);
             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($unloading->id);
            $this->createEditLog('created', 'unloadings', $unloading->id, $unloading->unloading_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$unloading->id, 'unloading_number'=>$unloading->unloading_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Unloading');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $unloading = Unloading::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($unloading->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'unloading_number' => 'required',
            'challan_number' => 'required',
            'arrival_from' => 'required',
        ], [
            'branch.required' => 'Required',
            'unloading_number.required' => 'Required',
            'challan_number.required' => 'Required',
            'arrival_from.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW UNLOADING
            $unloading->arrival_date = $request->arrival_date;
            $unloading->arrival_time = $request->arrival_time;

            $unloading->challan_number = $request->challan_number;
            $unloading->challan_date = $request->challan_date;
            $unloading->challan_time = $request->challan_time;

            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $unloading->vehicle_id = $vehicle_id;
            $unloading->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
                
            $unloading->vehicle_type = $request->vehicle_type;
            $unloading->driver_id = $request->driver ? $request->driver['id'] : null;
            $unloading->driver_name = $request->driver ? $request->driver['emp_name'] : null;

            $unloading->seal_number = $request->seal_number;
            $unloading->status = $request->status;
            $unloading->remarks = $request->remarks;

            $unloading->total_to_pay = $request->total_to_pay;
            $unloading->del_comm = $request->del_comm;
            $unloading->kaat = $request->kaat;
            $unloading->door_del = $request->door_del;
            $unloading->driver_amt_type = $request->driver_amt_type;
            $unloading->driver_amt = $request->driver_amt;
            $unloading->driver_amt_acc_id = $request->driver_amt_acc ? $request->driver_amt_acc['id'] : null;
            $unloading->branch_balance = $request->branch_balance;

            $unloading->updated_by = $request->user->id;
            $unloading->save();
            // NEW CHALLAN RECEIPT SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_unloading_desc = $request->newly_added_unloading_desc;
            foreach ($newly_added_unloading_desc as $naud) {
                if( !$naud['gr_number'] || !$naud['branch'] || !$naud['consignor'] || !$naud['consignee'] || !$naud['material'] || !$naud['pkg'] || !$naud['rate'] || !$naud['chg_unit'] )
                continue;

                $gr = new BookGr();
                $gr->branch_id = $naud['branch']['id'];
                $gr->gr_number = $naud['gr_number'];
                $gr->gr_date = $naud['gr_date'];
                $gr->gr_time = $request->challan_time;
                $source_id = $this->add_if_not_in_master('Station', $request->source, $request->current_year);
                $gr->source_id = $source_id;
                $gr->source_name = $naud['source']['station_name'];

                $destination_id = $this->add_if_not_in_master('Station', $request->destination, $request->current_year);
                $gr->destination_id = $destination_id;
                $gr->destination_name = $naud['destination']['station_name'];

                
                $gr->tpt_mode = 'SURFACE';

                $gr->crossing = 1;

                $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
                $gr->vehicle_id = $vehicle_id;
                $gr->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
                
                $gr->vehicle_type = $request->vehicle_type;
                $gr->driver_id = $request->driver ? $request->driver['id'] : null;
                $gr->driver_name = $request->driver ? $request->driver['emp_name'] : null;

                $gr->pay_mode = $naud['pay_mode'];

                $gr->consignor_id = $naud['consignor'] ? $naud['consignor']['id'] : null;
                $gr->consignor_name = $naud['consignor'] ? $naud['consignor']['party_name'] : null;

                $gr->consignee_id = $naud['consignee'] ? $naud['consignee']['id'] : null;
                $gr->consignee_name = $naud['consignee'] ? $naud['consignee']['party_name'] : null;

                $gr->acc_party_id = $naud['acc_party'] ? $naud['acc_party']['id'] : null;

                $gr->delivery_pt = $naud['delivery_pt'];

                $gr->total_pkg = $naud['pkg'];
                $gr->total_material_weight = $naud['material_weight'];
                $gr->total_chg_qty = $naud['pkg'];
                $gr->total_desc_value = $naud['desc_total'];

                $gr->p_marka = $naud['p_marka'];
                 
                $gr->freight = $naud['desc_total'];
                $gr->payment_dynamic_fields = $naud['payment_dynamic_fields'];
                $gr->total_without_rounding = $naud['total_without_rounding'];
                $gr->rounded_off = $naud['rounded_off'];
                $gr->grand_total = $naud['grand_total'];
                $gr->created_by = $request->user->id;
                $gr->fin_year = $request->current_year;
                $gr->save();


                $pkg_det = new GrDesc();
                $pkg_det->gr_id = $gr->id;
                $pkg_det->gr_number = $gr->gr_number;

                $material_id = $this->add_if_not_in_master('Material', $naud['material'], $request->current_year);
                $pkg_det->material_id = $material_id;
                
                $pkg_det->destination_id = $naud['destination']['id'];
                $pkg_det->packing_method = $naud['packing_method'];
                $pkg_det->pkg = $naud['pkg'];
                $pkg_det->material_weight = $naud['material_weight'];
                $pkg_det->chg_unit = $naud['chg_unit'];
                $pkg_det->chg_qty = $naud['pkg'];
                $pkg_det->rate = $naud['rate'];
                $pkg_det->total = $naud['desc_total'];

                $pkg_det->free_pkgs = $naud['pkg'];
                $pkg_det->fin_year = $request->current_year;
                $pkg_det->save();
                 
                
                // SAVING PACKAGE POSITIONS
                $pkg_position = new PackagePosition();
                $pkg_position->gr_id = $gr->id;
                $pkg_position->gr_number = $gr->gr_number;
                $pkg_position->gr_desc_id = $pkg_det->id;
                $pkg_position->position = $request->branch['id'];
                $pkg_position->pkg = $naud['pkg'];
                $pkg_position->origin = $request->branch['id'];
                $pkg_position->save();
                // PACKAGE POSITIONS SAVED

                $unld_desc = new UnloadingDesc();
                $unld_desc->unloading_id = $unloading->id;
                $unld_desc->gr_id = $gr->id;
                $unld_desc->gr_number = $gr->gr_number;
                $unld_desc->gr_desc_id = $pkg_det->id;
                $unld_desc->fin_year = $request->current_year;
                $unld_desc->save();
            }
            // PACKAGE DETAILS SAVED

            // SAVING UPDATES ON EXISTING  PACKAGE DETAILS
            $updated_unloading_desc = $request->updated_unloading_desc;
            if( !empty($updated_unloading_desc) ){
                foreach ($updated_unloading_desc as $ucd) {
                    if( !$ucd['gr_number'] || !$ucd['branch'] || !$ucd['consignor'] || !$ucd['consignee'] || !$ucd['material'] || !$ucd['pkg'] || !$ucd['rate'] || !$ucd['chg_unit'] )
                    continue;

                    $gr = BookGr::find($ucd['gr_id']);
                    $gr->payment_dynamic_fields = $ucd['payment_dynamic_fields'];
                    $gr->total_without_rounding = $ucd['total_without_rounding'];
                    $gr->rounded_off = $ucd['rounded_off'];
                    $gr->grand_total = $ucd['grand_total'];
                    $gr->updated_by = $request->user->id;
                    $gr->save();
                }
            }
            // UPDATES SAVED ON EXISTING PACKAGE DETAILS

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_unloading_desc = $request->deleted_unloading_desc;
            if( !empty($deleted_unloading_desc) ){
                foreach ($deleted_unloading_desc as $dcd) {
                    $unld_desc = UnloadingDesc::find($dcd);
                    $unld_desc->delete();
                    
                                        
                }      
            }
            //DELETED EXISTING PACKAGE DETAILS
            

            $this->record_transaction_in_accounts($request, $unloading);


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($unloading->id);
            $this->createEditLog('updated', 'unloadings', $unloading->id, $unloading->unloading_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'Unloading');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $unloading = Unloading::find($pk);

         // DELETE THE TRANSACTION VOUCHER
         $t_data = [];
         $t_data['t_against'] = 'Unloading';
         $t_data['t_against_no'] = $unloading->unloading_number;
         $t_data['t_against_id'] = $unloading->id;
         $t_data['fin_year'] = $request->current_year;
 
         $this->delete_transaction_voucher($t_data);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($unloading->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $unloading_desc_ids = UnloadingDesc::select('id')->where('unloading_id', $pk)->pluck('id')->toArray();
        
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        // DELETING CHALLAN DESCRIPTIONS
        foreach ($unloading_desc_ids as $cdi) {
            $unld_desc = UnloadingDesc::find($cdi);
            $unld_desc->delete();           
        }

        //FIND AND DELETE THE BRANCH
        $unloading->delete();

        $this->createEditLog('deleted', 'unloadings', $unloading->id, $unloading->unloading_number, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = Unloading::select('id', 'unloading_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }


        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('unloading_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $unloading = $query->get();
        return response(['unloadings' => $unloading, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Unloading');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Unloading::select('id', 'unloading_number', 'arrival_from', 'arrival_date', 'arrival_time' ,'challan_number', 'challan_date','challan_time' , 'branch_id')
                ->with(['branch:id,branch_name,branch_code', 'arrival_from:id,branch_name,branch_code',]) 
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('unloading_number', 'like', $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Unloading::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $unloadingQuery = Unloading::select('id', 'unloading_number')->where('fin_year',$request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $unloadingQuery->whereIn('branch_id', $request->user_branch_ids);
        }
    
        $unloadings = $unloadingQuery->orderBy('id', 'desc')->take(10)->get();
    
        return response()->json(['unloadings' => $unloadings]);
    }
    


    public function get_next_unloading_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        if($type=='SYSTEM'){
            $max = Unloading::where('branch_id',$branch_id)->where('unloading_type', $type)->max('max_number');
            $unloading_number = $branch_code.'-'.sprintf('%06d', $max+1);
            return response()->json(['unloading_number' => strtoupper($unloading_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = Unloading::where('prefix',$prefix)->where('unloading_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Unloading')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $unloading_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['unloading_number' => strtoupper($unloading_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Unloading')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $unloading_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['unloading_number' => strtoupper($unloading_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['unloading_number' => ['No Valid Unloading Number']]],422);
            }
        }
    }

    protected function get_data_for_obj($pk){
       //FETCH AND SEND ALL
       $unloading = Unloading::with([
            'branch:id,branch_name,branch_code',
            'arrival_from:id,branch_name,branch_code',
            'vehicle:id,vehicle_number',
            'unloading_desc',
            'driver_amt_acc:id,account_name',
            'unloading_desc.gr_desc:id,gr_id,gr_number,pkg,material_id,material_weight,packing_method,chg_unit,rate,total',
            'unloading_desc.gr_desc.material:id,material_name',
            'unloading_desc.bookGr',
            'unloading_desc.bookGr.branch:id,branch_name,branch_code',
            'unloading_desc.bookGr.consignor:id,party_name',
            'unloading_desc.bookGr.consignee:id,party_name',
            'unloading_desc.bookGr.source:id,station_name',
            'unloading_desc.bookGr.destination:id,station_name',
        ])->find($pk);

        $unloading->unloading_desc->transform(function ($desc) {
            return [
                'unloading_desc_id' => $desc->id,
                'gr_id' => $desc->gr_id,
                'gr_number' => $desc->gr_number,
                'gr_desc_id' => $desc->gr_desc_id,
                'gr_date' => $desc->bookGr->gr_date,
                'branch' => $desc->bookGr->branch,
                'consignor' => $desc->bookGr->consignor ? $desc->bookGr->consignor : ['id'=>null, 'party_name'=>$desc->bookGr->consignor_name],
                'consignee' => $desc->bookGr->consignee ? $desc->bookGr->consignee : ['id'=>null, 'party_name'=>$desc->bookGr->consignee_name],
                'source' => $desc->bookGr->source ? $desc->bookGr->source : ['id'=>null, 'station_name'=>$desc->bookGr->source_name],
                'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=>null, 'station_name'=>$desc->bookGr->destination_name],
                'pay_mode' => $desc->bookGr->pay_mode,
                'delivery_pt' => $desc->bookGr->delivery_pt,
                'p_marka' => $desc->bookGr->p_marka,
                'material' => $desc->gr_desc->material,
                'pkg' => $desc->gr_desc->pkg,
                'material_weight' => $desc->gr_desc->material_weight,
                'packing_method' => $desc->gr_desc->packing_method,
                'chg_unit' => $desc->gr_desc->chg_unit,
                'rate' => $desc->gr_desc->rate,
                'desc_total' => $desc->gr_desc->total,
                'payment_dynamic_fields' => $desc->bookGr->payment_dynamic_fields,
                'grand_total' => $desc->bookGr->grand_total,
                'total_without_rounding' => $desc->bookGr->total_without_rounding,
                'rounded_off' => $desc->bookGr->rounded_off
            ];
        });

        return ['unloading'=>$unloading];
    }


    protected function record_transaction_in_accounts($request, $unloading){
        $t_data = [];
        $t_data['t_against'] = 'Unloading';
        $t_data['t_against_id'] = $unloading->id;
        $t_data['fin_year'] = $unloading->fin_year;
        $t_data['t_against_no'] = $unloading->unloading_number;
        
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $unloading->arrival_date;
        $t_data['voucher_time'] = $unloading->arrival_time;
        $t_data['t_against_narration'] = $unloading->remarks;
        $t_data['fin_year'] = $unloading->fin_year;

        $t_data['voucher_accounting_details'] = [];
    
            $t_data['cancellable'] = 0;


            $arrival_from_id = $request->arrival_from['account_id'];

            
            $total_to_pay = $unloading->total_to_pay;
            $to_pay_delivery_stock = $this->get_acc_id('TO PAY DELIVERY STOCK');

            if(!$to_pay_delivery_stock){
                dd('TO PAY DELIVERY STOCK Account Not Found');
            }
            if($to_pay_delivery_stock && $arrival_from_id && $unloading->total_to_pay>0){

                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$to_pay_delivery_stock, 'nature'=>'dr' , 'dr_amount'=>$unloading->total_to_pay, 'cr_amount'=>0 ];
                    
                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$arrival_from_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$unloading->total_to_pay ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                                // Round both $total_dr and $total_cr to two decimal places
                    $total_dr_rounded = round(floatval($total_dr), 2);
                    $total_cr_rounded = round(floatval($total_cr), 2);

                    // Check if they are not equal
                    if ($total_dr_rounded != $total_cr_rounded) {
                        // Output the values for debugging
                        dd($total_dr_rounded, $total_cr_rounded);
                        
                        // Return response with an error message
                        dd('Error In Accounting Entries');
                    }


                                $t_data['total_dr_amount'] = $total_dr;
                                $t_data['total_cr_amount'] = $total_cr; 


                                $this->create_transaction_voucher( $t_data );
            }

            $delivery_commision = $this->get_acc_id('DELIVERY COMMISSION');
            if(!$delivery_commision){
                dd('DELIVERY COMMISSION Account Not Found');
            }

            if($delivery_commision && $arrival_from_id && $unloading->del_comm>0){

                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$arrival_from_id, 'nature'=>'dr' , 'dr_amount'=>$unloading->del_comm, 'cr_amount'=>0 ];
                
                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$delivery_commision, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$unloading->del_comm ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                            // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);

                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    dd('Error In Accounting Entries');
                }


                            $t_data['total_dr_amount'] = $total_dr;
                            $t_data['total_cr_amount'] = $total_cr; 


                            $this->create_transaction_voucher( $t_data );
            }

            $kaat = $this->get_acc_id('KAAT');
            if(!$kaat){
                dd('KAAT Account Not Found');
            }

            if($kaat && $arrival_from_id && $unloading->kaat>0){

                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$arrival_from_id, 'nature'=>'dr' , 'dr_amount'=>$unloading->kaat, 'cr_amount'=>0 ];
                
                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$kaat, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$unloading->kaat ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                            // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);

                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    dd('Error In Accounting Entries');
                }

                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 
                $this->create_transaction_voucher( $t_data );

            }

            $door_delivery = $this->get_acc_id('DOOR DELIVERY');
            if(!$door_delivery){
                dd('DOOR DELIVERY Account Not Found');
            }

            if($door_delivery && $arrival_from_id && $unloading->door_del>0){

                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$arrival_from_id, 'nature'=>'dr' , 'dr_amount'=>$unloading->door_del, 'cr_amount'=>0 ];
                
                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$door_delivery, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$unloading->door_del ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                            // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);

                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    dd('Error In Accounting Entries');
                }

                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 
                $this->create_transaction_voucher( $t_data );

            }


            if($arrival_from_id && $unloading->driver_amt>0){

                
                $driver_amt_acc_id = $unloading->driver_amt_acc_id;

                if(!$driver_amt_acc_id){
                    dd('DRIVER AMOUNT Account Not Found');
                }

                if($unloading->driver_amt_type=='Paid To Driver'){
                    $dr_ac = $arrival_from_id;
                    $cr_ac = $driver_amt_acc_id;
                }
                else{
                    $dr_ac = $driver_amt_acc_id;
                    $cr_ac = $arrival_from_id;
                }

                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$dr_ac, 'nature'=>'dr' , 'dr_amount'=>$unloading->driver_amt, 'cr_amount'=>0 ];
                
                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$cr_ac, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$unloading->driver_amt ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                            // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);

                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    dd('Error In Accounting Entries');
                }

                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 
                $this->create_transaction_voucher( $t_data );
            }


            

    
}

    protected function get_max_number_from_unloading_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }
}
