<?php

namespace App\Http\Controllers\Inventory;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Bill\BillDesc;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\Inventory\InventoryBalances;
use App\Models\Inventory\ProductSerial;
use App\Models\Inventory\Purchase;
use App\Models\Inventory\PurchaseDesc;
use App\Models\Master\Vehicle;
use App\Models\Master\VehicleTyres;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class PurchaseController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Bills');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $bill = Purchase::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name,unit,maintain_serial',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.dr_account:id,account_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
        ])->where('fin_year', $request->current_year)->get();


        return response()->json(['bills'=>$bill]);
    }

    public function show(Request $request, $pk){
        //FETCH AND SEND ALL
        $bill = Purchase::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name,unit,maintain_serial',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.dr_account:id,account_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
        ])->find($pk);


        return response()->json(['bills'=>$bill]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'Bills');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'bill_number' => 'required',
        ], [
            'branch.required' => 'Required',
            'bill_number.required' => 'Required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_bill_number($request->bill_number);

            
            // SAVING A NEW Bill
            $bill = new Purchase();
            $bill->bill_name = $request->bill_name;
            $bill->branch_id = $request->branch['id'];
            $bill->bill_type = $request->bill_type;
            $bill->prefix = $request->prefix;
            $bill->max_number = $max_number;
            $bill->bill_number = $request->bill_number;
            $bill->bill_date = $request->bill_date;
            $bill->acc_party_id = $request->acc_party?$request->acc_party['id']:null;
            $bill->invoice_number = $request->invoice_number;
            $bill->invoice_date = $request->invoice_date;
            $bill->bill_attachment = $request->bill_attachment; 
            $bill->bill_attachment2 = $request->bill_attachment2;
            $bill->bill_attachment3 = $request->bill_attachment3;
            $bill->bill_attachment4 = $request->bill_attachment4;
            $bill->bill_attachment5 = $request->bill_attachment5;
            $bill->remarks = $request->remarks;
            $bill->total_discount = $request->total_discount;
            $bill->total_without_gst = $request->total_without_gst;
            $bill->total_igst_amount = $request->total_igst_amount;
            $bill->total_cgst_amount = $request->total_cgst_amount;
            $bill->total_sgst_amount = $request->total_sgst_amount;
            $bill->total_without_rounding = $request->total_without_rounding;
            $bill->rounded_off = $request->rounded_off;
            $bill->grand_total_amount = $request->grand_total_amount;
            
            $bill->fin_year = $request->current_year;
            $bill->is_round_off = $request->is_round_off;

            $bill->created_by = $request->user->id;

            // Save the model to the database
            $bill->save();

             

            // SAVING BILL DESC
            $bill_desc = $request->bill_desc;

                foreach ($bill_desc as $bd) {
                    if($bill->bill_name == 'Stock Issue' || $bill->bill_name == 'Sale'){
                        $bal_data = [
                            'product_id'=>$bd['product']['id'],
                            'branch_id'=>$bill->branch_id,
                            'current_year'=>$request->current_year,
                        ];

                        $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                        $current_bal = $balance_of_stock['balance']['current_balance'];

                        $future_bal = $current_bal - $bd['qty'];

                        if($future_bal<0){
                            return response()->json(['error'=>'Current Balance Exceeded', 'current_bal'=>$current_bal], 500);
                        }
                    }

                    $bill_desc = new PurchaseDesc();
                    $bill_desc->bill_id = $bill->id;
                    $bill_desc->bill_number = $bill->bill_number;
                    $bill_desc->product_id = $bd['product']['id'];
                    $bill_desc->dr_account_id = $bd['dr_account']?$bd['dr_account']['id']:null;
                    $bill_desc->vehicle_id = $bd['vehicle']?$bd['vehicle']['id']:null;
                    $bill_desc->reading = $bd['reading'];
                    $bill_desc->hsn = $bd['hsn'];
                    $bill_desc->qty = $bd['qty'];
                    $bill_desc->rate = $bd['rate'];
                    $bill_desc->discount = $bd['discount'];
                    $bill_desc->amount = $bd['amount'];
                    $bill_desc->cgst_percent = $bd['cgst_percent'];
                    $bill_desc->cgst_amount = $bd['cgst_amount'];
                    $bill_desc->sgst_percent = $bd['sgst_percent'];
                    $bill_desc->sgst_amount = $bd['sgst_amount'];
                    $bill_desc->igst_percent = $bd['igst_percent'];
                    $bill_desc->igst_amount = $bd['igst_amount'];
                    $bill_desc->net_amount = $bd['net_amount'];
                    $bill_desc->fin_year = $request->current_year;
                    $bill_desc->save();

                    $product_serial_desc = $bd['product_serial_desc'];

                    foreach ($product_serial_desc as $psd) {
                        $product_serial = new ProductSerial();
                        $product_serial->bill_id = $bill->id;
                        $product_serial->bill_number = $bill->bill_number;
                        $product_serial->product_id = $psd['product_id'];
                        $product_serial->product_name = $psd['product_name'];
                        $product_serial->ref_number = $psd['ref_number'] ?? null;
                        $product_serial->position = $psd['position'] ?? null;
                        $product_serial->replacing_number = $psd['replacing_number'] ?? null;
                        $product_serial->vehicle_id = $psd['vehicle'] ? $psd['vehicle']['id'] : null;
                        $product_serial->life_km = $psd['life_km'] ?? null;
                        $product_serial->bill_desc_id = $bill_desc->id;
                        $product_serial->save(); 
                    }


                   

                }

            // BILL DESC SAVED

            
            

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $bill);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($bill->id);
            $this->createEditLog('created', 'purchases', $bill->id, $bill->bill_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$bill->id, 'bill_number'=>$bill->bill_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Bills');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $bill = Purchase::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($bill->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'bill_number' => 'required',
        ], [
            'branch.required' => 'Required',
            'bill_number.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);
            
            // SAVING A NEW BILL
            $bill->bill_date = $request->bill_date;
            $bill->acc_party_id = $request->acc_party?$request->acc_party['id']:null;
            $bill->invoice_number = $request->invoice_number;
            $bill->invoice_date = $request->invoice_date;
            $bill->remarks = $request->remarks;
            $bill->bill_attachment = $request->bill_attachment;
            $bill->bill_attachment2 = $request->bill_attachment2;
            $bill->bill_attachment3 = $request->bill_attachment3;
            $bill->bill_attachment4 = $request->bill_attachment4;
            $bill->bill_attachment5 = $request->bill_attachment5;
            $bill->total_discount = $request->total_discount;
            $bill->total_without_gst = $request->total_without_gst;
            $bill->total_igst_amount = $request->total_igst_amount;
            $bill->total_cgst_amount = $request->total_cgst_amount;
            $bill->total_sgst_amount = $request->total_sgst_amount;
            $bill->total_without_rounding = $request->total_without_rounding;
            $bill->rounded_off = $request->rounded_off;
            $bill->is_round_off = $request->is_round_off;
            $bill->grand_total_amount = $request->grand_total_amount;

            $bill->updated_by = $request->user->id;
            $bill->save();
            // NEW BILL SAVED
            
            // SAVING Bill DESC
            $newly_added_bill_desc = $request->newly_added_bill_desc;
            foreach ($newly_added_bill_desc as $nabd) { 
                if($bill->bill_name == 'Stock Issue' || $bill->bill_name == 'Sale'){
                    $bal_data = [
                        'product_id'=>$nabd['product']['id'],
                        'branch_id'=>$bill->branch_id,
                        'current_year'=>$request->current_year,
                    ];

                    $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                    $current_bal = $balance_of_stock['balance']['current_balance'];

                    $future_bal = $current_bal - $nabd['qty'];

                    if($future_bal<0){
                        return response()->json(['error'=>'Current Balance Exceeded', 'current_bal'=>$current_bal], 500);
                    }
                }

                $bill_desc = new PurchaseDesc();
                $bill_desc->bill_id = $bill->id;
                $bill_desc->bill_number = $bill->bill_number;
                $bill_desc->product_id = $nabd['product']['id'];
                $bill_desc->dr_account_id = $nabd['dr_account']?$nabd['dr_account']['id']:null;
                $bill_desc->vehicle_id = $nabd['vehicle']?$nabd['vehicle']['id']:null;
                $bill_desc->reading = $nabd['reading'];
                $bill_desc->hsn = $nabd['hsn'];
                $bill_desc->qty = $nabd['qty'];
                $bill_desc->rate = $nabd['rate'];
                $bill_desc->discount = $nabd['discount'];
                $bill_desc->amount = $nabd['amount'];
                $bill_desc->cgst_percent = $nabd['cgst_percent'];
                $bill_desc->cgst_amount = $nabd['cgst_amount'];
                $bill_desc->sgst_percent = $nabd['sgst_percent'];
                $bill_desc->sgst_amount = $nabd['sgst_amount'];
                $bill_desc->igst_percent = $nabd['igst_percent'];
                $bill_desc->igst_amount = $nabd['igst_amount'];
                $bill_desc->net_amount = $nabd['net_amount'];
                $bill_desc->fin_year = $request->current_year;
                $bill_desc->save();

                $product_serial_desc = $nabd['product_serial_desc'];

                foreach ($product_serial_desc as $psd) {
                    $product_serial = new ProductSerial();
                    $product_serial->bill_id = $bill->id;
                    $product_serial->bill_number = $bill->bill_number;
                    $product_serial->product_id = $psd['product_id'];
                    $product_serial->product_name = $psd['product_name'];
                    $product_serial->ref_number = $psd['ref_number'] ?? null;
                    $product_serial->position = $psd['position'] ?? null;
                    $product_serial->replacing_number = $psd['replacing_number'] ?? null;
                    $product_serial->vehicle_id = $psd['vehicle'] ? $psd['vehicle']['id'] : null;
                    $product_serial->life_km = $psd['life_km'] ?? null;
                    $product_serial->bill_desc_id = $bill_desc->id;
                    $product_serial->save(); 
                }
                
            }
            // Bill DESC SAVED

            $updated_bill_desc = $request->updated_bill_desc;
            foreach ($updated_bill_desc as $ubd) {    
                $bill_desc = PurchaseDesc::find($ubd['id']);

                $bal_data = [
                    'product_id'=>$ubd['product']['id'],
                    'branch_id'=>$bill->branch_id,
                    'current_year'=>$request->current_year,
                ];

                $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                $current_bal = $balance_of_stock['balance']['current_balance'];

                if($bill->bill_name == 'Purchase' || $bill->bill_name == 'Stock Inwards')
                $future_bal = $current_bal - $bill_desc['qty'] + $ubd['qty'];
                else if($bill->bill_name == 'Stock Issue' || $bill->bill_name == 'Sale')
                $future_bal = $current_bal + $bill_desc['qty'] - $ubd['qty'];


                if($future_bal<0){
                    return response()->json(['error'=>'Current Balance Exceeded', 'current_bal'=>$current_bal], 500);
                }

                $bill_desc->bill_id = $bill->id;
                $bill_desc->bill_number = $bill->bill_number;
                $bill_desc->qty = $ubd['qty'];
                $bill_desc->reading = $ubd['reading'];
                $bill_desc->rate = $ubd['rate'];
                $bill_desc->discount = $ubd['discount'];
                $bill_desc->amount = $ubd['amount'];
                $bill_desc->cgst_percent = $ubd['cgst_percent'];
                $bill_desc->cgst_amount = $ubd['cgst_amount'];
                $bill_desc->sgst_percent = $ubd['sgst_percent'];
                $bill_desc->sgst_amount = $ubd['sgst_amount'];
                $bill_desc->igst_percent = $ubd['igst_percent'];
                $bill_desc->igst_amount = $ubd['igst_amount'];
                $bill_desc->net_amount = $ubd['net_amount'];
                $bill_desc->save();

                $product_serial = ProductSerial::where('bill_desc_id', $bill_desc->id)->delete();
                 
                $product_serial_desc = $ubd['product_serial_desc'];
                foreach ($product_serial_desc as $psd) {
                    $product_serial = new ProductSerial();
                    $product_serial->bill_id = $bill->id;
                    $product_serial->bill_number = $bill->bill_number;
                    $product_serial->product_id = $psd['product_id'];
                    $product_serial->product_name = $psd['product_name'];
                    $product_serial->ref_number = $psd['ref_number'] ?? null;
                    $product_serial->position = $psd['position'] ?? null;
                    $product_serial->replacing_number = $psd['replacing_number'] ?? null;
                    $product_serial->life_km = $psd['life_km'] ?? null;
                    $product_serial->vehicle_id = $psd['vehicle'] ? $psd['vehicle']['id'] : null;
                    $product_serial->bill_desc_id = $bill_desc->id;
                    $product_serial->save(); 
                }

            }



            // $updated_product_serial = $request->updated_product_serial;
            // foreach ($updated_product_serial as $psd) {
            //     $product_serial = ProductSerial::find($psd['id']);
            //     $product_serial->ref_number = $psd['ref_number'] ?? null;
            //     $product_serial->position = $psd['position'] ?? null;
            //     $product_serial->life_km = $psd['life_km'] ?? null;
            //     $product_serial->save(); 
            // }
                

            // DELETING EXISTING POD DESC
            $deleted_bill_desc = $request->deleted_bill_desc;
            if( !empty($deleted_bill_desc) ){
                foreach ($deleted_bill_desc as $dbd) {
                    $d_bil_d = PurchaseDesc::find($dbd);

                    if($bill->bill_name == 'Purchase' || $bill->bill_name == 'Stock Inwards'){
                        $bal_data = [
                            'product_id'=>$d_bil_d['product_id'],
                            'branch_id'=>$bill->branch_id,
                            'current_year'=>$request->current_year,
                        ];

                        $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                        $current_bal = $balance_of_stock['balance']['current_balance'];

                        $future_bal = $current_bal - $d_bil_d['qty'];

                        if($future_bal<0){
                            return response()->json(['error'=>'Current Balance Falls Below 0', 'current_bal'=>$current_bal], 500);
                        }
                    }

                    $product_serial = ProductSerial::where('bill_desc_id', $d_bil_d->id)->delete();

                    $d_bil_d->delete();
                }      
            }
            //DELETED EXISTING POD DESC

            // RECORD THE TRANSACTION IN ACCOUNTS
             $this->record_transaction_in_accounts($request, $bill);
            // TRANSACTION RECORDED IN ACCOUNTS
            
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($bill->id);
            $this->createEditLog('updated', 'purchases', $bill->id, $bill->bill_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'Bills');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
        
        $bill = Purchase::find($pk);
                
        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($bill->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // LOOP THROUGH BILL DESC, CHECK selection_type, If 'GR' send gr_number  , If 'Loading Slip' send ls_number
        // $bill_desc_ids = PurchaseDesc::select('id')->where('bill_id', $bill->id)->pluck('id')->toArray();
        
        // DELETING BILL DESCRIPTIONS

        $product_serial = ProductSerial::where('bill_id',$pk)->delete();

        $bill_desc = PurchaseDesc::select('id')->where('bill_id', $pk)->pluck('id')->toArray();

        // DELETING EXISTING POD DESC
        if( !empty($bill_desc) ){
            foreach ($bill_desc as $dbd) {
                $d_bil_d = PurchaseDesc::find($dbd);
                  
                  
                if($bill->bill_name == 'Purchase' || $bill->bill_name == 'Stock Inwards'){
                    $bal_data = [
                        'product_id'=>$d_bil_d['product_id'],
                        'branch_id'=>$bill->branch_id,
                        'current_year'=>$request->current_year,
                    ];



                    $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                    $current_bal = $balance_of_stock['balance']['current_balance'];

                    $future_bal = $current_bal - $d_bil_d['qty'];

                    if($future_bal<0){
                        return response()->json(['error'=>'Current Balance Falls Below 0', 'current_bal'=>$current_bal], 500);
                    }
                }

                $d_bil_d->delete();
            }      
        }
        //DELETED EXISTING POD DESC


        //FIND AND DELETE THE BILL
        $bill->delete();


        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'Purchase Bills';
        $t_data['t_against_id'] = $bill->id;
        $t_data['t_against_no'] = $bill->bill_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $this->createEditLog('deleted', 'purchases', $bill->id, $bill->bill_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = Purchase::select('id', 'bill_number')->where('bill_name',$request->bill_name)->where('fin_year', $request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

            // Loop through the whereFilters array
            foreach ($whereFilters as $whereFilter) {
                // Apply the filter to the query
                $query->where($whereFilter['key'], $whereFilter['value']);
            }

            // Loop through the orWhereFilters array
            foreach ($orWhereFilters as $orWhereFilter) {
                // Apply the filter to the query
                $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
            }
        
        }


        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('bill_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $bill = $query->get();

        return response(['bills' => $bill, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Bills');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Purchase::with(['branch:id,branch_name,branch_code',
        'acc_party:id,account_name',
        'purchase_desc',
        'purchase_desc.product:id,product_name,unit,maintain_serial',
        'purchase_desc.vehicle:id,vehicle_number',
        'purchase_desc.dr_account:id,account_name',
        'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number'])->where('fin_year', $request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Purchase::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $billQuery = Purchase::select('id', 'bill_number')->where('fin_year', $request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $billQuery->whereIn('branch_id', $request->user_branch_ids);
        }

        if(isset($request->bill_name)){
            $billQuery->where('bill_name',$request->bill_name);
        }
    
        $bills = $billQuery->orderBy('id', 'desc')->take(10)->get();
    
        return response()->json(['bills' => $bills]);
    }

    public function get_unused_ref_numbers(Request $request){
        $product_id = $request->product_id;

        $query = ProductSerial::select('id', 'product_id', 'vehicle_id','bill_id', 'ref_number','bill_id')
                ->with(["purchase:id,bill_number,bill_name"])
                ->where('product_id', $product_id)
                ->whereIn('id', function ($subquery) {
                    $subquery->select(DB::raw('MAX(id)'))
                        ->from('product_serials')
                        ->groupBy('ref_number');
                })
                ->whereHas('purchase', function($qr) {
                    $qr->where('bill_name','!=','Sale');
                })
                ->whereNotNull('ref_number');

            // Check if searchVariable is provided and apply the filter
            if ($request->has('searchVariable')) {
                $searchVariable = $request->searchVariable;
                $query->where('ref_number', 'like', '%' . $searchVariable . '%');
            }

            $results = $query->get();

            $filtered = [];
            // Filter the results based on the condition
            foreach ($results as $key ) {
                if ($key->purchase && $key->purchase->bill_name != 'Stock Inwards' && $key->vehicle_id === null) {
                    $filtered[] = $key;
                }
            
                if($key->purchase->bill_name == 'Stock Inwards')
                {
                    $filtered[] = $key;
                }
            }



        
        // Pluck ref_numbers from product_serials
        // $query = ProductSerial::select('id', 'product_id', 'ref_number')
        //         ->where('product_id', $product_id)
        //         ->whereNull('vehicle_id'); // Select records where vehicle_id is null

        // // Exclude records with ref_number having associated vehicle_id
        // $query->whereNotExists(function ($subquery) {
        //         $subquery->select(DB::raw(1))
        //             ->from('product_serials as psd')
        //             ->whereRaw('psd.ref_number = product_serials.ref_number')
        //             ->whereNotNull('vehicle_id');
        //     });

        

        // // Check if searchVariable is provided and apply the filter
        // if ($request->has('searchVariable')) {
        //      $searchVariable = $request->searchVariable;
        //      $query->where('ref_number', 'like', '%' . $searchVariable . '%');
        // }



        return response()->json(['product_serials' => $filtered]);
    }

    public function autocomplete_for_serials(Request $request){
        $product_id = $request->product_id;
        $vehicle_id = $request->vehicle_id;

        

        // First query for ProductSerial
        // First query for ProductSerial
        $query = ProductSerial::select('ref_number', DB::raw('MAX(id) as max_id'))
        ->where('product_id', $product_id)
        ->whereNotNull('ref_number')
        ->groupBy('ref_number');

        // Second query for VehicleTyres
        $query2 = VehicleTyres::select('ref_number', DB::raw('MAX(id) as max_id'))
        ->where('product_id', $product_id)
        ->whereNotNull('ref_number')
        ->groupBy('ref_number');

        // Check if searchVariable is provided and apply the filter to both queries
        if ($request->has('searchVariable')) {
        $searchVariable = $request->searchVariable;
        $query->where('ref_number', 'like', '%' . $searchVariable . '%');
        $query2->where('ref_number', 'like', '%' . $searchVariable . '%');
        }

        // Union the two queries
        $unionQuery = $query->union($query2);

        // Retrieve results, ordering by the maximum ID
        $results = DB::table(DB::raw("({$unionQuery->toSql()}) as unioned"))
        ->mergeBindings($unionQuery->getQuery()) // Merging bindings to avoid parameter issues
        ->orderBy('max_id', 'desc')
        ->limit(50)
        ->get();


        return response()->json(['product_serials' => $results]);

    }

    public function autocomplete_for_replacing(Request $request){
        $positions = ['L1', 'L2 out', 'L2 in', 'L3 out', 'L3 in', 'L4 out', 'L4 in', 'L5 out', 'L5 in', 'L6 out', 'L6 in', 'R1', 'R2 out', 'R2 in', 'R3 out', 'R3 in', 'R4 out', 'R4 in', 'R5 out', 'R5 in', 'R6 out', 'R6 in','Position-less','Position-less 2','Position-less 3','Position-less 4','Position-less 5'];
    
        $product_id = $request->product_id;
        $vehicle_id = $request->vehicle_id;

        $searchVariable = $request->searchVariable;
    
        $results = [];
    
             // Get distinct positions for the current vehicle from VehicleTyres and ProductSerials
            $vehicleTyrePositions = VehicleTyres::where('vehicle_id', $vehicle_id)
                                    ->whereNotNull("position")
                                    ->distinct()
                                    ->pluck('position')
                                    ->toArray();

            $productSerialPositions = ProductSerial::where('vehicle_id', $vehicle_id)
                                    ->whereNotNull("position")
                                    ->distinct()
                                    ->pluck('position')
                                    ->toArray();

            // Combine positions and remove duplicates
            $positions = array_unique(array_merge($vehicleTyrePositions, $productSerialPositions));


            // Iterate through each position
            foreach ($positions as $position) {
                $productSerial = ProductSerial::where('vehicle_id',$vehicle_id)
                    ->where('position', $position)
                    ->when($product_id, function($query) use ($product_id) {
                        $query->where('product_id', $product_id);
                    })
                    ->whereNotNull('ref_number')
                    ->whereHas('purchase', function($query) {
                        $query->where('bill_name', '!=', 'Sale');
                    })
                    ->orderBy('id', 'desc')
                    ->first();
    
                if ($productSerial) {
                    $results[] = [
                        'position' => $position,
                        'product_name' => $productSerial->product->product_name,
                        'ref_number' => $productSerial->ref_number,
                        'id' => ''
                    ];
                } else {
                    // Check in vehicle tyres if no result in ProductSerial
                    $vehicleTyre = VehicleTyres::where('vehicle_id', $vehicle_id)
                        ->where('position', $position)
                        ->when($product_id, function($query) use ($product_id) {
                            $query->where('product_id', $product_id);
                        })
                        ->where('fin_year', $request->current_year)
                        ->first();
    
                    if ($vehicleTyre) {
                        $results[] = [
                            'position' => $vehicleTyre->position,
                            'product_name' => $vehicleTyre->product->product_name,
                            'ref_number' => $vehicleTyre->ref_number,
                            'id' => ''
                        ];
                    } else {
                        // If no data found, set to null
                        $results[] = [
                            'position' => $position,
                            'product_name' => null,
                            'ref_number' => null,
                            'id' => ''
                        ];
                    }
                }
            }


            // I want to filter results where ref_Number is like searchVariable
            if ($searchVariable) {
                $results = array_filter($results, function($result) use ($searchVariable) {
                    return strpos($result['ref_number'], $searchVariable) !== false;
                });
                $results = array_values($results);
            }

        return response()->json(['product_serials' => $results]);
    }

    public function get_vehicle_stock_report(Request $request) {
        $positions = ['L1', 'L2 out', 'L2 in', 'L3 out', 'L3 in', 'L4 out', 'L4 in', 'L5 out', 'L5 in', 'L6 out', 'L6 in', 'R1', 'R2 out', 'R2 in', 'R3 out', 'R3 in', 'R4 out', 'R4 in', 'R5 out', 'R5 in', 'R6 out', 'R6 in','Satepany','Position-less','Position-less 2','Position-less 3','Position-less 4','Position-less 5','Position-less 6','Position-less 7','Position-less 8','Position-less 9'];
    
    
        $product_id = $request->report_data['product']['id'] ?? null;
        $vehicle_id = $request->report_data['vehicle']['id'] ?? null;
    
        $results = [];
    
        // Get all vehicles or filter by vehicle_id if provided
        $vehicles = Vehicle::select('id', 'vehicle_number', 'owner_id')
            ->when($vehicle_id, function($query) use ($vehicle_id) {
                $query->where('id', $vehicle_id);
            })
            ->where('self_market','SELF')
            ->get();
    
        foreach ($vehicles as $vehicle) {
            $vehicle_data = ['vehicle_number' => $vehicle->vehicle_number, 'positions' => []];
           
             // Get distinct positions for the current vehicle from VehicleTyres and ProductSerials
            $vehicleTyrePositions = VehicleTyres::where('vehicle_id', $vehicle->id)
                                    ->whereNotNull("position")
                                    ->distinct()
                                    ->pluck('position')
                                    ->toArray();

            $productSerialPositions = ProductSerial::where('vehicle_id', $vehicle->id)
                                    ->whereNotNull("position")
                                    ->distinct()
                                    ->pluck('position')
                                    ->toArray();

            // Combine positions and remove duplicates
            $positions = array_unique(array_merge($vehicleTyrePositions, $productSerialPositions));


            // Iterate through each position
            foreach ($positions as $position) {
                $productSerial = ProductSerial::where('vehicle_id',$vehicle['id'])
                    ->where('position', $position)
                    ->when($product_id, function($query) use ($product_id) {
                        $query->where('product_id', $product_id);
                    })
                    ->whereNotNull('ref_number')
                    ->whereHas('purchase', function($query) {
                        $query->where('bill_name', '!=', 'Sale');
                    })
                    ->orderBy('bill_id', 'desc')
                    ->first();
    
                if ($productSerial) {
                    $vehicle_data['positions'][] = [
                        'position' => $position,
                        'product_name' => $productSerial->product->product_name,
                        'ref_number' => $productSerial->ref_number
                    ];
                } else {
                    // Check in vehicle tyres if no result in ProductSerial
                    $vehicleTyre = VehicleTyres::where('vehicle_id', $vehicle['id'])
                        ->where('position', $position)
                        ->when($product_id, function($query) use ($product_id) {
                            $query->where('product_id', $product_id);
                        })
                        ->where('fin_year', $request->current_year)
                        ->first();
    
                    if ($vehicleTyre) {
                        $vehicle_data['positions'][] = [
                            'position' => $vehicleTyre->position,
                            'product_name' => $vehicleTyre->product->product_name,
                            'ref_number' => $vehicleTyre->ref_number
                        ];
                    } else {
                        // If no data found, set to null
                        $vehicle_data['positions'][] = [
                            'position' => $position,
                            'product_name' => null,
                            'ref_number' => null
                        ];
                    }
                }
            }
    
            $results[] = $vehicle_data;
        }
    
        return response()->json($results);
    }


    public function get_replaced_ref_number(Request $request){
            $product_id = $request->product_id;
            $vehicle_id = $request->vehicle_id;
            $position = $request->position;
            $replacing_number = $request->replacing_number;

            $results = null;
    
            $query = ProductSerial::select('id', 'product_id', 'vehicle_id', 'bill_id','position', 'ref_number','bill_id')
                ->with(['purchase:id,bill_number,bill_name'])
                ->where('product_id', $product_id)
                ->where('vehicle_id', $vehicle_id)
                ->where('ref_number', $replacing_number)
                ->whereNotNull('ref_number')
                ->whereNotNull('position')
                ->whereNotNull('vehicle_id')
                ->whereHas('purchase', function($qr) {
                    $qr->where('bill_name','!=','Sale');
                })
                ->orderBy('id', 'desc') // Order by 'id' in descending order
                ->first(); // Get the first record from the ordered results

            if($query)
            $results = $query;
              
            if(!$query){
                // Check in vehicle tyres
                $vehicle_tyres = VehicleTyres::where('vehicle_id', $vehicle_id)->where('product_id', $product_id)->where('ref_number', $replacing_number)->where('fin_year',$request->current_year)->first();
                if($vehicle_tyres){
                    $results = $vehicle_tyres;
                }
            }

    
            return response()->json(['product_serials' => $results ]);
    }
    


    public function get_next_bill_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;
        $bill_name = $request->bill_name;

        $year_stamp = $this->get_short_name($request->current_year);

        if($bill_name=='Purchase'){
            $prefix = 'PUR';
        }
        elseif($bill_name=='Service'){
            $prefix = 'SER';
        }
        elseif($bill_name=='Stock Issue'){
            $prefix = 'SI';
        }
        elseif($bill_name=='Sale'){
            $prefix = 'SAL';
        }
        elseif($bill_name=='Stock Inwards'){
            $prefix = 'SIN';
        }
        

        if($type=='SYSTEM'){
            $max = Purchase::where('branch_id',$branch_id)->where('bill_type', $type)->where('bill_name',$bill_name)->where('created_by',$request->user->id)->max('max_number');
           
            // First 4 characters of the user;
            $user_tag = str_replace(' ', '', substr($request->user->username, 0, 4));
            $bill_number = $prefix.'-'.$branch_code.'-'.$user_tag.'-'.sprintf('%06d', $max+1);        

            return response()->json(['bill_number' => strtoupper($bill_number), 'max_number' => $max+1]);
        }
    }

    public function get_inventory_bill_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;

        $vehicle_id = isset($request->report_data['vehicle']) ? $request->report_data['vehicle']['id'] :null;
                                                                                                                 
        $query = Purchase::with(['branch:id,branch_name,branch_code', 
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name,unit,maintain_serial',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
            'voucher_desc'=>function($query){
            $query->where('against','Purchase')->where('voucher_name','Payment Voucher');
        }])
        ->where('fin_year', $request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['bill_num_from']) && isset($report_data['bill_num_to'])){
            $query->where('bill_number','>=',$report_data['bill_num_from']['bill_number'])->where('bill_number','<=',$report_data['bill_num_to']['bill_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('bill_date','>=',$report_data['from_date'])->where('bill_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['bill_name']) ){
            $query->where('bill_name', $report_data['bill_name']);
        }

        if (isset($report_data['product'])) {
            $query->whereHas('purchase_desc', function ($qr) use ($report_data) {
                $qr->where('product_id', $report_data['product']['id']);
            });
        }

        if (isset($report_data['vehicle'])) {
            $query->whereHas('purchase_desc', function ($qr) use ($report_data) {
                $qr->where('vehicle_id', $report_data['vehicle']['id']);
            });
        }
        
        


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', '%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_inventory_issue_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = PurchaseDesc::where('vehicle_id','!=', null)
                ->with(['product:id,product_name,unit,maintain_serial','vehicle:id,vehicle_number','purchase:id,bill_number,bill_date','product_serial_desc','product_serial_desc.product:id,product_name'])
                ->whereHas('purchase', function($qr) use ($report_data) {
                    $qr->whereIn('bill_name', ['Stock Issue','Purchase','Stock Inwards'])
                    ->when($report_data['from_date'], function($sub) use ($report_data) {
                        $sub->where('bill_date', '>=', $report_data['from_date'])
                            ->where('bill_date', '<=', $report_data['to_date']);
                    });
                })
                ->where('fin_year', $request->current_year);

        
        if (isset($report_data['product'])) {
            $query->where('product_id', $report_data['product']['id']);
        }
        
        if (isset($report_data['vehicle'])) {
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', '%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    public function get_stock_ledger(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
        $branch_id = isset($report_data['branch']) ? $report_data['branch']['id'] : null;

        $balance = InventoryBalances::where('product_id', $report_data['product']['id'])
            ->where('fin_year', $request->current_year)
            ->when($branch_id, function ($query) use ($branch_id) {
                $query->where('branch_id', $branch_id);
            }, function ($query) {
                $query->selectRaw('SUM(opening_bal) as total_balance');
            })
            ->first();
        
        
    

        $opening_balance = 0;
        if($balance){
            // Get the balance value
        $opening_balance = $branch_id ? $balance->opening_bal : $balance->total_balance;
        }       
    
        // Calculate opening balance
        $min_max_date = $this->get_min_max_date($request->current_year);
        $min_date = $min_max_date->min_date;
        $from_date = $report_data['from_date'];
        $currentYear = $request->current_year;
        $to_date = $report_data['to_date']; 
        $product_id = $report_data['product']['id'];


        if ($min_date !== $report_data['from_date']) {
            $purchase_sum_for_period = PurchaseDesc::where('product_id', $product_id)
                                ->where('vehicle_id', null)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date) {
                                    $qr->where('bill_date', '>=', $min_date)
                                    ->where('bill_date', '<', $from_date)
                                    ->where('bill_name', '!=', 'Stock Issue')
                                    ->where('bill_name', '!=', 'Sale')
                                    ->where('bill_name', '!=', 'Stock Inwards');
                                })
                                ->where('fin_year', $currentYear)
                                ->when($branch_id, function($query) use ($branch_id) {
                                    $query->whereHas('purchase', function($qr) use ($branch_id) {
                                        $qr->where('branch_id', $branch_id);
                                    });
                                })
                                ->sum('qty');
            
            $purchase_sum_for_STOCK_INWARDS = PurchaseDesc::where('product_id', $product_id)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date) {
                                    $qr->where('bill_date', '>=', $min_date)
                                    ->where('bill_date', '<', $from_date)
                                    ->where('bill_name', 'Stock Inwards');
                                })
                                ->where('fin_year', $currentYear)
                                ->when($branch_id, function($query) use ($branch_id) {
                                    $query->whereHas('purchase', function($qr) use ($branch_id) {
                                        $qr->where('branch_id', $branch_id);
                                    });
                                })
                                ->sum('qty');

            $issue_sum_for_period = PurchaseDesc::where('product_id', $product_id)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date) {
                                    $qr->where('bill_date', '>=', $min_date)->where('bill_date', '<', $from_date)->where('bill_name', 'Stock Issue');
                                })
                                ->where('fin_year', $currentYear)
                                ->when($branch_id, function($query) use ($branch_id) {
                                    $query->whereHas('purchase', function($qr) use ($branch_id) {
                                        $qr->where('branch_id', $branch_id);
                                    });
                                })
                                ->sum('qty');

            $sale_sum_for_period = PurchaseDesc::where('product_id', $product_id)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date) {
                                    $qr->where('bill_date', '>=', $min_date)->where('bill_date', '<', $from_date)->where('bill_name', 'Sale');
                                })
                                ->where('fin_year', $currentYear)
                                ->when($branch_id, function($query) use ($branch_id) {
                                    $query->whereHas('purchase', function($qr) use ($branch_id) {
                                        $qr->where('branch_id', $branch_id);
                                    });
                                })
                                ->sum('qty');

            $opening_balance = $opening_balance+$purchase_sum_for_period+$purchase_sum_for_STOCK_INWARDS-$issue_sum_for_period-$sale_sum_for_period;
        }

        if($opening_balance>=0){
            $opening_dr = $opening_balance;
            $opening_cr = 0;
        }
        else{
            $opening_dr = 0;
            $opening_cr = abs($opening_balance);
        }


                                                                                                                 
        $query = PurchaseDesc::with(['product:id,product_name,unit,maintain_serial','vehicle:id,vehicle_number','purchase:id,bill_number,bill_date,bill_name,acc_party_id','purchase.acc_party:id,account_name','product_serial_desc','product_serial_desc.product:id,product_name'])
                ->whereHas('purchase', function($qr) use ($report_data) {
                    $qr->when($report_data['from_date'], function($sub) use ($report_data) {
                        $sub->where('bill_date', '>=', $report_data['from_date'])
                            ->where('bill_date', '<=', $report_data['to_date']);
                    });
                })
                ->when($branch_id, function($query) use ($branch_id) {
                    $query->whereHas('purchase', function($qr) use ($branch_id) {
                        $qr->where('branch_id', $branch_id);
                    });
                })
                ->where('fin_year', $request->current_year);

        
        if (isset($report_data['product'])) {
            $query->where('product_id', $report_data['product']['id']);
        }
        
        if (isset($report_data['vehicle'])) {
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', '%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy('bill_number', $dir)->paginate($length);
        } else {
            $data = $query->orderBy('bill_number', $dir)->paginate( $query->count() );
        }

        
        
     
        $results = [];
        foreach ($data as $element) {
            $bill_name = '';
            $dr_qty = 0;
            $cr_qty = 0;

            if($element->purchase->bill_name == 'Stock Issue' || $element->purchase->bill_name == 'Sale'){
                $cr_qty = $element->qty;
            }

            if($element->purchase->bill_name == 'Purchase' || $element->purchase->bill_name == 'Stock Inwards'){
                $dr_qty = $element->qty;
            }

            

            $results[] = [
                'id' => $element->id,
                'bill_id' => $element->purchase->id,
                'bill_number' => $element->purchase->bill_number,
                'bill_date' => $element->purchase->bill_date,
                'vehicle' => $element->vehicle ? $element->vehicle->vehicle_number : "",
                'party' => $element->purchase->acc_party ? $element->purchase->acc_party->account_name : '',
                'bill_name' => $element->purchase->bill_name,
                'product' => $element->product->product_name,
                'dr_qty' => $dr_qty,
                'cr_qty' => $cr_qty,
                'bal_qty' => $dr_qty - $cr_qty,

            ];
        }

        // order results based on bill_date
        usort($results, function($a, $b) {
            return $a['bill_date'] <=> $b['bill_date'];
        });
          
        return response()->json(['data' => $results,'opening_dr'=>$opening_dr, 'opening_cr'=>$opening_cr, 'draw' => $request->draw]);
    }

    public function get_issue_history_report(Request $request){
        $report_data = $request->report_data;
             
        
        $vehicleTyre = VehicleTyres::where('vehicle_id', $report_data['vehicle']['id'])
                         ->with('product:id,product_name');
        
        if (isset($report_data['product'])) {
            $vehicleTyre->where('product_id', $report_data['product']['id']);
        }
        $vehicleTyres = $vehicleTyre->get();

        $issue_record = PurchaseDesc::where('vehicle_id','!=', null)
                ->with(['product:id,product_name,unit,maintain_serial','vehicle:id,vehicle_number','purchase:id,bill_number,bill_date','product_serial_desc','product_serial_desc.product:id,product_name'])
                ->whereHas('purchase', function($qr) use ($report_data) {
                    $qr->whereIn('bill_name', ['Stock Issue','Purchase','Stock Inwards']);
                });
        
        if (isset($report_data['product'])) {
            $issue_record->where('product_id', $report_data['product']['id']);
        }
        $issue_record->where('vehicle_id', $report_data['vehicle']['id']);   
        $data = $issue_record->orderBy('bill_id')->get();

          
        return response()->json(['issue_list' => $data, 'vehicle_opening'=>$vehicleTyres , 'draw' => $request->draw]);
    }

    public function get_issue_history_of_serial_report(Request $request){
        $report_data = $request->report_data;
             

        $vehicleTyre = VehicleTyres::with(['product:id,product_name,unit,maintain_serial','vehicle:id,vehicle_number']);
        
        if (isset($report_data['product'])) {
            $vehicleTyre->where('product_id', $report_data['product']['id']);
        }
        if (isset($report_data['serial_number'])) {
            $vehicleTyre->where('ref_number', $report_data['serial_number']['ref_number']);
        }
        $vehicleTyres = $vehicleTyre->get();


        $issue_record = ProductSerial::with(['product:id,product_name,unit,maintain_serial','vehicle:id,vehicle_number','purchase:id,bill_number,bill_name,bill_date','bill_desc:id,bill_number,bill_id,reading']);
        
        if (isset($report_data['product'])) {
            $issue_record->where('product_id', $report_data['product']['id']);
        }

        if (isset($report_data['serial_number'])) {
            $issue_record->where(function($qr) use ($report_data){
                $qr->where('ref_number', $report_data['serial_number']['ref_number'])->orWhere('replacing_number', $report_data['serial_number']['ref_number']);
            });
        }



        $data = $issue_record->orderBy('bill_id','asc')->get();

          
        return response()->json(['issue_list' => $data, 'vehicle_opening'=>$vehicleTyres]);
    }


    public function get_tyre_report(Request $request){
        // I want these columns for each distinct tyre number:-  id,product,ref_number,status,km_life,km_run,remaining_km,remaining_percent,
        // Keeping in mind the opening reading added in VehicleTyres.reading, or PurchaseDesc.reading, and calculate the run km through in and out reading in purchase_desc

        $report_data = $request->report_data;
        $filter_vehicle_id = $report_data['vehicle'] ? $report_data['vehicle']['id'] : null;
        $product_id = $report_data['product'] ? $report_data['product']['id'] : null;
        $serial_number = $report_data['serial_number'] ? $report_data['serial_number']['ref_number'] : null;
        $status_filter = $report_data['status'] ?? null;
        $health = $report_data['health'];

        $not_zero = $report_data['not_zero'];

        $searchValue = $request->search;


        // GET DISTINCT TYRE NUMBERS
        $distinct_tyre_number = ProductSerial::select('ref_number')
                ->where('product_id', $product_id)
                ->whereNotNull('ref_number')
                ->when($serial_number, function($query) use ($serial_number) {
                    $query->where('ref_number', $serial_number);
                })
                ->distinct()
                ->orderBy('ref_number', 'desc')
                ->pluck('ref_number'); // Use pluck to get an array of values

        $distinct_opening_tyres = VehicleTyres::select('ref_number')
                ->where('product_id', $product_id)
                ->whereNotNull('ref_number')
                ->when($serial_number, function($query) use ($serial_number) {
                    $query->where('ref_number', $serial_number);
                })
                ->distinct()
                ->orderBy('ref_number', 'desc')
                ->pluck('ref_number'); // Use pluck to get an array of values

        

        // Merge the arrays and get unique values
        $combined_distinct_tyres = $distinct_tyre_number->merge($distinct_opening_tyres)->unique();

        
        $results = [];

        foreach ($combined_distinct_tyres as $tyre_number) {
            $opening_reading = 0;
            $life_km = 0;

            $vehicleTyre = VehicleTyres::with(['product:id,product_name','vehicle:id,vehicle_number'])
                ->where('ref_number', $tyre_number)
                ->when($product_id, function($query) use ($product_id) {
                    $query->where('product_id', $product_id);
                })
                ->first();

            $product_serial = ProductSerial::with(['product:id,product_name','vehicle:id,vehicle_number','bill_desc:id,bill_id,bill_number,reading'])
                ->where(function($qr) use ($tyre_number){
                    $qr->where('ref_number', $tyre_number)->orWhere('replacing_number', $tyre_number);
                })
                ->when($product_id, function($query) use ($product_id) {
                    $query->where('product_id', $product_id);
                })
                ->whereHas('purchase',function($query) {
                    $query->where('bill_name','Purchase');
                })
                ->first();
            
            if($vehicleTyre){
                $opening_reading = $vehicleTyre->reading;
                $life_km = $vehicleTyre->life_km;
            }
            elseif($product_serial && !$vehicleTyre){
                $opening_reading = $product_serial->bill_desc->reading;
                $life_km = $product_serial->life_km;
            }

           

            $all_product_serial = ProductSerial::with(['product:id,product_name','vehicle:id,vehicle_number','bill_desc:id,bill_id,bill_number,reading','purchase:id,bill_number,bill_name,bill_date'])
                ->where(function($qr) use ($tyre_number){
                    $qr->where('ref_number', $tyre_number)->orWhere('replacing_number', $tyre_number);
                })
                ->when($product_id, function($query) use ($product_id) {
                    $query->where('product_id', $product_id);
                })
                ->orderBy('bill_id','asc')
                ->get();

            // Initialize values
            

            $km_run = 0;
            $last_reading = 0;

            $entry_date = '';
            $exit_date = '';

            $last_reading = $opening_reading;
            $stock_issue_reading = $opening_reading;
            $current_vehicle_id = null;

            foreach ($all_product_serial as $serial) {
                $bill_name = $serial->purchase->bill_name;
                $reading = $serial->bill_desc->reading;

                $vehicle_id = $serial->vehicle_id; // Track current vehicle
                
            
                // Handling stock inwards or sale (tire removed from vehicle)
                if (($bill_name == 'Stock Inwards' || $bill_name == 'Sale') && !$vehicle_id && $stock_issue_reading > 0) {
                    // Calculate km_run when tire is removed from the vehicle
                    $km_run += $reading - $stock_issue_reading;
                    $stock_issue_reading = 0; // Reset after processing
            
                    // Track exit date and clear vehicle information
                    $exit_date = $serial->purchase->bill_date;
                    $current_vehicle_id = null;
                } 
                elseif (($bill_name == 'Stock Inwards' || $bill_name == 'Sale') && $vehicle_id) {
                    // Tire is newly applied to a vehicle
                    $stock_issue_reading = $reading;
            
                    // Track the entry date when tire is attached to a vehicle
                    $entry_date = $serial->purchase->bill_date;
                    $exit_date = null;
                    $current_vehicle_id = $vehicle_id;
                } 
                elseif ($bill_name == 'Stock Issue') {
                    // Handle cases when the tire is replaced or reissued
                    if ($serial->replacing_number == $tyre_number) {
                        // Replaced tire, stop km recording
                        $km_run += $reading - $stock_issue_reading;
                        $stock_issue_reading = 0;
            
                        // Track the exit date and clear vehicle information
                        $exit_date = $serial->purchase->bill_date;
                        $current_vehicle_id = null;
                    } else {
                        // New issuance, start tracking again
                        $stock_issue_reading = $reading;
            
                        // Track the entry date when tire is attached to a vehicle
                        $entry_date = $serial->purchase->bill_date;
                        $exit_date = null;
                        $current_vehicle_id = $vehicle_id;

                        
                    }
                } 
                elseif ($bill_name == 'Purchase') {
                    // Handle tire purchases
                    if ($serial->replacing_number == $tyre_number) {
                        // Replaced tire, stop km recording
                        $km_run += $reading - $stock_issue_reading;
                        $stock_issue_reading = 0;
            
                        // Track the exit date and clear vehicle information
                        $exit_date = $serial->purchase->bill_date;
                        $current_vehicle_id = null;
                    } else {
                        // Ignore purchase for km_run calculation, but track the reading
                        $last_reading = $reading;
                        $stock_issue_reading = $reading;
            
                        // Track the entry date when tire is attached to a vehicle
                        $entry_date = $serial->purchase->bill_date;
                        $exit_date = null;
                        $current_vehicle_id = $vehicle_id;

                       
                    }
                }
                

                
            }

            // If there is no exit date after the last issuance, continue updating km_run using DieselReceipts
           
            if ($current_vehicle_id && $entry_date && !isset($exit_date)) {

                // Fetch DieselReceipts after the last entry_date
                $dieselReceipts = DieselReceipt::where('vehicle_id', $current_vehicle_id)
                    ->where('booking_date', '>=', $entry_date)
                    ->orderBy('booking_date', 'asc')
                    ->get();
        
                foreach ($dieselReceipts as $receipt) {
                    // Only consider readings greater than the last recorded reading
                  
                    if ($receipt->reading > $stock_issue_reading) {
                       
                        $km_run += $receipt->reading - $stock_issue_reading;
                        $stock_issue_reading = $receipt->reading;
                    }
                }
            }
            
            // Handle the case when the tire is removed (final exit event) if needed
            if ($current_vehicle_id && isset($exit_date) && $reading > $stock_issue_reading) {
               
                // Calculate km_run between the last stock issue reading and the final exit reading
                $km_run += $reading - $stock_issue_reading;
                $stock_issue_reading = 0;
            }
            
            

            $remaining_km = $life_km - $km_run;
            $remaining_percent = ($life_km !=0 && $remaining_km!=0 ) ? ($remaining_km / $life_km) * 100 : 0;

            $vehicle_number = '';
            $status = '';
            // To find status In Stock, Assigned, Not in Stock
            $last_existence_in_bills = ProductSerial::with(['product:id,product_name','vehicle:id,vehicle_number','bill_desc:id,bill_id,bill_number,reading,vehicle_id','purchase:id,bill_number,bill_name'])
                            ->where(function($qr) use ($tyre_number){
                                $qr->where('ref_number', $tyre_number)->orWhere('replacing_number', $tyre_number);
                            })
                            ->when($product_id, function($query) use ($product_id) {
                                $query->where('product_id', $product_id);
                            })
                            ->orderBy('bill_number','desc')
                            ->first();

            

            if(!$last_existence_in_bills){
                $last_existence_in_vehicle_tyres = VehicleTyres::with(['product:id,product_name','vehicle:id,vehicle_number'])
                                            ->where('ref_number', $tyre_number)
                                            ->when($product_id, function($query) use ($product_id) {
                                                $query->where('product_id', $product_id);
                                            })
                                            ->orderBy('id','desc')
                                            ->first();

                                           

                if($last_existence_in_vehicle_tyres){
                    $status = 'Assigned';
                    $vehicle_number = $last_existence_in_vehicle_tyres->vehicle ? $last_existence_in_vehicle_tyres->vehicle['vehicle_number'] : '';
                    $last_existence_in_bills = $last_existence_in_vehicle_tyres;
                   
                }
                
            }
            else{


                $purchase = $last_existence_in_bills->purchase;

                if($purchase->bill_name=='Sale' || ($purchase->bill_name=='Stock Issue' && !$last_existence_in_bills->vehicle_id)){
                    $status = 'Not in Stock';
                }

                if($last_existence_in_bills->vehicle_id){
                    $status = 'Assigned';
                    $vehicle_number = $last_existence_in_bills->vehicle ? $last_existence_in_bills->vehicle['vehicle_number'] : '';
                }

                if($purchase->bill_name!='Sale' && $purchase->bill_name!='Stock Issue' && !$last_existence_in_bills->vehicle_id){
                    $status = 'In Stock';
                    $vehicle_number = '';
                }

                if($last_existence_in_bills->replacing_number == $tyre_number){
                    $status = 'In Stock';
                    $vehicle_number = '';
                }

            }

            
            

            $add = true;
            if($status_filter){
                if($status_filter == $status)
                $add = true;
                else
                $add = false;
            }

            if($filter_vehicle_id){
                if($filter_vehicle_id == $last_existence_in_bills->vehicle_id)
                $add = true;
                else
                $add = false;
            }

            if($health){
                if($remaining_percent <= $health)
                $add = true;
                else
                $add = false;
            }

            if($not_zero){
                if($remaining_percent!=0){
                    $add=true;
                }
                else{
                    $add=false;
                }
            }

        

            if($add)
            $results[] = [
                'product_id' => $product_id,
                'product' => $report_data['product']['product_name'],
                'serial_number' => $tyre_number,
                'status' => $status,
                'opening_reading' => $opening_reading,
                'km_life' => $life_km,
                'km_run' => $km_run,
                'remaining_km' => $remaining_km,
                'remaining_percent' => $remaining_percent,
                'vehicle_number' => $vehicle_number,
            ];

        }

        return response()->json(['tyre_list' => $results]);

    }

    
    
    
    


    protected function check_for_balance($against,$default_balance,$selection){
            $filteredVoucherDesc = $against->voucher_desc->where('against', $selection);
            
            if ($filteredVoucherDesc->isEmpty()) {
                // No voucher_desc, return grand_total_amount as balance
                return $default_balance;
            }
            
            $recievedAmtSum = $filteredVoucherDesc->sum('receipt_amt');
            $deductionSum = $filteredVoucherDesc->sum('deduction');
            
            $balance = $default_balance - $recievedAmtSum - $deductionSum ;
            return $balance;
    }





    protected function get_data_for_obj($pk){
        $bill = Purchase::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name,unit,maintain_serial',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.dr_account:id,account_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
        ])->find($pk);


        return ['bills'=>$bill];
    }

    protected function get_max_number_from_bill_number($input){
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function calculate_balance_for_inventory($request){
        $pk = $request['product_id'];
        $branch_id = $request['branch_id'];
        $current_year = $request['current_year'];

        $min_max_date = $this->get_min_max_date($current_year);
        $min_date = $min_max_date->min_date;
        $from_date = $min_date;

        // Today's date
        $to_date = Carbon::now()->toDateString();


        $balance_details = $this->calculate_balance_for_inventory_stock($pk,$min_date,$from_date,$to_date,$current_year,$branch_id);

        return ['balance' => $balance_details];
    }

    


    protected function record_transaction_in_accounts($request, $bill){
        $t_data = [];
        $t_data['t_against'] = 'Purchase Bills';
        $t_data['t_against_id'] = $bill->id;
        $t_data['t_against_no'] = $bill->bill_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->bill_date;
        $t_data['voucher_time'] = $request->bill_time;
        $t_data['t_against_narration'] = $bill->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];


        if($bill->bill_name=='Purchase' || $bill->bill_name=='Service'){
            $purchaseDescQuery = PurchaseDesc::where('bill_id', $bill->id);

            $bill_desc = $purchaseDescQuery->get();

            $sums = $purchaseDescQuery->selectRaw('SUM(cgst_amount) as total_cgst, SUM(sgst_amount) as total_sgst, SUM(igst_amount) as total_igst, SUM(amount) as total_amount')
                ->first();

            
            $total_cgst = $sums->total_cgst ?? 0;
            $total_sgst = $sums->total_sgst ?? 0;
            $total_igst = $sums->total_igst ?? 0;
            $total_amount = $sums->total_amount ?? 0;

            
            $t_data['cancellable'] = 1;
            
            if($bill->acc_party_id && $total_amount>0 ){
                foreach ($bill_desc as $key) {
                    // DEBIT ALL THE ACCOUNTS IN DESC
                    if($key['dr_account_id'] && $key['amount']){
                        $t_data['voucher_accounting_details'][] = ['account_id'=>$key['dr_account_id'], 'nature'=>'dr' , 'dr_amount'=>$key['amount'], 'cr_amount'=>0 ];
                    }
                }

                if($bill->rounded_off > 0){ 
                    $short_and_excess = $this->get_acc_id('SHORT AND EXCESS');
                    if(!$short_and_excess){
                        dd("SHORT AND EXCESS ACCOUNT NOT FOUND");
                    }
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$short_and_excess, 'nature'=>'dr' , 'dr_amount'=>abs($bill->rounded_off), 'cr_amount'=>0 ];
                }

                if($bill->rounded_off < 0){ 
                    $short_and_excess = $this->get_acc_id('SHORT AND EXCESS');
                    if(!$short_and_excess){
                        dd("SHORT AND EXCESS ACCOUNT NOT FOUND");
                    }
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$short_and_excess, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>abs($bill->rounded_off) ];
                }


                // CREDIT PARTY AND GST
                // CREDIT SIDE
                if($total_cgst > 0){ 
                    // CREDIT CGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>9, 'nature'=>'dr' , 'dr_amount'=>$total_cgst, 'cr_amount'=>0 ];
                }
                if($total_sgst > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>10, 'nature'=>'dr' , 'dr_amount'=>$total_sgst, 'cr_amount'=>0 ];
                }
                if($total_igst > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>11, 'nature'=>'dr' , 'dr_amount'=>$total_igst, 'cr_amount'=>0 ];
                }

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$bill->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$bill->grand_total_amount ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }


                // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);

                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }


                


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 

                

                $this->create_transaction_voucher( $t_data );
            }
        }
        else if($bill->bill_name=='Sale'){
            $purchaseDescQuery = PurchaseDesc::where('bill_id', $bill->id);

            $bill_desc = $purchaseDescQuery->get();

            $sums = $purchaseDescQuery->selectRaw('SUM(cgst_amount) as total_cgst, SUM(sgst_amount) as total_sgst, SUM(igst_amount) as total_igst, SUM(amount) as total_amount')
                ->first();

            
            $total_cgst = $sums->total_cgst ?? 0;
            $total_sgst = $sums->total_sgst ?? 0;
            $total_igst = $sums->total_igst ?? 0;
            $total_amount = $sums->total_amount ?? 0;

            
            $t_data['cancellable'] = 1;
            
            if($bill->acc_party_id && $total_amount>0 ){
                foreach ($bill_desc as $key) {
                    // DEBIT ALL THE ACCOUNTS IN DESC
                    if($key['dr_account_id'] && $key['amount']){
                        $t_data['voucher_accounting_details'][] = ['account_id'=>$key['dr_account_id'], 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$key['amount'] ];
                    }
                }

                if($bill->rounded_off > 0){ 
                    $short_and_excess = $this->get_acc_id('SHORT AND EXCESS');
                    if(!$short_and_excess){
                        dd("SHORT AND EXCESS ACCOUNT NOT FOUND");
                    }
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$short_and_excess, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>abs($bill->rounded_off) ];

                }

                if($bill->rounded_off < 0){ 
                    $short_and_excess = $this->get_acc_id('SHORT AND EXCESS');
                    if(!$short_and_excess){
                        dd("SHORT AND EXCESS ACCOUNT NOT FOUND");
                    }
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$short_and_excess, 'nature'=>'dr' , 'dr_amount'=>abs($bill->rounded_off), 'cr_amount'=>0 ];
                }


                // CREDIT PARTY AND GST
                // CREDIT SIDE
                if($total_cgst > 0){ 
                    // CREDIT CGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$total_cgst ];
                }
                if($total_sgst > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$total_sgst ];
                }
                if($total_igst > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$total_igst ];
                }

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$bill->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$bill->grand_total_amount, 'cr_amount'=>0 ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }


                // Round both $total_dr and $total_cr to two decimal places
                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);

                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                    // Output the values for debugging
                    dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }


                


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 

                

                $this->create_transaction_voucher( $t_data );
            }
        }
        elseif($bill->bill_name=='Stock Issue'){
        }

        

    }

    
    
}

