<?php

namespace App\Http\Controllers\Master;

use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\MasterNarration;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class MasterNarrationController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request){
        
        // Fetch all the Narration Lists with the relationships on acc_party, source, destination, consignor, consignee
        $narration_list = MasterNarration::with([
            'vehicle:id,vehicle_number,vehicle_type',
            'party:id,party_name',
            'driver:id,emp_name',
            'narration_by:id,username',
        ])
        ->when($request->against, function ($query) use ($request) {
            $query->where('against', $request->against);
        })
        ->when($request->vehicle, function ($query) use ($request) {
            $query->where('vehicle_id', $request->vehicle['id']);
        })
        ->when($request->party, function ($query) use ($request) {
            $query->where('party_id', $request->party['id']);
        })
        ->when($request->driver, function ($query) use ($request) {
            $query->where('emp_id', $request->driver['id']);
        })
        ->when($request->user, function ($query) use ($request) {
            $query->where('narration_by_id', $request->user['id']);
        })
        ->when(isset($request->mn_id), function ($query) use ($request) {
            $query->where('id', $request->mn_id);
        })
        ->get();
        

        return response()->json(['narration_list' => $narration_list]);

    }

    public function show(Request $request, $pk){
            $narration_list = MasterNarration::with([
                'vehicle:id,vehicle_number,vehicle_type',
                'party:id,party_name',
                'driver:id,emp_name',
                'narration_by:id,username',
            ])->find($pk);

            
            return response()->json(['narration_list' => $narration_list]);
    }

    public function store(Request $request){
        

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  


            // Loop through $request->narration_lists and create new MasterNarration for each of them.
            foreach ($request->narration_list as $index => $pl) {
                if( !$pl['against'] ||  !$pl['date'] || !$pl['time'] || !$pl['remarks'])
                continue;
                 
                $veh_lis = new MasterNarration();
                $veh_lis->against = $pl['against'];
                $veh_lis->vehicle_id = $pl['vehicle'] ?  $pl['vehicle']['id'] : null;
                $veh_lis->emp_id = $pl['driver'] ?  $pl['driver']['id'] : null;
                $veh_lis->party_id = $pl['party'] ?  $pl['party']['id'] : null;
                

                $veh_lis->date = $pl['date'];
                $veh_lis->time = $pl['time'];
                $veh_lis->remarks = $pl['remarks'];
                $veh_lis->alert_date = $pl['alert_date'];
                $veh_lis->alert_on = $pl['alert_on'];
                $veh_lis->narration_by_id = $request->user->id;
                
                $veh_lis->save();

                $newVal = $this->get_data_for_obj($veh_lis->id);
                $this->createEditLog('created', 'narration_lists', $veh_lis->id, 'MASTER NARRATION'  , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }
            
            DB::commit();
            
            return response()->json(['success' => 'Narration List Created Successfully']);

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['narration_list' => [$e->getMessage()]]], 422);
        }    



    }

    public function update(Request $request){

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            // Loop through $request->narration_lists and create new MasterNarration for each of them.
            foreach ($request->newly_added_narration_list as $index => $napl) {
                if( !$napl['against'] || !$napl['date'] || !$napl['time'] || !$napl['remarks'] )
                continue;


                $veh_lis = new MasterNarration();
                $veh_lis->against = $napl['against'];
                $veh_lis->vehicle_id = $napl['vehicle'] ?  $napl['vehicle']['id'] : null;
                $veh_lis->emp_id = $napl['driver'] ?  $napl['driver']['id'] : null;
                $veh_lis->party_id = $napl['party'] ?  $napl['party']['id'] : null;

                $veh_lis->date = $napl['date'];
                $veh_lis->time = $napl['time'];
                $veh_lis->remarks = $napl['remarks'];
                $veh_lis->alert_date = $napl['alert_date'];
                $veh_lis->alert_on = $napl['alert_on'];
                $veh_lis->narration_by_id = $request->user->id;

                $veh_lis->save();

                $newVal = $this->get_data_for_obj($veh_lis->id);
                $this->createEditLog('created', 'narration_lists', $veh_lis->id, 'MASTER NARRATION' , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }
             
            // Loop through $request->updated_narration_list and update MasterNarration for each of them.
            foreach ($request->updated_narration_list as $index => $upl) {
                if( !$upl['against'] || !$upl['date'] || !$upl['time'] || !$upl['remarks'] )
                continue;

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($upl['id']);

                $veh_lis = MasterNarration::find($upl['id']);
                
                $veh_lis->date = $napl['date'];
                $veh_lis->time = $napl['time'];
                $veh_lis->remarks = $napl['remarks'];
                $veh_lis->alert_date = $napl['alert_date'];
                $veh_lis->alert_on = $napl['alert_on'];
                $veh_lis->save();

                // Save Edit Log
                $newVal = $this->get_data_for_obj($veh_lis->id);
                $this->createEditLog('updated', 'narration_lists', $veh_lis->id, 'MASTER NARRATION', $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            // Loop through $request->deleted_narration_list and delete MasterNarration for each of them.
            foreach ($request->deleted_narration_list as $dpl) {
                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);

                
                $veh_lis = MasterNarration::find($dpl);
                $veh_lis->delete();

                // Save Edit Log
                $this->createEditLog('deleted', 'narration_lists', $veh_lis->id, 'MASTER NARRATION', $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();

            return response()->json(['success' => 'Narration List Updated Successfully']);
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['narration_list' => [$e->getMessage()]]], 422);
        }
    
    }

    public function destroy(Request $request){
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            // Loop through $request->deleted_narration_list and delete MasterNarration for each of them.
            foreach ($request->deleted_narration_list as $dpl) {

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);

                $veh_lis = MasterNarration::find($dpl);
                $veh_lis->delete();

                // Save Edit Log
                $this->createEditLog('deleted', 'narration_lists', $veh_lis->id, 'MASTER NARRATION' , $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();
            return response()->json(['success' => 'Narration List Deleted Successfully']);
            
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['narration_list' => [$e->getMessage()]]], 422);
        }
    }

    

    public function get_narration_list_report(Request $request){
        $report_data = $request->report_data;
        $narration_list = MasterNarration::with([
            'vehicle:id,vehicle_number,vehicle_type',
            'party:id,party_name',
            'driver:id,emp_name',
            'narration_by:id,username',
        ])
        ->when($request->against, function ($query) use ($request) {
            $query->where('against', $request->against);
        })
        ->when($request->vehicle, function ($query) use ($request) {
            $query->where('vehicle_id', $request->vehicle['id']);
        })
        ->when($request->party, function ($query) use ($request) {
            $query->where('party_id', $request->party['id']);
        })
        ->when($request->driver, function ($query) use ($request) {
            $query->where('emp_id', $request->driver['id']);
        })
        ->when($request->user, function ($query) use ($request) {
            $query->where('narration_by_id', $request->user['id']);
        })
        ->when(isset($request->mn_id), function ($query) use ($request) {
            $query->where('id', $request->mn_id);
        })
        ->get();
        

        return response()->json(['data' => $narration_list]);
    }

    protected function get_data_for_obj($pk){
        $narration_list = MasterNarration::with([
            'vehicle:id,vehicle_number,vehicle_type',
            'party:id,party_name',
            'driver:id,emp_name',
            'narration_by:id,username',
        ])->find($pk);


            
        return ['narration_list' => $narration_list];
    }



}
