<?php

namespace App\Http\Controllers\Settings;

use App\Http\Controllers\Controller;
use App\Models\Master\Unit;
use App\Models\Master\VehicleType;
use App\Models\Settings\CalculationStrings;
use App\Models\Settings\DynamicFields;
use App\Models\Settings\GeneralSettings;
use Illuminate\Http\Request;

class SettingsController extends Controller
{
    public function fetch_for_select(Request $request)
    {
        // Fetch and send all units
        $units = Unit::select('id','unit_symbol')->get();


        return response()->json(['units'=>$units]);
    }

    public function get_dynamic_fields(Request $request){
        if($request->user->power < 2){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $dynamic_fields = DynamicFields::select('id','name','dynamic_fields')->get();
        
        $data = [];
        //For each dynamic field, decode the json and add the id to the json
        foreach($dynamic_fields as $item){
            $data[ $item['name'] ] = $item;
        }

        return response()->json($data);
    }

    public function get_single_dynamic_fields(Request $request, $name){   
        $dynamic_field = DynamicFields::select('id','name','dynamic_fields')->where('name', $name)->first();
        return response()->json(['dynamic_fields' => $dynamic_field]);
    }

    public function update_dynamic_fields(Request $request, $name){  
        if($request->user->power < 2){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $dynamic_field = DynamicFields::select('id','name','dynamic_fields')->where('name', $name)->first();

        if(!$dynamic_field){
            $dynamic_field = new DynamicFields();
            $dynamic_field->name = $name;
        }

        $dynamic_field->dynamic_fields = $request->dynamic_fields;
        $dynamic_field->save();
        return response()->json("Success");
    }



    public function get_calculation_strings(Request $request){
        if($request->user->power < 2){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $calculation_strings = CalculationStrings::select('id','name','string')->get();
        
        $data = [];
        //For each dynamic field, decode the json and add the id to the json
        foreach($calculation_strings as $item){
            $data[ $item['name'] ] = $item;
        }

        return response()->json($data);
    }

    public function get_single_calculation_strings(Request $request, $name){   
        $calculation_string = CalculationStrings::select('id','name','string')->where('name', $name)->first();
        return response()->json(['calculation_string' => $calculation_string]);
    }

    public function update_calculation_strings(Request $request, $name){  
        if($request->user->power < 2){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $calculation_string = CalculationStrings::select('id','name','string')->where('name', $name)->first();

        if(!$calculation_string){
            $calculation_string = new CalculationStrings();
            $calculation_string->name = $name;
        }

        $calculation_string->string = $request->string;
        $calculation_string->save();
        return response()->json("Success");
    }


    public function get_general_settings(Request $request){
        $general_settings =  GeneralSettings::select('id','setting_name','value')->get();
        
        $data = [];
        //For each general_settings
        foreach($general_settings as $item){
            $data[ $item['setting_name'] ] = $item;
        }

        return response()->json($data);
    }

    public function get_single_general_setting(Request $request, $name){   
        $general_setting = GeneralSettings::select('id','setting_name','value')->where('setting_name', $name)->first();
        return response()->json(['general_setting' => $general_setting]);
    }

    public function update_general_settings(Request $request, $name){
        if($request->user->power < 2){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $general_settings = GeneralSettings::select('id','setting_name','value')->where('setting_name', $name)->first();

        if(!$general_settings){
            $general_settings = new GeneralSettings();
            $general_settings->setting_name = $name;
        }
        
        $general_settings->value = $request->value;
        $general_settings->save();

        return response()->json("Success");
    }
}
