<?php

namespace App\Http\Controllers\Z_Core;

use App\Http\Controllers\Controller;
use App\Models\Pod\Pod;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class DocumentController extends Controller
{
    public function upload(Request $request)
    {
        // Get the client ID or username
        $clientId = $request->company['company_code'];

        // Get the uploaded file
        $file = $request->file('file');

        // Generate a unique filename
        $originalFilename = $file->getClientOriginalName();
        $uniqueFilename = time() . '_' . $originalFilename;

        // Create a directory for the client if it doesn't exist
        $directory = 'clients/' . $clientId . '/documents/'. $request->dir ;
        Storage::disk('public')->makeDirectory($directory);

        // Move the uploaded file to the client's directory
        $file->storeAs($directory, $uniqueFilename, 'public');

        // Save the file path in the database or perform any other required actions
        $filePath = 'storage/'.$directory . '/' . $uniqueFilename;
        // Save $filePath in the database or perform other operations
        
        // Return a response indicating successful upload
        return response()->json(['path' => $filePath]);

    }

    public function upload_pods(Request $request)
    {
        // Validate the request to ensure it contains files
        $request->validate([
            'files.*' => 'required|file', // Validate each file
            'company.company_code' => 'required|string', // Validate company code
        ]);

        // Get the client ID or username
        $clientId = $request->company['company_code'];

        // Create a directory for PODs
        $directory = 'clients/' . $clientId . '/pods';
        Storage::disk('public')->makeDirectory($directory);

        // Initialize an array to store the response for each file
        $responses = [];

        // Loop through each file and upload it
        foreach ($request->file('files') as $file) {
            // Generate a unique filename
            $originalFilename = $file->getClientOriginalName();
            $uniqueFilename = $originalFilename;


            $success = true;
            // BREAK FILE NAME WITHHOUT EXTENSION AND CHECK IF IT MATCHES TO ANY POD IN THE DATABASE
            $file_name = pathinfo($originalFilename, PATHINFO_FILENAME);
            $pod = Pod::where('pod_number', $file_name)->first();
            if(!$pod){
                $success = false;
                $filePath = null;
            }
            else{
                // Store the file in the directory
                $file->storeAs($directory, $uniqueFilename, 'public');
    
                // Save the file path in the response array
                $filePath = 'storage/' . $directory . '/' . $uniqueFilename;

                $pod->pod_pic = $filePath;
                $pod->save();
            }

            
            $responses[] = [
                'originalName' => $originalFilename,
                'uniqueName' => $uniqueFilename,
                'path' => $filePath,
                'success' => $success,
            ];
        }

        // Return a response with all file paths and statuses
        return response()->json($responses);
    }
}
