@php
    function formatDate($date) {
        // Check if the date is valid
        if (!strtotime($date)) {
            return '';
        }
        
        return date('d-m-Y', strtotime($date));
    }

    
    function printTillSpecificLength($string, $maxLength) {
        // Check if the length of the string is less than or equal to the maxLength
        if (strlen($string) <= $maxLength) {
            echo $string;
        } else {
            // If the length exceeds maxLength, print only up to maxLength characters
            echo substr($string, 0, $maxLength);
        }
    }

    function numberToWords($number) {
        $no = floor($number);
        $point = round($number - $no, 2) * 100;
        $hundred = null;
        $digits_1 = strlen($no);
        $i = 0;
        $str = array();
        $words = array(
            '0' => '', '1' => 'One', '2' => 'Two', '3' => 'Three', '4' => 'Four', '5' => 'Five', '6' => 'Six', '7' => 'Seven', '8' => 'Eight', '9' => 'Nine',
            '10' => 'Ten', '11' => 'Eleven', '12' => 'Twelve', '13' => 'Thirteen', '14' => 'Fourteen', '15' => 'Fifteen', '16' => 'Sixteen', '17' => 'Seventeen',
            '18' => 'Eighteen', '19' => 'Nineteen', '20' => 'Twenty', '30' => 'Thirty', '40' => 'Forty', '50' => 'Fifty', '60' => 'Sixty', '70' => 'Seventy',
            '80' => 'Eighty', '90' => 'Ninety'
        );
        $digits = array('', 'Hundred', 'Thousand', 'Lakh', 'Crore');
        while ($i < $digits_1) {
            $divider = ($i == 2) ? 10 : 100;
            $number = floor($no % $divider);
            $no = floor($no / $divider);
            $i += ($divider == 10) ? 1 : 2;
            if ($number) {
                $plural = (($counter = count($str)) && $number > 9) ? 's' : null;
                $hundred = ($counter == 1 && $str[0]) ? ' and ' : null;
                $str [] = ($number < 21) ? $words[$number] .
                    " " . $digits[$counter] . $plural . " " . $hundred
                    :
                    $words[floor($number / 10) * 10]
                    . " " . $words[$number % 10] . " "
                    . $digits[$counter] . $plural . " " . $hundred;
            } else $str[] = null;
        }
        $str = array_reverse($str);
        $result = implode('', $str);
        $points = ($point) ?
            "." . $words[$point / 10] . " " . 
                $words[$point = $point % 10] : '';
                echo $result . "Rupees  ";
        // echo $result . "Rupees  " . $points . " Paise";
    }

    function getMonthName($monthNumber) {
        $months = [
            1 => 'January',
            2 => 'February',
            3 => 'March',
            4 => 'April',
            5 => 'May',
            6 => 'June',
            7 => 'July',
            8 => 'August',
            9 => 'September',
            10 => 'October',
            11 => 'November',
            12 => 'December',
        ];

        if ($monthNumber < 1 || $monthNumber > 12) {
            return '';
        }

        return $months[$monthNumber];
    }

    // if(isset($_GET['copies']))
    // $copies = $_GET['copies'];
    // else
    // $copies = 'Original,Duplicate';
    

    // // BREAK $copies INTO ARRAY
    // $copies = explode(',', $copies);





@endphp



<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Employee Salary Details</title>
    <style>
        body {
            font-family: Arial, sans-serif;
            margin: 20px;
            background-color: #f9f9f9;
            color: #333;
        }
        .container {
            width: 100%;
            max-width: 1100px;
            margin: auto;
            padding: 20px;
            background: white;
            border-radius: 10px;
            box-shadow: 0 2px 10px rgba(0, 0, 0, 0.1);
        }
        .header-section {
            margin-bottom: 20px;
        }
        .header-section h1 {
            text-align: center;
            color: #4CAF50;
            margin-bottom: 10px;
        }
        .header-section p {
            font-size: 14px;
            text-align: center;
            color: #555;
        }
        .details, .description {
            width: 100%;
            border-collapse: collapse;
            margin: 20px 0;
        }
        th, td {
            border: 1px solid #ddd;
            text-align: left;
            padding: 10px;
        }
        th {
            background-color: #4CAF50;
            color: white;
        }
        tr:nth-child(even) {
            background-color: #f2f2f2;
        }
    </style>
</head>
<body>

    <div class="container">
        <!-- Header Section -->
        <div class="header-section">
            <h1>Employee Salary Details</h1>
            <p>Summary of employee salaries for the given month</p>
        </div>

        <!-- Heading Details -->
        <table class="details">
            <thead>
                <tr>
                    <th>SS Number *</th>
                    <th>Month *</th>
                    <th>Date *</th>
                    <th>Branch</th>
                    <th>Category</th>
                </tr>
            </thead>
            <tbody>
                <tr>
                    <td>{{ $salary->salary_number }}</td>
                    <td>{{ getMonthName($salary->month) }}</td>
                    <td>{{ formatDate($salary->date) }}</td>
                    <td>{{$salary->branch->branch_name}}</td>
                    <td>{{$salary->category}}</td>
                </tr>
            </tbody>
        </table>

        <!-- Looped Description -->
        <table class="description">
            <thead>
                <tr>
                    <th>#</th>
                    <th>Name</th>
                    <th>Working Days</th>
                    <th>Absent</th>
                    <th>Present</th>
                    <th>Salary On</th>
                    <th>Basic</th>
                    <th>SPL Allowance</th>
                    <th>In Hand</th>
                    <th>Old Balance</th>
                    <th>To Pay</th>
                </tr>
            </thead>
            <tbody>
            @foreach($salary->ss_desc as $index => $salary_des)
                <tr>
                    <td>{{ $index + 1 }}</td>
                    <td>{{ $salary_des->employee['emp_name']}}</td>
                    <td>{{ $salary_des->days_in_month }}</td>
                    <td>{{ $salary_des->absent }}</td>
                    <td>{{ $salary_des->working }}</td>
                    <td>{{ $salary_des->salary_on }}</td>
                    <td>{{ $salary_des->fixed_basic }}</td>
                    <td>{{ $salary_des->fixed_spl_pf }}</td>
                    <td>{{ $salary_des->salary_paid }}</td>
                    <td>{{ $salary_des->prev_balance }}</td>
                    <td>{{ $salary_des->to_pay }}</td>
                </tr>
                @endforeach
            </tbody>
        </table>
    </div>
</body>
</html>
